C----------------------------------------------------------------------C
C  WinGX     Program Package for Single Crystal Structure Analysis     C
C            System written & assembled by L. J. Farrugia,             C
C            Department of Chemistry, University of Glasgow            C
C            Copyright (C) University of Glasgow, 1995-2001            C
C                                                                      C
C     Program        : IUCRVAL Version 2a                              C
C     Source file    : IUCRVAL.f (UNIX version)                        C
C     Description    : CIF validation according to IUCr algorithms     C
C     Author         : L. J. Farrugia  (September 1998)                C
C     Last modified  : 18-12-98                                        C
C                                                                      C
C     Notes          : The files SGINFO.DAT and CIF_CORE.DIC should    C
c                      either be present in the working directory      C
C                      or in the location set by environment variable  C
C                      "IUCRDIR"                                       C
C                                                                      C
C                      Uses routines GETSPEC (Altermatt and Brown)     C
C                                    CHFORM  (Kilger, Sievers)         C
C                                    TABSPH  (Nardelli)                C
C                                                                      C
C----------------------------------------------------------------------C
C     
C                     HOW TO COMPILE THE PROGRAM
C
C     NOTE: it is essential that the backslash character in strings is
C           NOT treated as a control character by the compiler.
C
C     IMPORTANT: see comments at the end of this file regarding routines
C                ieee_flags (SUN machines) and getenv
C
C     SUN OS4/5      :  f77 -xl iucrval.f -o iucrval
C     SG irix5/6     :  f77 iucrval.f -o iucrval
C     DEC-ALPHA osf1 :  f77 -assume backslash iucrval.f -o iucrval
C     Linux (g77)    :  f77 -fno-backslash iucrval.f -o iucrval
C
C----------------------------------------------------------------------C
      program iucrval
      parameter(nt=54)
      include 'ciftbx.cmn'
      
      common /tests/itest(nt),ntests,ifail(3)
      character ct(nt)*9,cifin*12,buf*80,hms*80
      external  ABSMU_01,ABSTM_01,ABSTM_02,ABSTY_01,ABSTY_02,CELLK_01,
     1 CELLT_01,CELLV_01,CELLV_02,CELLZ_01,CHEMM_01,CHEMM_02,CHEMS_01,
     1 CHEMS_02,CHEMW_01,CHEMW_02,CHEMW_03,CRYSC_01,CRYSR_01,CRYSS_01,
     1 CRYSS_02,DENSD_01,DENSM_01,DENSX_01,DIFMN_01,DIFMN_02,DIFMN_03,
     1 DIFMX_01,DIFMX_02,FCOEF_01,GOODF_01,HYDTR_01,RADNT_01,RADNW_01,
     1 REFLE_01,REFLG_01,REFLL_01,REFLT_01,REFLT_02,REFLT_03,REFNR_01,
     1 RFACG_01,RFACR_01,RINT_01,SHIFSU_01,STRDET_01,STRVAL_01,
     1 STRVAL_02,SYMMG_01,SYMMG_02,SYMMS_01,SYMMS_02,THETM_01,
     1 WEIGHT_01
       data ct/  'ABSMU_01','ABSTM_01','ABSTM_02','ABSTY_01','ABSTY_02',
     1'CELLK_01','CELLT_01','CELLV_01','CELLV_02','CELLZ_01','CHEMM_01',
     1'CHEMM_02','CHEMS_01','CHEMS_02','CHEMW_01','CHEMW_02','CHEMW_03',
     1'CRYSC_01','CRYSR_01','CRYSS_01','CRYSS_02','DENSD_01','DENSM_01',
     1'DENSX_01','DIFMN_01','DIFMN_02','DIFMN_03','DIFMX_01','DIFMX_02',
     1'FCOEF_01','GOODF_01','HYDTR_01','RADNT_01','RADNW_01','REFLE_01',
     1'REFLG_01','REFLL_01','REFLT_01','REFLT_02','REFLT_03','REFNR_01',
     1'RFACG_01','RFACR_01','RINT_01','SHIFSU_01','STRDET_01',
     1'STRVAL_01','STRVAL_02','SYMMG_01','SYMMG_02','SYMMS_01',
     1'SYMMS_02','THETM_01','WEIGHT_01'/

      
      call ciftbx_init
      call iucr_input(cifin)
1     ntests=0
      do i=1,3
        ifail(i)=0
      enddo
      if(.not. data_(' ')) then
         write(*,'(/a)')  ' >> No (more) DATA_ statements found.'
         write(*,'(a)')
     1' >> Detailed listing of WARNINGS & COMMENTS in file IUCRVAL.LST'
         write(14,'(/a)') ' >> No (more) DATA_ statements found.'
         close(13,status='delete')
         stop
      else
        buf=bloc_
        call upcase(buf)
        if(buf .eq. 'GLOBAL') goto 1
        write(*,'(/a,a)') ' >> Reading from DATA_ block ',buf
        write(14,'(/a,a)') ' >> Reading from DATA_ block ',buf
        write(14,'(a)') ' '
        write(14,120)
        write(14,130)
        write(14,121)
      endif
C ---- check whether SGINFO.DAT is present
      hms='P21'
      call getspmat1(hms,ispgr,nlatt)
      if(ispgr .lt. 0) 
     1 call comment('Cannot open SGINFO.DAT. No space group checking !')
      call iucr_test(ABSMU_01)
      call iucr_test(ABSTM_01)
      call iucr_test(ABSTM_02)
      call iucr_test(ABSTY_01)
      call iucr_test(ABSTY_02)
      call iucr_test(CELLK_01)
      call iucr_test(CELLT_01)
      call iucr_test(CELLV_01)
      call iucr_test(CELLV_02)
      call iucr_test(CELLZ_01)
      call iucr_test(CHEMM_01)
      call iucr_test(CHEMM_02)
      call iucr_test(CHEMS_01)
      call iucr_test(CHEMS_02)
      call iucr_test(CHEMW_01)
      call iucr_test(CHEMW_02)
      call iucr_test(CHEMW_03)
      call iucr_test(CRYSC_01)
      call iucr_test(CRYSR_01)
      call iucr_test(CRYSS_01)
      call iucr_test(CRYSS_02)
      call iucr_test(DENSD_01)
      call iucr_test(DENSM_01)
      call iucr_test(DENSX_01)
      call iucr_test(DIFMN_01)
      call iucr_test(DIFMN_02)
      call iucr_test(DIFMN_03)
      call iucr_test(DIFMX_01)
      call iucr_test(DIFMX_02)
      call iucr_test(FCOEF_01)
      call iucr_test(GOODF_01)
      call iucr_test(HYDTR_01)
      call iucr_test(RADNT_01)
      call iucr_test(RADNW_01)
      call iucr_test(REFLE_01)
      call iucr_test(REFLG_01)
      call iucr_test(REFLL_01)
      call iucr_test(REFLT_01)
      call iucr_test(REFLT_02)
      call iucr_test(REFLT_03)
      call iucr_test(REFNR_01)
      call iucr_test(RFACG_01)
      call iucr_test(RFACR_01)
      call iucr_test(RINT_01)
      call iucr_test(SHIFSU_01)
      call iucr_test(STRDET_01)
      call iucr_test(STRVAL_01)
      call iucr_test(STRVAL_02)
      call iucr_test(SYMMG_01)
      call iucr_test(SYMMG_02)
      call iucr_test(SYMMS_01)
      call iucr_test(SYMMS_02)
      call iucr_test(THETM_01)
      call iucr_test(WEIGHT_01)
      write(*,'(a)') ' '
      write(*,120)
      write(14,'(a)') ' '
      write(14,120)
      write(14,140)
      write(14,122)
      if(ntests .eq. 0 )then
        write(*,'(3(a))') ' >> DATA_ block ',buf(1:leng(buf)),
     1 ' has passed all tests !'
        write(14,'(3(a))') ' >> DATA_ block ',buf(1:leng(buf)),
     1  ' has passed all tests !'
      else
        write(*,150)  (ifail(i),i=1,3)
        write(*,'(a)') ' '
        write(*,100) (ct(itest(i)),i=1,ntests)
        write(*,110)
        write(14,150)  (ifail(i),i=1,3)
        write(14,160)
        write(14,161)
        write(14,162)
        write(14,'(a)') ' '
        write(14,100) (ct(itest(i)),i=1,ntests)
        write(14,110)
        write(14,170)
      endif
      write(14,120)
      goto 1

100   format(' This data block has failed the following test(s) :',/
     1 10(/5x,7(a10)))
110   format(/
     1' (note that missing data items may be required for more than one' 
     1,' test)'//
     1' For an exact specification of all tests, see the IUCr web-docum'
     1,'ent :'/,5x,
     1 'http://www.iucr.ac.uk/iucr-top/cif/.checking/validlist.html')
120   format(77('-'))
121   format(25x,22('-'))
122   format(30x,9('-'))
130   format(25x,'COMMENTS and WARNINGS:')
140   format(30x,'SYNOPSIS:')
150   format(' Number of ALERTS issued : level A = ',i2,': level B = ',
     1 i2,': level C = ',i2)
160   format( /
     1' An  ALERT  at level C indicates that an outlier to the defined '
     1,'standards has'/
     1' been detected, and that this may need to be corrected.  When co'
     1,'rrelated with'/
     1' other such ALERTS, it may signal more serious problems.')
161   format(//
     1' An  ALERT  at level B indicates that a violation at a more seri'
     1,'ous level has'/
     1' been detected,  and this must be examined carefully.  Some corr'
     1,'ective action'/
     1' is probably required. Its occurrence usually indicates, individ'
     1,'ually or with'/
     1' other ALERTS,the need for significant revision. If coupled with'
     1,' other ALERTS'/
     1' at this level it may represent grounds for rejection from Acta '
     1,'Cryst C.')
162   format(//
     1' An  ALERT  at level A indicates either that a mandatory data it'
     1,'em is missing'/
     1' (or has an inappropriate value),  or that a major standards fai'
     1,'lure has been'/
     1' detected.  In the latter case,  it represents probable grounds '
     1,'for rejection'/
     1' from Acta Cryst C, or for major revision of the structural anal'
     1,'ysis,  unless'/
     1' there are mitigating circumstances.')
170   format(/32x,6('-')/
     1' IUCR_VALIDATE carries out all the DVN checks given in validlist'
     1,'.html, except'/
     1' for test CHEMM_02 (which verifies that chemical_formula_moiety '  
     1,'is consistent'/
     1' with the atomic connectivities from the _atom_site data)')
      end
C
C----------------------------------------------------------------------C
C
      subroutine iucr_input(cifin)
      include 'ciftbx.cmn'
      logical lexist
      character*12 cifin,dictionary,filename,option
      character*80 filepath,dicpath

      dictionary='CIF_CORE.DIC'
      dicpath=filepath(dictionary)
      option=' '
      call ciftbx_init
      open(14,file='IUCRVAL.LST',status='unknown')
      write(*,'(a)') ' '
      write(*,100)
      write(14,100)
      write(*,110)
     1 'IUCR_VALIDATE - Version 2a. A program to validate CIF files'
      write(*,130)'according to Acta Cryst C (IUCr) algorithms'
      write(*,140) 'see http://www.iucr.ac.uk/iucr-top/cif/.checking/va'
     1,'lidlist.html'
      write(*,120)
     1 'Author: L. J. Farrugia, (C) University of Glasgow, 1998'
      write(*,100)
      write(14,110)
     1 'IUCR_VALIDATE - Version 2a. A program to validate CIF files'
      write(14,130)'according to Acta Cryst C (IUCr) algorithms'
      write(14,140)'see http://www.iucr.ac.uk/iucr-top/cif/.checking/va'
     1,'lidlist.html'
      write(14,150)   'Using ',tbxver_
      write(14,100)
C
      iarg=iargc()
      if( iarg .eq. 0) then
        write(*,'(/a)')'   Command line syntax is : iucrval -v cif_file'
        write(*,'(/a)')'  -v is an optional flag for data validation wi'
     1//'th a CIF dictionary'
        write(*,'(a)') '  cif_file is name of CIF (extension .cif assum'
     1//'ed)'
        stop
      elseif(iarg .eq. 1) then
        call getarg(1,cifin)
      else
        call getarg(1,option)
        call locase(option)
        call getarg(2,cifin)
      endif
      ind=index(cifin,'.')
      if(ind .eq. 0) then
        filename=cifin(1:leng(cifin))//'.cif'
      else
        filename=cifin
      endif
      inquire(file=filename,exist=lexist)
      if(.not. lexist) 
     1 call errstop('File '//filename//' does not exist !')
      if(.not. init_(11,12,13,14)) stop
      if(index(option,'-v') .eq. 0) goto 10
      inquire(file=dicpath,exist=lexist)
      if(lexist) then
        write(*,'(/a,a,a)')' >> Loading CIF dictionary ',dictionary,
     1                  ' for data validation. Please wait ....'
      else
        write(*,'(/a,a,a)')
     1 ' >> Dictionary ',dictionary,' not found - No data validation'
      endif
      if(.not. dict_(dicpath,'valid')) then
         write(*,'(/a,a,a)')
     1 ' >> Dictionary ',dictionary,' not valid - No data validation'
      endif
10    if(.not. ocif_(filename)) then
        write(*,'(a,a)') ' >> Failure opening CIF file: ',filename
        close(14,status='delete')
        stop
      else
          write(14,'(/a,a)') ' >> Validating CIF file : ',filename
      endif
100   format(3x,72('*'))
110   format(3x,'*',6x,a,5x,'*')
120   format(3x,'*',7x,a,8x,'*')
130   format(3x,'*',13x,a,14x,'*')
140   format(3x,'*',3x,a,a,4x,'*')
150   format(3x,'*',16x,a,a,16x,'*')
      end
C
C----------------------------------------------------------------------C
C
      logical function ABSMU_01()
C ---- test 1
      parameter (itypes=30,nel=92)
      real cuamu(nel),moamu(nel),agamu(nel),fnamtp(itypes)
      character namtyp(itypes)*8
      logical copper,silver,moly,chknum,chkchar
      character*80 item,chemfm
      data cuamu/.0655, .194, .576, 1.66, 4.15, 8.99, 17.3, 30.4,
     1 49.8, 76.8, 114., 161., 222., 297., 388., 497., 624., 772.,
     1 940., 1130., 1350., 1590., 1850., 2130., 2460., 2800., 3140.,
     1 476., 547., 629.,719., 819., 929., 1050., 1180., 1320., 1480.,
     1 1650., 1830., 2030., 2230., 2460., 2700., 2950., 3230., 3520.,
     1 3820., 4150., 4500., 4860., 5250., 5650., 6070., 6520., 7000.,
     1 7500., 8030., 8570., 9120., 9680., 10200., 10800., 11000.,
     1 10500., 8470., 9770., 3470., 3670., 3930., 4100., 4500., 4600.,
     1 4850., 5130., 5720., 5800., 6240., 6340., 6690., 6680., 7540.,
     1 7980., 8430., 8810., 8650., 9720., 10200., 10200., 14300.,
     1 11800., 10600., 11200./
      data moamu/.0624, .134, .228, .383, .661, 1.15, 1.96, 3.25,
     1 5.15, 7.86, 11.6, 16.5, 22.9, 31., 41., 53.2, 67.8, 85.1, 105.,
     1 129., 156., 186., 220., 258., 302., 349., 401., 457., 518.,
     1 586., 660., 738., 822., 911., 1000., 1100., 1210., 1320., 1430.,
     1 247., 273., 300., 332., 364., 399., 436., 476., 518., 563., 611., 
     1 662., 716., 773., 834., 898., 965., 1040., 1110., 1190., 1270., 
     1 1350., 1440., 1540., 1630., 1740., 1840., 1950., 2070., 2190.,
     1 2310., 2440., 2580., 2720., 2860., 3010., 3160., 3310., 3480.,
     1 3650., 3820., 4010., 4190., 4380., 4580., 4070., 3980., 3220.,
     1 3300., 5400., 3700.,3870., 4030./
      data agamu /.0614, .128, .206, .313, .479, .745, 1.17, 1.82, 2.77,
     1 4.12, 5.96, 8.42, 11.6, 15.6, 20.6, 26.7, 34.1, 42.9, 53.2, 65.2, 
     1 78.9, 94.7, 112., 133., 155., 180., 207., 238., 271., 307., 346.,
     1 387., 433., 482., 535., 592., 652., 715., 780., 847., 922.,1150.,
     1 1070., 192., 210., 230., 251., 273., 297., 323., 350., 378.,409.,
     1 441., 475., 511., 549., 588., 630., 674., 720., 768., 819., 872.,
     1 927., 985., 1040., 1110., 1170., 1240., 1310., 1390., 1460.,
     1 1540., 1620., 1710., 1800., 1890., 1990., 2090., 2190., 2290.,
     1 2400., 2510., 2620., 2730., 2850., 2980., 3110., 3230., 3420.,
     1 3500./

      ABSMU_01=.false.
      totamu=0.0
      copper=.false.
      silver=.false.
      moly=.false.

      item='_diffrn_radiation_wavelength'
      if(.not. chknum(item,valwl,dum,'A')) return
      item='_exptl_absorpt_coefficient_mu'
      if(.not. chknum(item,valmu,dum,'A')) return
      item='_cell_volume'
      if(.not. chknum(item,valvol,dum,'A')) return
      item='_cell_formula_units_Z'
      if(.not. chknum(item,valz,dum,'A'))   return
      item='_chemical_formula_sum'
      if(.not. chkchar(item,chemfm,'A')) return

      if(valwl .gt. 0.68 .and. valwl .lt. 0.73) moly=.true.
      if(valwl .gt. 1.50 .and. valwl .lt. 1.60) copper=.true.
      if(valwl .gt. 0.50 .and. valwl .lt. 0.60) silver=.true.
      call chform(chemfm,namtyp,fnamtp,natmtp)
      if(natmtp .le. 0) then
        call alert('A','Cannot interpret chemical formula '//chemfm)
        call comment1('Cannot interpret chemical formula '//chemfm)
        return
      endif
      do i=1,natmtp
        fnamtp(i)=fnamtp(i)*valz
        call ELINFO(namtyp(i),ielem,atwt)
        if(ielem .eq. 0) then
          call alert('A','Unrecognised element '//namtyp(i))
          return
        endif
        if(moly) then
           totamu=totamu + moamu(ielem)*fnamtp(i)
        elseif (copper) then
           totamu=totamu + cuamu(ielem)*fnamtp(i)
        elseif (silver) then
           totamu=totamu + agamu(ielem)*fnamtp(i)
        else
          call alert('A','Unrecognised wavelength for calcn of mu')
          return
        endif
      enddo
      calcmu=totamu/valvol
      if(calcmu .le. 0.0) call errstop('CALCMU has zero value !')
      rmu=valmu/calcmu
      ABSMU_01=.true.
      if(rmu .le. 0.90 .or. rmu .ge. 1.10) then
        call alert('A','Experimental and calculated mu differ by > 10%')
        ABSMU_01=.false.
      elseif(rmu .le. 0.95 .or. rmu .ge. 1.05) then
        call alert('B','Experimental and calculated mu differ by > 5%')
        ABSMU_01=.false.
      elseif(rmu .le. 0.99 .or. rmu .ge. 1.01) then
        call alert('C','Experimental and calculated mu differ by > 1%')
        ABSMU_01=.false.
      endif
      if(.not. ABSMU_01)  then
        write(item,100) valmu,calcmu
        call comment(item)
      endif
100   format('EXPTL absorption-coeff MU = ',f7.3,': CALC = ', f7.3,
     1' mm-1')
      end
C
C----------------------------------------------------------------------C
C
      logical function ABSTM_01()
C ---- test 2
      character*80 item,value
      logical chkchar,chknum
      ABSTM_01=.false.
      valmin=1.0
      valmax=1.0
      value=' '
      item='_exptl_absorpt_correction_type'
      if(.not. chkchar(item,value,'A')) return
      ABSTM_01=.true.
      if(index(value,'none') .ne. 0) return
      item='_exptl_absorpt_correction_T_min'
      if(.not. chknum(item,valmin,dum,'A')) ABSTM_01=.false.
      item='_exptl_absorpt_correction_T_max'
      if(.not. chknum(item,valmax,dum,'A')) ABSTM_01=.false. 
      if(.not. ABSTM_01) return
      if(valmin .gt. valmax) then
        call alert('A','Value of T_min is greater than T_max')
        ABSTM_01=.false.
      endif
      end
C
C----------------------------------------------------------------------C
C
      logical function ABSTM_02()
C ---- test 3
      character*80 item,value,message
      character level
      logical chknum,chkchar,none
      real mu,tmax,tmid,tmin,tmaxrep,tminrep
      tol=1.0e-6
      tmaxrep=1.0
      tminrep=1.0
      ABSTM_02=.false.
      item='_exptl_absorpt_coefficient_mu'
      if(.not. chknum(item,mu,dum,' ')) return
      item='_exptl_absorpt_correction_type'
      if(.not. chkchar(item,value,' ')) return
      none=(index(value,'none') .ne. 0) 
      ABSTM_02=.true.
      item='_exptl_absorpt_correction_T_max'
      level='A'
      if(none) level=' '
      if(.not. chknum(item,tmaxrep,dum,level)) then
        if(.not. none) ABSTM_02=.false.
      endif
      if(tmaxrep .le. 0.0) tmaxrep=1.0
      item='_exptl_absorpt_correction_T_min'
      if(.not. chknum(item,tminrep,dum,level)) then
        if(.not. none) ABSTM_02=.false.
      endif
      if(tminrep .le. 0.0) tminrep=1.0
      item='_exptl_crystal_size_max'
      if(.not. chknum(item,tmax,dum,'A')) ABSTM_02=.false.
      item='_exptl_crystal_size_mid'
      if(.not. chknum(item,tmid,dum,'A')) ABSTM_02=.false.
      item='_exptl_crystal_size_min'
      if(.not. chknum(item,tmin,dum,'A')) ABSTM_02=.false.
      if(.not. ABSTM_02) return
      if( (index(value,'psi-scan')  .ne.0) .or.
     1    (index(value,'empirical') .ne.0) .or.
     1    (index(value,'multi-scan').ne.0) .or.
     1    (index(value,'refdelf')   .ne.0)) then
        if(mu*tmid .gt. 3.0) then
          call error('Crystal and compound unsuitable for non-numerical
     1corrections')
          return
        endif
      endif
      if( (index(value,'psi-scan')   .ne.0) .or.
     1    (index(value,'empirical')  .ne.0) .or.
     1    (index(value,'multi-scan') .ne.0) .or.
     1    (index(value,'refdelf')    .ne.0) .or.
     1    (index(value,'none')       .ne.0)) then

        item='_diffrn_reflns_theta_max'
        if(.not. chknum(item,thmax,dum,'A')) return

        temx=max(tmid,min(1.2*tmid,tmax))
        tmaxshape=exp(-tmin*mu)
        tminshape=exp(-temx*mu)
        tminshape1=exp(-tmax*mu)
        tmaxangle=1.0

C ---- get minimum spherical crystal correction factor 
        call TABSPH(thmax,mu*tmid,tminangle,tfact)

        tmaxexp=tmaxshape*tmaxangle
        tminexp=tminshape*tminangle
        tminexp1=tminshape1*tminangle
        if(abs(tminrep) .le. tol)  call errstop('TMINREP is zero !')
        rtrep=tmaxrep/tminrep
        if(abs(tminexp) .le. tol)  call errstop('TMINEXP is zero !')
        rtexp=tmaxexp/tminexp
        if(abs(tminexp1) .le. tol) call errstop('TMINEXP1 is zero !')
        rtexp1=tmaxexp/tminexp1
        if(abs(rtrep) .le. tol)    call errstop('RTREP is zero !')
        rr=rtexp/rtrep
        rr1=rtexp1/rtrep
      endif
      ABSTM_02=.true.
      if( (index(value,'psi-scan')  .ne.0) .or.
     1    (index(value,'empirical') .ne.0) .or.
     1    (index(value,'multi-scan').ne.0) .or.
     1    (index(value,'refdelf')   .ne.0)) then

        if(rr1 .lt. 0.5 .or. rr .gt. 2.0) then
          call alert('A',
     1'Discrepancy between calc. and exptl ratio Tmax/Tmin is > 50%')
          ABSTM_02=.false.
        elseif(rr1 .lt. 0.75 .or. rr .gt. 1.5) then
          call alert('B',
     1'Discrepancy between calc. and exptl ratio Tmax/Tmin is > 25%')
          ABSTM_02=.false.
        elseif(rr1 .lt. 0.9 .or. rr .gt. 1.1) then
          call alert('C',
     1'Discrepancy between calc. and exptl ratio Tmax/Tmin is > 10%')
          ABSTM_02=.false.
        endif
      elseif(index(value,'none') .ne. 0) then
        if(rtexp .gt. 1.3) then
          ABSTM_02=.false.
          call alert('A',
     1'Calculated Tmax/Tmin > 1.3 - absorption correction essential')
        elseif(rtexp .gt. 1.2) then
          ABSTM_02=.false.
          call alert('B',
     1'Calculated Tmax/Tmin > 1.2 - absorption correction needed')
        elseif(rtexp .gt. 1.1) then
          ABSTM_02=.false.
          call alert('C',
     1'Calculated Tmax/Tmin > 1.1 - absorption correction advisable')
        endif
      endif
C --- now scale transmission coefficients
      if( (index(value,'psi-scan')  .ne.0) .or.
     1    (index(value,'empirical') .ne.0) .or.
     1    (index(value,'multi-scan').ne.0) .or.
     1    (index(value,'refdelf')   .ne.0)) then
        if(tmaxexp/tmaxrep .lt. 0.95 .or. tmaxexp/tmaxrep .gt. 1.05)
     1  then
          tmaxscl=tmaxexp
          tminscl=tmaxexp/rtrep
          write(message,100) tmaxscl,tminscl
          call comment(message)
          write(message,110) tmaxexp,tminexp
          call comment(message)
        endif
      endif
C ?      ratio=tmaxrep**2/tminrep**2
      ratio=tmaxrep/tminrep
      diff_path=(log(ratio))/mu
      if(ratio .gt. 1.0) then
        write(message,120) diff_path
        call comment(message)
      endif
      if(.not. ABSTM_02) return
      ABSTM_02=.true.
100   format('Absolute values of T(max) = ',f6.4,' : T(min) = ',f6.4)
110   format('Expected values of T(max) = ',f6.4,' : T(min) = ',f6.4)
120   format('Maximum path difference from reported T values = ',f6.3,
     1 ' mm')
      end
C
C----------------------------------------------------------------------C
C
      logical function ABSTY_01()
C ---- test 4  - to check whether _exptl_absorpt_correction_type has a
C                recognised value
      parameter (nvalues=11)
      logical chkchar
      character*80 item,value,allowed_values(nvalues)
      data allowed_values/'none','analytical','integration','numerical',
     1 'gaussian','empirical','psi-scan','multi-scan','refdelf',
     1 'cylinder','sphere'/
      value=' '
      ABSTY_01=.false.
      item='_exptl_absorpt_correction_type'
      if(.not. chkchar(item,value,' ')) return
      do i=1,nvalues
        ilen=leng(allowed_values(i))
        if(index(value,allowed_values(i)(1:ilen)).ne.0) ABSTY_01=.true.
      enddo
      if(.not. ABSTY_01) then
         call alert_char('A',item,value)
         call notice
     1('Allowed values are : none,analytical,integration,numerical,gauss
     1ian')
         call notice
     1('empirical,psi-scan,multi-scan,refdelf,cylinder or sphere')
      endif
      end
C
C----------------------------------------------------------------------C
C
      logical function ABSTY_02()
C ---- test 5 - to check whether _exptl_absorpt_process_details is given
C               if _exptl_absorpt_correction_type .ne. 'none'
      character*80 item,value
      logical chkchar
      ABSTY_02=.false.
      item='_exptl_absorpt_correction_type'
      if(.not. chkchar(item,value,' ')) return
      if(value .ne. 'none' ) then
        item='_exptl_absorpt_process_details'
        if(.not. chkchar(item,value,'A')) return
      endif
      ABSTY_02=.true.
      end
C
C----------------------------------------------------------------------C
C
      logical function CELLK_01()
C ---- test 6 - to check whether  _cell_measurement_temperature is given
C               in deg Kelvin
      character*80 item
      logical chknum
      CELLK_01=.false.
      item='_cell_measurement_temperature'
      if(.not. chknum(item,temp,dum,'A')) return
      if(temp .le. 25.0) then
        call alert('C','Check that _cell_measurement_temperature given i
     1n degrees K')
        return
      endif
      CELLK_01=.true.
      end
C
C----------------------------------------------------------------------C
C
      logical function CELLT_01()
C ---- test 7 -to check that _cell_measurement_theta_min  <
C              _cell_measurement_theta_max      
      character*80 item
      logical chknum
      CELLT_01=.true.
      item='_cell_measurement_theta_min'
      if(.not. chknum(item,thmin,dum,'A'))  CELLT_01=.false.
      item='_cell_measurement_theta_max'
      if(.not. chknum(item,thmax,dum,'A'))  CELLT_01=.false. 
      if(.not. CELLT_01) return
      if(thmax .lt. thmin) then
        call alert('A','Theta min > theta max for cell measurement')
        CELLT_01=.false.
      endif
      end
C
C----------------------------------------------------------------------C
C
      logical function CELLV_01()
C ---- test 8  - to check whether cell volume agrees with cell dimensions
      character*80 item
      logical chknum
      CELLV_01=.true.
      item='_cell_angle_alpha'
      if(.not. chknum(item,alpha,dum,'A')) CELLV_01=.false.
      item='_cell_angle_beta'
      if(.not. chknum(item,beta,dum,'A'))  CELLV_01=.false.
      item='_cell_angle_gamma'
      if(.not. chknum(item,gamma,dum,'A')) CELLV_01=.false.
      item='_cell_length_a'
      if(.not. chknum(item,a,dum,'A'))     CELLV_01=.false.
      item='_cell_length_b'              
      if(.not. chknum(item,b,dum,'A'))     CELLV_01=.false.
      item='_cell_length_c'
      if(.not. chknum(item,c,dum,'A'))     CELLV_01=.false.
      item='_cell_volume'
      if(.not. chknum(item,vol,dum,' '))   CELLV_01=.false.
      if(.not. CELLV_01) return
      pi=4.0*atan(1.0)
      alpha=alpha*pi/180.0
      beta=beta*pi/180.0
      gamma=gamma*pi/180.0
      s=(alpha + beta + gamma)/2.0
      cvol= a * B * C * 2 * sqrt(sin(s)*sin(s-alpha)*sin(s-beta)
     1     *sin(s-gamma))
      if(cvol .le. 0.0) 
     1 call errstop('Cell dimensions give zero or negative volume !')
      rv=vol/cvol
      if(rv .lt. 0.999 .or. rv .gt. 1.001) then
        CELLV_01=.false.
        call alert('A','Cell volume disagrees with cell dimensions')
      endif
      end
C
C----------------------------------------------------------------------C
C
      logical function CELLV_02()
C ---- test 9 - to check whether cell volume su's agree with cell su's
      character*80 item,value
      logical chknum,chkchar
      tol=1.0e-6
      CELLV_02=.false.
      item='_symmetry_cell_setting'
      if(.not. chkchar(item,value,'A')) return
      if(index(value,'triclinic') .ne. 0 .or. index(value,'monoclinic')
     1 .ne. 0  .or. index(value,'orthorhombic') .ne. 0) then
        c1=1
        c2=1
        c3=1
        c4=1
      elseif(index(value,'tetragonal') .ne. 0 .or.
     1 index(value,'trigonal') .ne. 0  .or. index(value,'hexagonal')
     1 .ne. 0) then
        c1=2
        c2=2
        c3=1
        c4=1
      elseif(index(value,'rhombohedral') .ne. 0) then
        c1=3
        c2=3
        c3=3
        c4=4
      elseif(index(value,'cubic') .ne. 0) then
        c1=3
        c2=3
        c3=3
        c4=1
      else
        return
      endif
      pi=4.0*atan(1.0)
      item='_cell_angle_alpha'
      if(.not. chknum(item,alpha,alpha_su,' ')) return
      alpha=alpha*pi/180.0
      alpha_su=alpha_su*pi/180.0
      item='_cell_angle_beta'
      if(.not. chknum(item,beta,beta_su,' ')) return
      beta=beta*pi/180.0
      beta_su=beta_su*pi/180.0
      item='_cell_angle_gamma'
      if(.not. chknum(item,gamma,gamma_su,' ')) return
      gamma=gamma*pi/180.0
      gamma_su=gamma_su*pi/180.0
      item='_cell_length_a'
      if(.not. chknum(item,a,a_su,' ')) return
      item='_cell_length_b'
      if(.not. chknum(item,b,b_su,' ')) return
      item='_cell_length_c'
      if(.not. chknum(item,c,c_su,' ')) return
      item='_cell_volume'
      if(.not. chknum(item,vol,vol_su,' ')) return
      if(vol .le. 0.0) return
      if(alpha .le. 0.0 .or. beta .le. 0.0 .or. gamma .le. 0.0) return
      if(a .le. 0.0 .or. b .le. 0.0 .or. c .le. 0.0) return
      csa=cos(alpha)
      csb=cos(beta)
      csg=cos(gamma)

      rvl=(a * b * c / vol)
      suvol = vol*sqrt(
     1 c1*(a_su/a)**2 + c2*(b_su/b)**2 + c3*(c_su/c)**2 +
     1 c4*(rvl**4)* ((alpha_su)**2*(csa-csb*csg)**2 +
     1               (beta_su )**2*(csb-csa*csg)**2 +
     1               (gamma_su)**2*(csg-csb*csa)**2))

      dsuv=abs(vol_su - suvol)
      if( dsuv .gt. 8.0) then
        call alert('A','Cell volume su disagrees with cell su''s')
        goto 1
      elseif( dsuv .gt. 4.0) then
        call alert('B','Cell volume su disagrees with cell su''s')
        goto 1
      elseif( dsuv .gt. 2.0) then
        call alert('C','Cell volume su disagrees with cell su''s')
        goto 1
      else
        CELLV_02=.true.
      endif
      return
1     if(abs(vol_su) .le. tol .or. suvol .le. tol) return 
      write(value,100)  vol_su, suvol
      call comment(value)
100   format('EXPTL cell volume SU = ',f8.2,': CALC SU = ',f8.2)
      end
C
C----------------------------------------------------------------------C
C
      logical function CELLZ_01()
C ---- test 10 - checks whether Z matches _chemical_formula_sum and
C                atomic contents from atom list
      parameter (itypes=30)
      real fnamtp(itypes)
      character namtyp(itypes)*8
      character*80 item,chemfm,value
      logical centro,chknum,chkchar

      CELLZ_01=.false.
      item='_cell_formula_units_Z'
      if(.not. chknum(item,valZ,dum,' ')) return

      item='_symmetry_equiv_pos_as_xyz'
      if(.not. chkchar(item,value,'A')) return
C ----- calculate formula weight from '_chemical_formula_sum
      item='_chemical_formula_sum'
      if(.not. chkchar(item,chemfm,' ')) return
      call chform(chemfm,namtyp,fnamtp,natmtp)
      if(natmtp .le. 0) return
      cfswt=0.0
      do i=1,natmtp
         call ELINFO(namtyp(i),iel,atwt)
         if(iel .ne. 0) then
           cfswt=cfswt + atwt*fnamtp(i)
         endif
      enddo
      call SYMMOPS(centro,isymm)
      if(isymm .eq. 0) return
      xx=SUMWT()/valZ
      if(xx .le. 0.0) return
      wt=xx*real(isymm)
      rc=cfswt/wt
      dc=abs(cfswt - wt)
      if(rc .lt. 0.9 .or. rc .gt. 1.1 ) then
       call alert('A','Calulated and given values of Z differ by > 10%')
       goto 3
      elseif(rc .lt. 0.95 .or. rc .gt. 1.05) then
       call alert('B','Calulated and given values of Z differ by > 5%')
       goto 3
      elseif(rc .lt. 0.99 .or. rc .gt. 1.01) then
       call alert('C','Calulated and given values of Z differ by > 1%')
       goto 3
      endif
      CELLZ_01=.true.
      return
3     write(value,100) cfswt,wt
100   format('FORMULA-WT (Z) = ',f9.2,':   FORMULA-WT (contents) = '
     1,f9.2)
      call notice(value)
      if(wt .lt. cfswt)
     1call notice('Are all the hydrogen atoms included in the model ?')
      end
C
C----------------------------------------------------------------------C
C
      logical function CHEMM_01()
      parameter(itypes=30)
C ---- test 11 - check moiety formula follows CIF dictionary definition
C                and is consistent with formula sum
      common/iucr_m/ierr
      character*80 string,item,value,chemfm
      character*80 moieties(20),buf,bufout
      character*8 namtyp(itypes),namtyp1(itypes)
      real fnamtp(itypes),fnamtp1(itypes)
      integer nmoi
      logical chkchar
      CHEMM_01 = .false.
      ierr=0
      item='_chemical_formula_moiety'
      if(.not. chkchar(item,value,'A')) return
      buf=value
      item='_chemical_formula_sum'
      if(.not. chkchar(item,value,' ')) return
      string=value
      call parsfm(buf,bufout,nmoi,moieties,ierr)
      if(ierr .eq. 1) return
      do i=1,nmoi
        chemfm=moieties(i)
        call chform(chemfm,namtyp,fnamtp,natmtp)
        if(natmtp .le. 0) goto 10
      enddo
      call chform(bufout,namtyp,fnamtp,natmtp)
      if(natmtp .le. 0) goto 10
      call chform(string,namtyp1,fnamtp1,natmtp1)
      if(natmtp1 .le. 0) goto 10
      if(natmtp .ne. natmtp1) goto 20
      if(natmtp .eq. 0) return
      do i=1,natmtp
         if(namtyp(i) .ne. namtyp1(i)) goto 20
         if(abs(fnamtp(i) - fnamtp1(i)) .gt. 1.0e-6) goto 20
      enddo
      CHEMM_01 = .true.
      return
10    continue
      call alert('C','Uninterpretable symbol in moiety : '//
     1moieties(i))
      return
20    continue
      call alert('C','Inconsistent chemical_formula_moiety and chemical_
     1formula_sum')
      end
C
C----------------------------------------------------------------------C
C
      logical function CHEMM_02()
C ----- test 12 - not implemented
      CHEMM_02=.true.
      end
C
C----------------------------------------------------------------------C
C
      logical function CHEMS_01()
C ---- test 13 - checks whether _chemical_formula_sum is properly
C                constructed according to Hill convention
      parameter (itypes=30)
      character line(148)
      integer fch(20),fwd(20)
      character namtyp(itypes)*2,namtypo(itypes)*2,dum1(itypes)*2,
     1          dum2(itypes)*2
      character*80 item,chemfm,value
      character*11 numbers
      logical chkchar
      numbers='0123456789.'
      CHEMS_01=.false.
      item='_chemical_formula_sum'
      if(.not. chkchar(item,chemfm,' ')) return
      value=chemfm
      do i=1,itypes
        namtyp(i)=' '
        namtypo(i)=' '
        dum1(i)=' '
        dum2(i)=' '
      enddo

C --- strip numbers from formula string
      do i=1,80
         do j=1,11
            if(value(i:i) .eq. numbers(j:j)) value(i:i)=' '
         enddo
      enddo
      do i=1,148
        line(i)=' '
        if(i .gt. 1 .and. i .le. 81) line(i)=value(i-1:i-1)
      enddo
      call parse(line,nfields,fch,fwd)
      do i=1,nfields
        call aformt(line,fch(i),fwd(i),namtyp(i))
        call locase(namtyp(i))
        call upcase(namtyp(i)(1:1))
      enddo
      do i=1,itypes
        namtypo(i)=namtyp(i)
      enddo
      ind = 0
      do i=1,nfields
         if(namtyp(i) .eq. 'C') ind = i
      enddo
      ind1 = 0
      do i=1,nfields
         if(namtyp(i) .eq. 'H') ind1 = i
      enddo
C ---- C is present but not first in list
      if(ind .gt. 1) goto 1

      if(ind .eq. 0) then
C ---- C not present - all elements in alphabetical order
        call sortchar(namtyp,nfields)
        icount=0
        do i=1,nfields
          if(namtyp(i) .ne. namtypo(i)) icount=icount+1
        enddo
        if(icount .gt. 0) goto 1
      elseif(ind .eq. 1) then

C ---- H also present but not second in list
         if(ind1 .ne. 0 .and. ind1 .gt. 2) goto 1

         if(ind1 .eq. 0) n = 2
         if(ind1 .eq. 2) n = 3
         if(n .ge. nfields) goto 2
         do i=n,nfields
            dum1(i-n+1)=namtypo(i)
            dum2(i-n+1)=namtypo(i)
         enddo
         call sortchar(dum2,nfields-n+1)
         icount=0
         do i=1,nfields-n+1
           if(dum1(i) .ne. dum2(i)) icount=icount+1
         enddo
         if(icount .gt. 0) goto 1
      endif
2     CHEMS_01=.true.
      return
1     call alert('C','Element ordering in _chemical_formula_sum is incor
     1rect')
      write(value,'(20a3)') (namtypo(i),i=1,nfields)
      call notice(value)
      end
C
C----------------------------------------------------------------------C
C
      logical function CHEMS_02()
C ---- test 14 - tests whether _publ_requested_category is appropriate
      parameter (itypes=30)
      real fnamtp(itypes)
      logical chkchar,contains_C,contains_H,contains_METAL
      character namtyp(itypes)*8
      character*80 item,chemfm,value
      CHEMS_02=.false.
      item='_chemical_formula_sum'
      if(.not. chkchar(item,chemfm,' ')) return
      item='_publ_requested_category'
      if(.not. chkchar(item,value,'A')) return
      call upcase(value)
      call chform(chemfm,namtyp,fnamtp,natmtp)
      if(natmtp .le. 0) then
        call comment('Uninterpretable chemical_formula_sum')
        return
      endif
      contains_C=.false.
      contains_H=.false.
      contains_metal=.false.
      do i= 1,natmtp
         call ELINFO(namtyp(i),iel,atwt)
         if(iel .eq. 6                   ) contains_C    =.true.
         if(iel .eq. 1                   ) contains_H    =.true.
         if(iel .ge. 3  .and. iel .le.  4) contains_METAL=.true.
         if(iel .ge. 11 .and. iel .le. 13) contains_METAL=.true.
         if(iel .ge. 19 .and. iel .le. 32) contains_METAL=.true.
         if(iel .ge. 37 .and. iel .le. 51) contains_METAL=.true.
         if(iel .ge. 55 .and. iel .le. 84) contains_METAL=.true.
         if(iel .ge. 87                  ) contains_METAL=.true.
      enddo
      if(.not. contains_C) then
        if(index(value,'FI') .eq. 0 .and. index(value,'CI') .eq. 0) then
          call alert('A','publ_requested_category is wrong')
          return
        endif
      elseif(contains_C .and. .not. contains_H) then
        if(index(value,'FI') .eq. 0 .and. index(value,'CI') .eq. 0) then
          call alert('C','publ_requested_category is probably wrong')
          return
        endif
      elseif(contains_C .and. contains_H .and. contains_METAL) then
        if(index(value,'FM') .eq. 0 .and. index(value,'CM') .eq. 0) then
          call alert('A','publ_requested_category is wrong')
          return
        endif
      elseif(contains_C .and. contains_H .and. .not. contains_METAL)then
        if(index(value,'FO') .eq. 0 .and. index(value,'CO') .eq. 0) then
          call alert('A','publ_requested_category is wrong')
          return
        endif
      endif
      CHEMS_02=.true.
      end
C
C----------------------------------------------------------------------C
C
      logical function CHEMW_01()
C ---- test 15 - compares value of _chemical_formula_weight with that
C                calculated from the _chemical_formula_sum
      parameter (itypes=30)
      real fnamtp(itypes)
      logical chkchar,chknum
      character namtyp(itypes)*8
      character*80 item,chemfm,value
      CHEMW_01=.false.
      item='_chemical_formula_sum'
      if(.not. chkchar(item,chemfm,' ')) return
      item='_chemical_formula_weight'
      if(.not. chknum(item,chemfwt,dum,'A')) return
      call chform(chemfm,namtyp,fnamtp,natmtp)
      if(natmtp .le. 0) return
      sumwt=0.0
      write(14,150)'Symbol ','Z','Atwt','No. in CFU'
160   format(5x,a2,5x,i3,6x,f7.3,6x,f8.2)
150   format(/' Elemental data according to _chemical_formula_sum:'
     1//2x,a7,5x,a1,8x,a5,6x,a10)
      do i=1,natmtp
         call ELINFO(namtyp(i),iel,atwt)
         write(14,160) namtyp(i),iel,atwt,fnamtp(i)
         if(iel .ne. 0) then
           sumwt=sumwt + atwt*fnamtp(i)
         endif
      enddo
      rw=chemfwt/sumwt
      dw=abs(chemfwt - sumwt)
      write(*,110) (namtyp(i),i=1,natmtp)
      if(rw .lt. 0.9 .or. rw .gt. 1.1 ) then
        call alert
     1  ('A','Given and calculated formula weights differ by > 10%')
        goto 1
      elseif(rw .lt. 0.95 .or. rw .gt. 1.05) then
        call alert
     1  ('B','Given and calculated formula weights differ by > 5%')
        goto 1
      elseif(rw .lt. 0.99 .or. rw .gt. 1.01) then
        call alert
     1  ('C','Given and calculated formula weights differ by > 1%')
        goto 1
      endif
      CHEMW_01=.true.
      item='_publ_requested_category'
      if(.not. chkchar(item,value,' ')) return
      if(dw .lt. 1.0) return
      call upcase(value)
      if(index(value,'FO') .eq. 0 .and. index(value,'FM') .eq. 0 .and.
     1 index(value,'CO') .eq. 0 .and. index(value,'CM') .eq. 0) return
      CHEMW_01=.false.
      call alert('A','Check formula weight & chemical_formula_sum')
1     write(value,100) chemfwt,sumwt
100   format('FORMULA-WT (given) = ',f9.2,':  FORMULA-WT (sum) = ',f9.2)
110   format(/' COMMENT - Elements present : ',10(a2,1x))
      call notice(value)
      end
C
C----------------------------------------------------------------------C
C
      logical function CHEMW_02()
C ---- test 16 - compares value of _chemical_formula_weight with that
C                calculated from the _chemical_formula_moiety
      parameter (itypes=30)
      common/iucr_m/ierr
      real fnamtp(itypes)
      character namtyp(itypes)*8
      character*80 item,chemfm,value,buf,moieties(20)
      logical chkchar,chknum
      CHEMW_02=.false.
      if(ierr .eq. 1) return
      item='_chemical_formula_moiety'
      if(.not. chkchar(item,chemfm,' ')) return
      item='_chemical_formula_weight'
      if(.not. chknum(item,chemfwt,dum,' ')) return
      call parsfm(chemfm,buf,nmoi,moieties,ierr)
      call chform(buf,namtyp,fnamtp,natmtp)
      if(natmtp .le. 0) then
         call alert('C','Uninterpretable formula_moiety : '//chemfm)
         return
      endif
      sumwt=0.0
      do i=1,natmtp
         call ELINFO(namtyp(i),iel,atwt)
         if(iel .ne. 0) then
           sumwt=sumwt + atwt*fnamtp(i)
         endif
      enddo
      rm=chemfwt/sumwt
      dm=abs(chemfwt - sumwt)
      if(rm .lt. 0.9 .or. rm .gt. 1.1 ) then
        call alert
     1 ('A','Calc and chemical_formula_moiety weights differ by > 10%')
        goto 1
      elseif(rm .lt. 0.95 .or. rm .gt. 1.05) then
        call alert
     1 ('B','Calc and chemical_formula_moiety weights differ by > 5%')
        goto 1
      elseif(rm .lt. 0.99 .or. rm .gt. 1.01) then
        call alert
     1 ('C','Calc and chemical_formula_moiety weights differ by > 1%')
        goto 1
      endif
      CHEMW_02=.true.
      item='_publ_requested_category'
      value=' '
      if(.not. chkchar(item,value,' ')) return
      if(dm .lt. 1.0) return
      call upcase(value)
      if(index(value,'FO') .eq. 0 .and. index(value,'FM') .eq. 0 .and.
     1 index(value,'CO') .eq. 0 .and. index(value,'CM') .eq. 0) return
      CHEMW_02=.false.
      call alert('A','Check formula_weight & chemical_formula_moiety')
1     write(value,100) chemfwt,sumwt
100   format('FORMULA-WT (given) = ',f9.2,':  FORMULA-WT (moiety) = '
     1,f9.2)
      call notice(value)
      end
C
C----------------------------------------------------------------------C
C
      logical function CHEMW_03()
C ---- test 17 - compares value of _chemical_formula_weight with that
C                calculated from the atomic contents
      logical centro,chknum,chkchar
      character*80 item,value
      CHEMW_03 = .false.
      item='_cell_formula_units_Z'
      if(.not. chknum(item,valZ,dum,' ')) return
      item='_chemical_formula_weight'
      if(.not. chknum(item,cfwt,dum,' ')) return

      call SYMMOPS(centro,isymm)
      if(isymm .eq. 0) return
      xx=SUMWT()/valZ
      if(xx .le. 0.0) return
      wt=xx*real(isymm)
      rc=cfwt/wt
      dc=abs(cfwt-wt)
      if(rc .lt. 0.9 .or. rc .gt. 1.1 ) then
        call alert
     1 ('A','Formula weight does not match Z and atomic contents by > 10
     1%')
        goto 3
      elseif(rc .lt. 0.95 .or. rc .gt. 1.05) then
        call alert
     1 ('B','Formula weight does not match Z and atomic contents by > 5%
     1')
        goto 3
      elseif(rc .lt. 0.99 .or. rc .gt. 1.01) then
        call alert
     1 ('C','Formula weight does not match Z and atomic contents by > 1%
     1')
        goto 3
      endif
      CHEMW_03=.true.
      if(dc .lt. 1.0) return
      item='_publ_requested_category'
      if(chkchar(item,value,' ')) then
        call upcase(value)
        if(index(value,'FO') .eq. 0 .and. index(value,'FM') .eq. 0 .and.
     1  index(value,'CO') .eq. 0 .and. index(value,'CM') .eq. 0) return
      endif
      CHEMW_03=.false.
      call alert
     1('A','Formula weight does not match Z and atomic contents')
3     write(value,100) cfwt,wt
100   format('FORMULA-WT (given) = ',f9.2,':  FORMULA-WT (contents)= ',
     1f9.2)
      call notice(value)
      if(wt .lt. cfwt)
     1call notice('Are all the hydrogen atoms included in the model ?')
      end
C
C----------------------------------------------------------------------C
C
      logical function CRYSC_01()
C ---- test 18  - checks whether _exptl_crystal_colour has allowed value
      parameter(nvalues=27)
      logical chkchar
      character*80 item,value,value1
      character*80 allowed_values(nvalues)
      data allowed_values/'metallic','lusterous','translucent','clear',
     1 'fluorescent','dark','light','intense','pale','white','black',
     1 'blue','violet','red','pink','yellow','gold','silver','bronze',
     1 'grey','gray','orange','green','colourless','colorless','brown',
     1 'purple'/
      CRYSC_01=.false.
      item='_exptl_crystal_colour'
      if(.not. chkchar(item,value,'A')) return
      value1=value
      do i=1,80
        if(value(i:i) .eq. '_' .or. value(i:i) .eq. '-') value(i:i)=' '
      enddo
      do i=1,nvalues
        ilen=leng(allowed_values(i))
        j=index(value,allowed_values(i)(1:ilen))
        if(j .ne. 0) then
          ilen=ilen-1
          value(j:j+ilen)=' '
        endif
      enddo
      if(value .ne. ' ') then
        call alert_char('B',item,value1)
        return
      endif
      CRYSC_01=.true.
      end
C
C----------------------------------------------------------------------C
C
      logical function CRYSR_01()
C ---- test 19
      character*80 item,value
      logical chkchar,chknum
      CRYSR_01=.false.
      item='_exptl_crystal_description'
      if(.not. chkchar(item,value,'A')) return
      if(index(value,'cylind') .ne. 0 .or. index(value,'radi') .ne. 0
     1 .or. index(value,'spher') .ne. 0) then
        item='_exptl_crystal_size_rad'
        if(.not. chknum(item,val,dum,'A')) return
      endif
      CRYSR_01=.true.
      end
C
C----------------------------------------------------------------------C
C
      logical function CRYSS_01()
C ---- test 20
      character*80 item
      logical chknum
      CRYSS_01=.false.
      item='_exptl_crystal_size_min'
      if(.not. chknum(item,cmin,dum,' ')) return
      item='_exptl_crystal_size_mid'
      if(.not. chknum(item,cmid,dum,' ')) return
      item='_exptl_crystal_size_max'
      if(.not. chknum(item,cmax,dum,' ')) return
      if(min(cmax,cmid,cmin) .le. 0.0) then
        call alert('A','One or more crystal dimensions are < 0.0 !!')
        return
      endif
      CRYSS_01=.true.
      if(cmin .gt. cmid) then
         call alert('B','exptl_crystal_size_min > mid')
         CRYSS_01=.false.
      endif
      if(cmid .gt. cmax) then
         call alert('B','exptl_crystal_size_mid > max')
         CRYSS_01=.false.
      endif
      if(.not. CRYSS_01) return
      CRYSS_01=.true.
      end
C
C----------------------------------------------------------------------C
C
      logical function CRYSS_02()
C ---- test 21  - to check that crystal dimensions do not exceed standard
C                 values (unless neutron diffraction study)
      character*80 item,value
      logical chknum,chkchar
      CRYSS_02=.false.
      item='_exptl_crystal_size_min'
      if(.not. chknum(item,cmin,dum,' ')) return
      item='_exptl_crystal_size_mid'
      if(.not. chknum(item,cmid,dum,' ')) return
      item='_exptl_crystal_size_max'
      if(.not. chknum(item,cmax,dum,' ')) return
      if(min(cmax,cmid,cmin) .le. 0.0) return
      item='_diffrn_radiation_type'
      if(.not. chkchar(item,value,'A')) return
      CRYSS_02=.true.
      if(index(value,'neutron') .ne. 0) return
      if(cmin .gt. 0.6) then
         call alert('B','exptl_crystal_size_min > 0.6')
         CRYSS_02=.false.
      endif
      if(cmid .gt. 0.8) then
         call alert('B','exptl_crystal_size_mid > 0.8')
         CRYSS_02=.false.
      endif
      if(cmax .gt. 1.0) then
         call alert('B','exptl_crystal_size_max > 1.0')
         CRYSS_02=.false.
      endif
      if(.not. CRYSS_02) return
      CRYSS_02=.true.
      end
C
C----------------------------------------------------------------------C
C
      logical function DENSD_01()
C ---- test 22
      character*80 item
      logical chknum
      DENSD_01=.true.
      item='_chemical_formula_weight'
      if(.not. chknum(item,cwt,dum,' ')) DENSD_01=.false.
      if(cwt .le. 0.0) DENSD_01=.false.
      item='_cell_formula_units_Z'
      if(.not. chknum(item,cfz,dum,' ')) DENSD_01=.false.
      if(cfz .le. 0.0) DENSD_01=.false.
      item='_cell_volume'
      if(.not. chknum(item,vol,dum,' ')) DENSD_01=.false.
      if(vol .le. 0.0) DENSD_01=.false.
      item='_exptl_crystal_density_diffrn'
      if(.not. chknum(item,ddens,dum,'A')) DENSD_01=.false.
      if(ddens .le. 0.0) DENSD_01=.false.
      if(.not. DENSD_01) return
      den=1.66042*cwt*cfz/vol
      rd=ddens/den
      if(rd .lt. 0.9 .or. rd .gt. 1.1) then
        call alert('A','EXPTL_DIFFRN & CALC densities differ by > 10%')
        return
      elseif(rd .lt. 0.95 .or. rd .gt. 1.05) then
        call alert('B','EXPTL_DIFFRN & CALC densities differ by > 5%')
        return
      elseif(rd .lt. 0.99 .or. rd .gt. 1.01) then
        call alert('C','EXPTL_DIFFRN & CALC densities differ by > 1%')
        return
      endif
      DENSD_01=.true.
      end
C
C----------------------------------------------------------------------C
C
      logical function DENSM_01()
C ---- test 23 - to check whether _exptl_crystal_density_method is given
C                if _exptl_crystal_density_meas is reported
      logical chkchar,chknum
      character*80 item,value
      DENSM_01=.true.
      item='_exptl_crystal_density_meas'
      if(.not. chknum(item,val,dum,' ')) return
      item='_exptl_crystal_density_method'
      if(.not. chkchar(item,value,'A')) DENSM_01=.false.
      end
C
C----------------------------------------------------------------------C
C
      logical function DENSX_01()
C ---- test 24
      logical chknum
      character*80 item
      tol=1.0e-4
      item='_exptl_crystal_density_meas'
      DENSX_01=chknum(item,dm,dum,' ')
      if(.not. DENSX_01) goto 1
      item='_exptl_crystal_density_diffrn'
      if(.not. chknum(item,dd,dum,' ')) DENSX_01=.false.
      if(.not. DENSX_01) return
      DENSX_01=.false.
      if(abs(dm) .le. tol .or. abs(dd) .le. tol) return
      rd=dd/dm
      if(rd .lt. 0.8 .or. rd .gt. 1.20) then
       call alert('A','Measured and diffraction densities differ by > 20
     1%')
       return
      elseif(rd .lt. 0.9 .or. rd .gt. 1.10) then
       call alert('B','Measured and diffraction densities differ by > 10
     1%')
       return
      elseif(rd .lt. 0.95 .or. rd .gt. 1.05) then
       call alert('C','Measured and diffraction densities differ by > 5%
     1')
       return
      endif
1     DENSX_01=.true.
      end
C
C----------------------------------------------------------------------C
C
      logical function DIFMN_01()
C ---- test 25
      logical chknum
      character*80 item
      DIFMN_01=.true.
      item='_refine_diff_density_min'
      if(.not. chknum(item,dmin,dum,'A')) DIFMN_01=.false.
      item='_refine_diff_density_max'
      if(.not. chknum(item,dmax,dum,'A')) DIFMN_01=.false.
      if(.not. DIFMN_01) return
      if( dmin .ge. dmax) then
        call alert('C','refine_diff_density_min > max')
        DIFMN_01=.false.
        return
      endif
      DIFMN_01=.true.
      end
C
C----------------------------------------------------------------------C
C
      logical function DIFMN_02()
C ---- test 26 - to check _refine_diff_density_min is within acceptable
C                limits
      logical chknum
      character*80 item
      character*6 buf1,buf2,buf3,buf4
      integer ZMAX,z_max
      DIFMN_02=.false.
      z_max=ZMAX()
      if(z_max .eq. 0) return
      item='_refine_diff_density_min'
      if(.not. chknum(item,dmin,dum,' ')) return
      dtest=-0.1*real(z_max)
      write(buf1,'(f6.1)') 2.0*dtest
      write(buf2,'(f6.1)') 1.0*dtest
      write(buf3,'(f6.1)') 0.75*dtest
      call trimstr(buf1)
      call trimstr(buf2)
      call trimstr(buf3)
      buf4='eA^3'
      if(dmin .lt. 2.0*dtest) then
        call alert('A','refine_diff_density_min is < '//buf1//buf4)
        return
      endif
      if(dmin .lt. dtest) then
        call alert('B','refine_diff_density_min is < '//buf2//buf4)
        return
      endif
      if(dmin .lt. 0.75*dtest) then
        call alert('C','refine_diff_density_min is < '//buf3//buf4)
        return
      endif
      DIFMN_02=.true.
      end
C
C----------------------------------------------------------------------C
C
      logical function DIFMN_03()
C ---- test 27 - to check that if  _refine_diff_density_min is not within
C                acceptable limits, atomic site is identified
      logical chknum
      character*80 item
      integer ZMAX,z_max
      DIFMN_03=.true.
      z_max=ZMAX()
      if(z_max .eq. 0) return
      item='_refine_diff_density_min'
      if(.not. chknum(item,dmin,dum,' ')) return
      dtest=0.1*real(z_max)
      DIFMN_03=.false.
      if(dmin .lt. -0.75*dtest) then
        call alert('C','Atom site specification needed for  refine_diff_
     1density_min ')
        return
      endif
      DIFMN_03=.true.
      end
C
C----------------------------------------------------------------------C
C
      logical function DIFMX_01()
C ---- test 28 - to check _refine_diff_density_max is within acceptable
C                limits
      logical chknum
      character*6 buf1,buf2,buf3,buf4
      character*80 item
      integer ZMAX,z_max
      DIFMX_01=.false.
      z_max=ZMAX()
      if(z_max .eq. 0) return
      item='_refine_diff_density_max'
      if(.not. chknum(item,dmax,dum,' ')) return
      dtest=0.1*real(z_max)
      write(buf1,'(f6.1)') 2.0*dtest
      write(buf2,'(f6.1)') 1.0*dtest
      write(buf3,'(f6.1)') 0.75*dtest
      call trimstr(buf1)
      call trimstr(buf2)
      call trimstr(buf3)
      buf4 = 'eA^3'
      DIFMX_01=.false.
      if(dmax .gt. 2.0*dtest) then
        call alert('A','refine_diff_density_max is > '//buf1//buf4)
        return
      endif
      if(dmax .gt. dtest) then
        call alert('B','refine_diff_density_max is > '//buf2//buf4)
        return
      endif
      if(dmax .gt. 0.75*dtest) then
        call alert('C','refine_diff_density_max is > '//buf3//buf4)
        return
      endif
      DIFMX_01=.true.
      end
C
C----------------------------------------------------------------------C
C
      logical function DIFMX_02()
C ---- test 29 - to check that if  _refine_diff_density_max is not within
C                acceptable limits, atomic site is identified
      logical chknum
      character*80 item
      integer ZMAX,z_max
      DIFMX_02=.false.
      z_max=ZMAX()
      if(z_max .eq. 0) return
      item='_refine_diff_density_max'
      if(.not. chknum(item,dmax,dum,' ')) return
      dtest=0.1*real(z_max)
      if(dmax .gt. 0.75*dtest) then
        call alert('C','Atom site specification needed for  refine_diff_
     1density_max ')
        return
      endif
      DIFMX_02=.true.
      end
C
C----------------------------------------------------------------------C
C
      logical function FCOEF_01()
C ---- test 30  - to chack value of _refine_ls_structure_factor_coef
C                 is recognised
      logical chkchar
      character*80 item,value
      item='_refine_ls_structure_factor_coef'
      FCOEF_01=.false.
      if(.not. chkchar(item,value,'A')) return
      if(value .ne. 'f' .and. value .ne. 'fsqd' .and. value .ne. 'inet')
     1 then
         call alert_char('A',item,value)
         return
      endif
      FCOEF_01=.true.
      end
C
C----------------------------------------------------------------------C
C
      logical function GOODF_01()
C ---- test 31
      logical chknum
      character*80 item
      GOODF_01=.false.
      item='_refine_ls_goodness_of_fit_ref'
      if(.not. chknum(item,goof,dum,' ')) then
        item='_refine_ls_goodness_of_fit_obs'
        if(.not. chknum(item,goof,dum,' ')) then
          call absent('A','_refine_ls_goodness_of_fit_ref(obs)')
          return
        endif
      endif
      if(goof .lt. 0.4 .or. goof .gt. 6.0) then
        call alert('A','GOOF is < 0.4 or > 6.0')
        return
      elseif(goof .lt. 0.6 .or. goof .gt. 4.0) then
        call alert('B','GOOF is < 0.6 or > 4.0')
        return
      elseif(goof .lt. 0.8 .or. goof .gt. 2.0) then
        call alert('C','GOOF is < 0.8 or > 2.0')
        return
      endif
      GOODF_01=.true.
      end
C
C----------------------------------------------------------------------C
C
      logical function HYDTR_01()
C ---- test 32 - to check that _refine_ls_hydrogen_treatment has a
C                recognised value
      parameter(nvalues=8)
      logical chkchar
      character*80 item,value,allowed_values(nvalues)
      data allowed_values/'refall','refxyz','refu','noref','undef',
     1 'constr','none','mixed'/
      HYDTR_01=.false.
      item='_refine_ls_hydrogen_treatment'
      if(.not. chkchar(item,value,'A')) return
      do i=1,nvalues
        ilen=leng(allowed_values(i))
        if(index(value,allowed_values(i)(1:ilen)) .ne. 0)HYDTR_01=.true.
      enddo
      if(.not. HYDTR_01) then
        call alert_char('A',item,value)
        call notice
     1('Allowed values are : refall,refxyz,refu,noref,undef,constr,none
     1or mixed')
      endif
      end
C
C----------------------------------------------------------------------C
C
      logical function RADNT_01()
C ---- test 33 - to check that radiation type is recognised
      parameter (nvalues = 8)
      logical chkchar
      character*80 item,value,allowed_values(nvalues),contstr
      data allowed_values/'cuk\a','mok\a','agk\a','neutron',
     1'synchrotron','mokalpha','cukalpha','agkalpha'/
      RADNT_01=.false.
      item='_diffrn_radiation_type'
      if(.not. chkchar(item,value,'A')) return
      value=contstr(value)
      do i=1,nvalues
        ilen=leng(allowed_values(i))
        if(index(value,allowed_values(i)(1:ilen)) .ne. 0)RADNT_01=.true.
      enddo
      if(.not. RADNT_01) then
        call alert_char('A',item,value)
        call notice
     1('Allowed values are : Cu K\a,Mo K\a,Ag K\a,neutron or synchrotron
     1')
      endif
      end
C
C----------------------------------------------------------------------C
C
      logical function RADNW_01()
C ---- test 34 - to check that _diffrn_radiation_type  matches the value
C                of _diffrn_radiation_wavelength
      logical chkchar,chknum
      character*80 item,value,contstr
      RADNW_01=.false.
      item='_diffrn_radiation_wavelength'
      if(.not. chknum(item,wavel,dum,' ')) return
      item='_diffrn_radiation_type'
      if(.not. chkchar(item,value,' ')) return
      value=contstr(value)
      if(index(value,'synchrotron') .ne. 0) goto 1
      if(index(value,'cuk\a') .ne. 0
     1  .or. index(value,'cukalpha') .ne. 0) then
        if(wavel .gt. 1.54180 .or. wavel .lt. 1.54175) then
          call alert('C','Incorrect wavelength for Cu Kalpha radiation')
          return
        endif
      elseif(index(value,'mok\a') .ne. 0
     1   .or. index(value,'mokalpha') .ne. 0) then
        if(wavel .gt. 0.71075 .or. wavel .lt. 0.71065) then
          call alert('C','Incorrect wavelength for Mo Kalpha radiation')
          return
        endif
      elseif(index(value,'agk\a') .ne. 0
     1   .or. index(value,'agkalpha') .ne. 0) then
        if(wavel .gt. 0.56085 .or. wavel .lt. 0.56080) then
          call alert('C','Incorrect wavelength for Ag Kalpha radiation')
          return
        endif
      else
        call alert('C','Check radiation type and wavelength carefully')
      endif
1     RADNW_01=.true.
      end
C
C----------------------------------------------------------------------C
C
      logical function REFLE_01()
C ---- test 35
      logical chkchar
      character*80 item,value,buf,buf1
      logical integ
      tol=1.0e-6
      REFLE_01=.false.
      item='_reflns_threshold_expression'
      if(.not. chkchar(item,value,' ')) then
        item='_reflns_observed_criterion'
        if(.not. chkchar(item,value,' ')) then
          call absent('A','_reflns_threshold_expression(observed_crit)')
          return
        endif
      endif
      i1=index(value,'(')
      i2=index(value,')')
      if(i1 .eq. 0 .or. i2 .eq. 0 .or. (i2-1) .lt. (i1+1) ) then
        call alert_char('A',item,value)
        return
      endif
      buf=value(i1+1:i2-1)
      if(buf .eq. ' ') then
        call alert_char('A',item,value)
        return
      endif
      buf1=value
      do i=1,80
        if(.not. integ(buf1(i:i)) ) buf1(i:i)=' '
      enddo
      call readbuf(buf1,val,1)
      if(abs(val) .le. tol) return
      if(index(buf,'i') .ne. 0 .or. index(buf,'f^2') .ne. 0) then
         if(val .ge. 6.0) then
           call alert('A','Threshold expression >= 6.0')
           return
         elseif(val .ge. 5.0) then
           call alert('B','Threshold expression >= 5.0')
           return
         elseif(val .ge. 4.0) then
           call alert('C','Threshold expression >= 4.0')
           return
         endif
      elseif(index(buf,'f') .ne. 0) then
         if(val .ge. 12.0) then
           call alert('A','Threshold expression >= 12.0')
           return
         elseif(val .ge. 10.0) then
           call alert('B','Threshold expression >= 10.0')
           return
         elseif(val .ge. 8.0) then
           call alert('C','Threshold expression >= 8.0')
           return
         endif
      else
        call alert_char('A',item,value)
        return
      endif
      REFLE_01=.true.
      end
C
C----------------------------------------------------------------------C
C
      logical function REFLG_01()
C ---- test 36 - to test whether _reflns_number_gt(observed) is less than
C                or equal to _diffrn_reflns_number
      logical chknum
      character*80 item
      REFLG_01=.false.
      item='_reflns_number_gt'
      if(.not. chknum(item,refgt,dum,' ')) then
        item='_reflns_number_observed'
        if(.not. chknum(item,refgt,dum,' ')) then
          call absent('A','_reflns_number_gt(observed)')
          return
        endif
      endif
      item='_diffrn_reflns_number'
      if(.not. chknum(item,drn,dum,'A')) return
      if (refgt .gt. drn) then
         call alert('A','reflns_number_gt(observed) >  diffrn_refln_numb
     1er')
        return
      endif
      REFLG_01=.true.
      end
C
C----------------------------------------------------------------------C
C
      logical function REFLL_01()
C ---- test 37 - to check _diffrn_reflns_limit are in the right order 
      logical chknum1
      character*80 item
      real hmin,hmax,kmin,kmax,lmin,lmax
      REFLL_01=.true.
      item='_diffrn_reflns_limit_h_min'
      if(.not. chknum1(item,hmin,dum,'A')) REFLL_01=.false.
      item='_diffrn_reflns_limit_h_max'
      if(.not. chknum1(item,hmax,dum,'A')) REFLL_01=.false.
      item='_diffrn_reflns_limit_k_min'
      if(.not. chknum1(item,kmin,dum,'A')) REFLL_01=.false.
      item='_diffrn_reflns_limit_k_max'
      if(.not. chknum1(item,kmax,dum,'A')) REFLL_01=.false.
      item='_diffrn_reflns_limit_l_min'
      if(.not. chknum1(item,lmin,dum,'A')) REFLL_01=.false.
      item='_diffrn_reflns_limit_l_max'
      if(.not. chknum1(item,lmax,dum,'A')) REFLL_01=.false.
      if(.not. REFLL_01) return
      if(hmin .ge. hmax) then
        call alert('A','H(min) is greater than H(max)')
        REFLL_01=.false.
      endif
      if(kmin .ge. kmax) then
        call alert('A','K(min) is greater than K(max)')
        REFLL_01=.false.
      endif
      if(lmin .ge. lmax) then
        call alert('A','L(min) is greater than L(max)')
        REFLL_01=.false.
      endif
      if(.not. REFLL_01) return
      REFLL_01=.true.
      end
C
C----------------------------------------------------------------------C
C
      logical function REFLT_01()
C ---- test 38  to check  _reflns_number_total is less than or equal to
C               _diffrn_reflns_number
      logical chknum
      character*80 item
      REFLT_01=.true.
      item='_diffrn_reflns_number'
      if(.not. chknum(item,d1,dum,'A')) REFLT_01=.false.
      item='_reflns_number_total'
      if(.not. chknum(item,d2,dum,'A')) REFLT_01=.false.
      if(.not. REFLT_01) return
      if( d2 .gt. d1) then
         call alert('A','reflns_number_total is > diffrn_reflns_number')
         REFLT_01=.false.
         return
      endif
      end
C
C----------------------------------------------------------------------C
C
      logical function REFLT_02()
C ---- test 39 - to check that _reflns_number_total is greater to or equal
C                to _reflns_number_gt
      logical chknum
      character*80 item
      REFLT_02=.false.
      item='_reflns_number_gt'
      if(.not. chknum(item,d1,dum,' ')) then
         item='_reflns_number_observed'
         if(.not. chknum(item,d1,dum,' ')) return
      endif
      item='_reflns_number_total'
      if(.not. chknum(item,d2,dum,' ')) return
      if( d2 .lt. d1) then
         call alert('A','reflns_number_total is < reflns_number_gt(obser
     1ved)')
         return
      endif
      REFLT_02=.true.
      end
C
C----------------------------------------------------------------------C
C
      logical function REFLT_03()
C ---- test 40
      logical chknum,chkchar
      character*80 item,value,message,htm
      character*3 type 
      logical SYMMS_01,centro
      REFLT_03=.false.
      pi=4.0*atan(1.0)
      item='_diffrn_reflns_theta_max'
      if(.not. chknum(item,d1,dum,' ')) return
      if(d1 .le. 0.0) return
      item='_reflns_number_total'
      if(.not. chknum(item,d2,dum,' ')) return
      if(d2 .le. 0.0) return
      item='_diffrn_radiation_wavelength'
      if(.not. chknum(item,wavel,dum,' ')) return
      if(wavel .le. 0.0) return
      item='_cell_volume'
      if(.not. chknum(item,vol,dum,' ')) return
      if(vol .le. 0.0) return
      item='_symmetry_space_group_name_H-M'
      if(.not. chkchar(item,value,'A')) return
      htm = value
      call getspmat1(htm,ispgr,nlatt)
      if(ispgr .lt. 0) return
      if(ispgr .eq. 0) then
         call alert('A','Unrecognised space group symbol : '//value)
         return
      endif
      if(.not. SYMMS_01()) return
      call SYMMOPS(centro,nsymm)
      ni=2
      type='no'
      if(centro) then
        type='yes'
        ni=1
      endif
      nlaue=(ni*nsymm)/nlatt
      if(nlaue .eq. 0) return
      write(message,110) nlatt, nsymm, nlaue,type
      call comment(message)
      s=sin(d1*pi/180.0)/wavel
      nref=nint(32.0*pi*vol*(s**3)/real(3*nlaue*nlatt))
      nref1=nint(d2)
      if(nref .ne. 0) then
        refrat=d2/real(nref)
      else
        refrat=100.0
      endif
      complete=100.0*(real(nref1)/real(nref))
      if(complete .gt. 100.0) complete=100.0
      write(message,100) nref1,nref,complete
100   format('Unique reflns = ',i6,': expected = ',i6,': complete = ',
     1 f6.2,'%')
110   format('NLATT = ',i1,': NSYMM = ',i2,': NLAUE = ',i2,
     1 ': CENTROSYMMETRIC = ',a)
      call comment(message)
      if(refrat .lt. 0.85 .or. refrat .gt. 1.15 ) then
        if(centro) then
        call alert('A','Observed and expected no. of reflns. differ by >
     1 15%')
        else
        call alert('C','Observed and expected no. of reflns. differ by >
     1 15%')
        endif
        if(refrat .gt. 1.05 .and. .not. centro) then
          call notice
     1('Please state how many Friedel pairs in count. If there are none,
     1')
          call notice
     1('explain why this count exceeds that calculated. ')
        endif
        return
      elseif(refrat .lt. 0.9 .or. refrat .gt. 1.10) then
        if(centro) then
        call alert('B','Observed and expected no. of reflns. differ by >
     1 10%')
        else
        call alert('C','Observed and expected no. of reflns. differ by >
     1 10%')
        endif
        if(refrat .gt. 1.05 .and. .not. centro) then
          call notice
     1('Please state how many Friedel pairs in count. If there are ')
          call notice
     1('none, explain why this count exceeds that calculated. ')
        endif
        return
      elseif(refrat .lt. 0.95 .or. refrat .gt. 1.05) then
        if(centro) then
        call alert('C','Observed and expected no. of reflns. differ by >
     1 5%')
        else
        call alert('C','Observed and expected no. of reflns. differ by >
     1 5%')
        endif
        if(refrat .gt. 1.05 .and. .not. centro) then
          call notice
     1('Please state how many Friedel pairs in count. If there are ')
          call notice
     1('none, explain why this count exceeds that calculated. ')
        endif
        return
      endif
      REFLT_03=.true.
      end
C
C----------------------------------------------------------------------C
C
      logical function REFNR_01()
C ---- test 41
      logical centro,chknum
      integer ZMAX
      character*80 item
      REFNR_01=.false.
      pi=4.0*atan(1.0)
      item='_diffrn_reflns_theta_max'
      if(.not. chknum(item,d1,dum,' ')) return
      item='_diffrn_radiation_wavelength'
      if(.not. chknum(item,wavel,dum,' ')) return
      item='_reflns_number_total'
      if(.not. chknum(item,r3,dum,' ')) return
      item='_refine_ls_number_reflns'
      if(.not. chknum(item,r1,dum,'A')) return
      item='_refine_ls_number_parameters'
      if(.not. chknum(item,r2,dum,'A')) return
      if(r2 .le. 0.0 .or. r3 .le. 0.0) return
      rrrt=r1/r3
      rrtp=r1/r2
      s=sin(d1*pi/180.0)/wavel
      imax=ZMAX()
      if(imax .eq. 0) return
      call SYMMOPS(centro,nsymm)
      if(nsymm .eq. 0) return
      if( s .lt. 0.60 .or. rrrt .gt. 0.95) then
        if(centro) then
          if(rrtp .lt. 6.) then
            call alert('A','Ratio of reflns/refined parameters < 6.0')
            return
          elseif(rrtp .lt. 8.0) then
            call alert('B','Ratio of reflns/refined parameters < 8.0')
            return
          elseif(rrtp .lt. 10.0) then
            call alert('C','Ratio of reflns/refined parameters < 10.0')
            return
          endif
        else
          if(imax .gt. 18) then
            if(rrtp .lt. 6.0 ) then
              call alert('A','Ratio of reflns/refined parameters < 6.0')
              return
            elseif(rrtp .lt. 8.0) then
              call alert('B','Ratio of reflns/refined parameters < 8.0')
              return
            elseif(rrtp .lt. 10.0) then
              call alert('C','Ratio of reflns/refined parameters < 10.0'
     1)
              return
            endif
          else
            if(rrtp .lt. 4.0 ) then
              call alert('A','Ratio of reflns/refined parameters < 4.0')
              return
            elseif(rrtp .lt. 6.0) then
              call alert('B','Ratio of reflns/refined parameters < 6.0')
              return
            elseif(rrtp .lt. 8.0) then
              call alert('C','Ratio of reflns/refined parameters < 8.0')
              return
            endif
          endif
        endif
      endif
      REFNR_01=.true.
      end
C
C----------------------------------------------------------------------C
C
      logical function RFACG_01()
C ---- test 42
      logical chknum
      character*80 item
      RFACG_01=.false.
      item='_refine_ls_R_factor_gt'
      if(.not. chknum(item,valR,dum,' ')) then
        item='_refine_ls_R_factor_obs'
        if(.not. chknum(item,valR,dum,' ')) then
          call absent('A','_refine_ls_R_factor_gt(obs)')
          return
        endif
      endif
      if(valR .gt. 0.20) then
        call alert('A','refine_ls_R_factor_ref(obs) > 0.20')
        return
      elseif(valR .gt. 0.15) then
        call alert('B','refine_ls_R_factor_ref(obs) > 0.15')
        return
      elseif(valR .gt. 0.10) then
        call alert('C','refine_ls_R_factor_ref(obs) > 0.10')
        return
      endif
      RFACG_01=.true.
      end
C
C----------------------------------------------------------------------C
C
      logical function RFACR_01()
C ---- test 43
      logical chknum
      character*80 item
      RFACR_01=.false.
      item='_refine_ls_wR_factor_ref'
      if(.not. chknum(item,valR,dum,' ')) then
        item='_refine_ls_wR_factor_obs'
        if(.not. chknum(item,valR,dum,' ')) then
          call absent('A','_refine_ls_wR_factor_ref(obs)')
          return
        endif
      endif
      if(valR .gt. 0.45) then
        call alert('A','refine_ls_wR_factor_ref(obs) > 0.45')
        return
      elseif(valR .gt. 0.35) then
        call alert('B','refine_ls_wR_factor_ref(obs) > 0.35')
        return
      elseif(valR .gt. 0.25) then
        call alert('C','refine_ls_wR_factor_ref(obs) > 0.25')
        return
      endif
      RFACR_01=.true.
      end
C
C----------------------------------------------------------------------C
C
      logical function RINT_01()
C ---- test 44
      logical chknum
      character*80 item
      RINT_01=.false.
      item='_diffrn_reflns_av_R_equivalents'
      if(.not. chknum(item,valR,dum,'A')) return
      if(valR .gt. 0.2) then
        call alert('A','diffrn_reflns_av_R_equivalents is > 0.20')
        return
      elseif(valR .gt. 0.15) then
        call alert('B','diffrn_reflns_av_R_equivalents is > 0.15')
        return
      elseif(valR .gt. 0.10) then
        call alert('C','diffrn_reflns_av_R_equivalents is > 0.10')
        return
      elseif(valR .lt. 0.0) then
        call alert('A','diffrn_reflns_av_R_equivalents is < 0.0')
        return
      endif
      RINT_01=.true.
      end
C
C----------------------------------------------------------------------C
C
      logical function SHIFSU_01()
C ---- test 45
      logical chknum1
      character*80 item
      SHIFSU_01=.false.
      item='_refine_ls_shift/su_max'
      if(.not. chknum1(item,valsu,dum,' ')) then
        item='_refine_ls_shift/esd_max'
        if(.not. chknum1(item,valsu,dum,' ')) then
           call absent('A','_refine_ls_shift/su(esd)_max')
           return
        endif
      endif
      if(valsu .le. 0.0) then
        call comment('refine_ls_shift/su(esd)_max is less than or equal
     1to zero ?')
      endif
      if(valsu .gt. 0.20 ) then
        call alert('A','refine_ls_shift/su(esd)_max > 0.20')
        return
      elseif(valsu .gt. 0.10) then
        call alert('B','refine_ls_shift/su(esd)_max > 0.10')
        return
      elseif(valsu .gt. 0.05) then
        call alert('C','refine_ls_shift/su(esd)_max > 0.05')
        return
      endif
      SHIFSU_01=.true.
      end
C
C----------------------------------------------------------------------C
C
      logical function STRDET_01()
C ---- test 46
      character*80 item,value
      character cspg*10,HallSymbol*17,cset*6,Scfl*8
      logical centro,chkchar,chknum
      integer*2 ch(230)
      STRDET_01=.false.
      do i=1,230
        ch(i)=0
      enddo
      ch(1)=1
      do i=3,5
        ch(i)=1
      enddo
      do i=16,24
        ch(i)=1
      enddo
      do i=75,80
        ch(i)=1
      enddo
      do i=89,98
        ch(i)=1
      enddo
      do i=143,146
        ch(i)=1
      enddo
      do i=149,155
        ch(i)=1
      enddo
      do i=168,173
        ch(i)=1
      enddo
      do i=177,182
        ch(i)=1
      enddo
      do i=195,199
        ch(i)=1
      enddo
      do i=207,214
        ch(i)=1
      enddo
      call SYMMOPS(centro,nsymm)
      if(nsymm .eq. 0) return
      STRDET_01=.true.
      if(centro) return
      item='_refine_ls_abs_structure_Flack'
      if(.not. chknum(item,val,dum,' ')) then
        item='_refine_ls_abs_structure_Rogers'
        if(.not. chknum(item,val,dum,' ')) then
          item='_symmetry_space_group_name_H-M'
          if(.not. chkchar(item,value,' ')) return 
          call SYMMPOS(value,ispgr,cspg,isym,HallSymbol,Scfl,cset)
          if(ispgr .eq. 0) return
          if(ch(ispgr) .eq. 0) return
          call comment
     1('Space group is chiral and absolute structure is not determined.'
     1)
          return
        endif
      endif
      STRDET_01=.false.
      item='_refine_ls_abs_structure_details'
      if(.not. chkchar(item,value,'A')) return 
      STRDET_01=.true.
      end
C
C----------------------------------------------------------------------C
C
      logical function STRVAL_01()
C ---- test 47
      character*80 item
      integer ZMAX,z_max
      logical centro,chknum
      STRVAL_01=.true.
      call SYMMOPS(centro,isymm)
      if(isymm .eq. 0) return
      z_max=ZMAX()
      if(z_max .eq. 0) return
      val=0.
      su=0.
      item='_refine_ls_abs_structure_Flack'
      if(.not. chknum(item,val,su,' ')) return
      if(centro) then
        call comment('Flack parameter refined for centric structure ?!')
        return
      endif
      STRVAL_01=.false.
      item='_diffrn_radiation_wavelength'
      if(.not. chknum(item,wavel,dum,' ')) return
C ---- extra condition for wavelength
      if( z_max .lt. 14 .and. wavel .lt. 1.4) then
        call comment('Flack parameter probably meaningless')
      endif
      if(su .gt. 0.5) then
        call alert('C','Esd of Flack parameter > 0.5')
        return
      endif
      if(su .gt. 1.0) then
        call alert('B','Esd of Flack parameter > 1.0')
        return
      endif
      if(val .lt. -0.3) then
        call alert('A','Flack parameter < -0.3 ??')
        return
      endif
      if(val .gt.  0.3) then
        call alert('C','Flack parameter >  0.3')
        return
      endif
      if(val .gt.  0.5) then
        call alert('B','Flack parameter >  0.5')
        return
      endif
      if(val .gt.  0.7) then
        call alert('A','Flack parameter >  0.7')
        return
      endif
      STRVAL_01=.true.
      val1=val+3.0*su
      val2=val-3.0*su
      val3=min(abs(val1),abs(val2))
      if(val3 .le. 3.0*su) then
        call comment
     1  ('Absolute configuration confirmed at 3(sigma) level - Flack')
        return
      endif
      STRVAL_01=.false.
      if(val .le. 0.0 .and. abs(su) .lt. abs(val) ) then
        call alert('C','Unusual value of Flack parameter ')
        return
      endif
      end
C
C----------------------------------------------------------------------C
C
      logical function STRVAL_02()
C ---- test 48 - algorithm by LJF (IUCr algorithm not available)
      character*80 item
      logical centro,chknum
      integer ZMAX,z_max
      STRVAL_02=.true.
      call SYMMOPS(centro,isymm)
      if(isymm .eq. 0) return
      z_max=ZMAX()
      if(z_max .eq. 0) return
      val=0.
      sur=0.
      item='_refine_ls_abs_structure_Rogers'
      if(.not. chknum(item,val,su,' ')) return
      if(centro) then
       call comment('Rogers parameter refined for centric structure ?!')
       return
      endif
      STRVAL_02=.false.
      item='_diffrn_radiation_wavelength'
      if(.not. chknum(item,wavel,dum,' ')) return
      if( z_max .lt. 14 .and. wavel .lt. 1.4) then
        call comment('Rogers parameter probably meaningless')
      endif
      if(su .gt. 0.5) then
        call alert('C','Esd of Rogers parameter > 0.5')
        return
      endif
      if(su .gt. 1.0) then
        call alert('B','Esd of Rogers parameter > 1.0')
        return
      endif
      if(val .lt.  -0.5 .and. su .lt. 0.5) then
        call alert('A','ROGERS parameter indicates opposite absolute con
     1figuration ??')
        return
      endif
      if(val .gt. -0.5 .and. val .lt. 0.5) then
        call alert('C','Unreliable indication of absolute configuration'
     1)
        return
      endif

      STRVAL_02=.true.
      val1=val+3.0*su
      val2=val-3.0*su
      val3=min(abs(1.0-val1),abs(1.0-val2))
      if(val3 .le. 3.0*su) then
        call comment
     1  ('Absolute configuration confirmed at 3(sigma) level - Rogers')
        return
      endif
      STRVAL_02=.false.
      end
C
C----------------------------------------------------------------------C
C
      logical function SYMMG_01()
C ---- test 49
      logical chkchar,chknum
      character*80 item,value,value1,message
      character cspg*10,HallSymbol*17,cset*6,Scfl*8
      SYMMG_01=.false.
      item='_symmetry_space_group_name_H-M'
      if(.not. chkchar(item,value,' ')) then
        return
      endif
      value1=value
      call SYMMPOS(value,ispgr,cspg,isym,HallSymbol,Scfl,cset)
      if(isym .eq. 0) return
      if(ispgr .eq. -1) return
      SYMMG_01=(ispgr .ne. 0)
      if(SYMMG_01) then
        write(message,100) value(1:leng(value)), ispgr, isym
        call comment(message)
      else
        call alert_char('A',item,value1)
      endif
      item='_symmetry_Int_Tables_number'
      if(.not. chknum(item,val,dum,' ')) return
      if(nint(val) .ne. ispgr) then
        SYMMG_01=.false.
        call alert('A','symmetry_Int_Tables_number does not match name')
      endif
100   format('Space group  = ',a,': Int. Tables no. = ',i3,
     1': N(equiv) = ',i3)
      end
C
C----------------------------------------------------------------------C
C
      logical function SYMMG_02()
C ---- test 50 - to check space group symbol is consistent with equivalent
C                positions
      logical centro,chkchar,CHECKRTMAT
      character*80 item,value
      character cspg*10,HallSymbol*17,cset*6,Scfl*8
      SYMMG_02=.false.
      item='_symmetry_space_group_name_H-M'
      if(.not. chkchar(item,value,' ')) return
      call SYMMOPS(centro,isymm)
      call SYMMPOS(value,ispgr,cspg,isym,HallSymbol,Scfl,cset)
      if(ispgr .eq. -1) return
      if(isym .eq. 0) return
      SYMMG_02=CHECKRTMAT(value,isym)
      if(.not. SYMMG_02 .and. isymm .ne. 0) then
        call comment1
     1  ('Space group symbol and equivalent positions do not match')
        call alert('A',
     1   'Space group symbol and equivalent positions do not match')
      endif
      end
C
C----------------------------------------------------------------------C
C
      logical function SYMMS_01()
C ---- test 51
      logical chkchar
      character*80 item,value
      SYMMS_01 = .false.
      item='_symmetry_cell_setting'
      if(.not. chkchar(item,value,' ')) return
      if(value .ne. 'triclinic'    .and. value .ne. 'monoclinic' .and.
     1   value .ne. 'orthorhombic' .and. value .ne. 'tetragonal' .and.
     2   value .ne. 'rhombohedral' .and. value .ne. 'trigonal'   .and.
     3   value .ne. 'hexagonal'    .and. value .ne. 'cubic') then
        call alert_char('A',item,value)
        call notice
     1('Allowed values are : triclinic, monoclinic, orthorhombic,tetrago
     1nal')
        call notice('rhombohedral,trigonal,hexagonal or cubic')
        return
      endif
      SYMMS_01 = .true.
      end
C
C----------------------------------------------------------------------C
C
      logical function SYMMS_02()
C ---- test 52
      logical chkchar,chknum
      character*80 item,value
      logical l1,l2,l3,l4,l5,l6,l7,l8,l9
      tol=1.0e-3
      SYMMS_02=.false.
      item='_symmetry_cell_setting'
      if(.not. chkchar(item,value,' ')) return
      item='_cell_angle_alpha'
      if(.not. chknum(item,alpha,dum,' ')) return
      item='_cell_angle_beta'
      if(.not. chknum(item,beta,dum,' ')) return
      item='_cell_angle_gamma'
      if(.not. chknum(item,gamma,dum,' ')) return
      item='_cell_length_a'
      if(.not. chknum(item,a,dum,' ')) return
      item='_cell_length_b'
      if(.not. chknum(item,b,dum,' ')) return
      item='_cell_length_c'
      if(.not. chknum(item,c,dum,' ')) return
      l1=(abs(a-b) .le. tol)                   ! a=b
      l2=(abs(a-c) .le. tol)                   ! a=c
      l3=(abs(b-c) .le. tol)                   ! b=c
      l4=(abs(alpha - 90.0) .le. tol)          ! alpha=90
      l5=(abs(beta - 90.0) .le. tol)           ! beta=90
      l6=(abs(gamma -90.0) .le. tol)           ! gamma=90
      l7=(abs(alpha-beta) .le. tol)            ! alpha=beta
      l8=(abs(alpha-gamma) .le. tol)           ! alpha=gamma
      l9=(abs(gamma-120.) .le. tol)            ! gamma=120
      if(index(value,'triclinic') .ne. 0) then
        if( l1 .or. l2 .or. l4 .or. l5 .or. l6) goto 1
      elseif(index(value,'monoclinic') .ne. 0) then
        if( l1 .or. l2 .or. l3 .or. (l4 .and. l5 .and. l6) .or.
     1 ( (.not. l4 .and. .not. l5) .or.
     1   (.not. l4 .and. .not. l6) .or.
     1   (.not. l5 .and. .not. l6))) goto 1
      elseif(index(value,'orthorhombic') .ne. 0) then
        if( l1 .or. l2 .or.(.not. l4 .or. .not. l5 .or. .not. l6))goto 1
      elseif(index(value,'tetragonal') .ne. 0) then
        if( .not. l1  .or. (.not. l4 .or. .not. l5 .or. .not. l6))goto 1
      elseif(index(value,'rhombohedral') .ne. 0) then
        if( (.not.l1)  .or. (.not.l2) .or. (.not. l7) .or. (.not. l8)
     1     .or. l4 .or. l5 .or. l6 ) goto 1
      elseif((index(value,'trigonal')+index(value,'hexagonal')) .ne. 0)
     1       then
        if( (.not.l1)  .or. ((.not.l4) .or. (.not.l5) .or. (.not.l9)))
     1 goto 1
      elseif(index(value,'cubic') .ne. 0) then
        if( (.not.l1)  .or. (.not.l2) .or. (.not.l4) .or. (.not.l5)
     1 .or. (.not.l6)) goto 1
      else
        return
      endif
      SYMMS_02=.true.
      return
1     call alert('B','Cell metrics indicate symmetry_cell_setting may be
     1 incorrect?')
      end
C
C----------------------------------------------------------------------C
C
      logical function THETM_01()
C ---- test 53
      logical chknum
      character*80 item
      real wavel,thmax,dum
      pi=4.0*atan(1.0)
      THETM_01=.false.
      item='_diffrn_radiation_wavelength'
      if(.not. chknum(item,wavel,dum,' ')) return
      if(wavel .le. 0.0) then
        call alert('A','diffrn_radiation_wavelength cannot be zero !')
        return
      endif
      item='_diffrn_reflns_theta_max'
      if(.not. chknum(item,thmax,dum,' ')) return
      s=sin(thmax*pi/180.0)/wavel
      write(item,100) s
      if( s .lt. 0.55) then
         call alert('A',item)
      elseif( s .lt. 0.575) then
         call alert('B',item)
      elseif( s .lt. 0.60) then
         call alert('C',item)
      elseif( s .ge. 0.6) then
        THETM_01=.true.
        goto 1
      endif
100   format('Sin(theta_max)/lambda = ',f6.3,': ideally it should be > 0
     1.600')
      call comment1(item)
1     end
C
C----------------------------------------------------------------------C
C
      logical function WEIGHT_01()
C ---- test 54
      logical chkchar
      character*80 item,value
      WEIGHT_01=.false.
      item='_refine_ls_weighting_scheme'
      if(.not. chkchar(item,value,'B')) return
      WEIGHT_01=.true.
      if(index(value,'calc') .ne. 0 .or. index(value,'sigma') .ne. 0
     1 .or. index(value,'unit') .ne. 0) return
      WEIGHT_01=.false.
      call alert_char('A',item,value)
      call notice('Allowed values are : sigma, unit or calc')
      end
C
C---------------- IUCRVAL UTILITY ROUTINES ----------------------------C
C
      subroutine absent(val,string)
      parameter(nt=54)
      common /tests/itest(nt),ntests,ifail(3)
      character*(*) string
      character val
      write(14,'(/a,a,a,a)')
     1 ' ALERT ',val,' - Mandatory item missing: ',
     1                string(1:leng(string))
      if(val .eq. 'A') ifail(1)=ifail(1)+1
      if(val .eq. 'B') ifail(2)=ifail(2)+1
      if(val .eq. 'C') ifail(3)=ifail(3)+1
      end
C
C----------------------------------------------------------------------C
C
      subroutine alert(val,string)
      parameter(nt=54)
      common /tests/itest(nt),ntests,ifail(3)
      character*(*) string
      character val
      write (14,100) val,string(1:leng(string))
100   format(/' ALERT ',a,' - ',a)
      if(val .eq. 'A') ifail(1)=ifail(1)+1
      if(val .eq. 'B') ifail(2)=ifail(2)+1
      if(val .eq. 'C') ifail(3)=ifail(3)+1
      end
C
C----------------------------------------------------------------------C
C
      subroutine alert_char(val,item,value)
      parameter(nt=54)
      common /tests/itest(nt),ntests,ifail(3)
      character*(*) item,value
      character val
      if(value .eq. ' ') value='?'
      write (14,100) val,item(1:leng(item)),value(1:leng(value))
100   format(/' ALERT ',a,' - Data item ',a,/
     1        '           has unrecognised value: ',a)
      if(val .eq. 'A') ifail(1)=ifail(1)+1
      if(val .eq. 'B') ifail(2)=ifail(2)+1
      if(val .eq. 'C') ifail(3)=ifail(3)+1
      end
C
C----------------------------------------------------------------------C
C
      subroutine cdin(line,j,a1,a2,a3,t,termin,empty)
C ---- interprets symmetry character string: either decimal or
C ---- fractions may be used for translations.
      character line(148),chara,digit(11),charb(148)
      integer sig
      real number
      logical first,termin,empty
      common /cifmod/nr,r(192,3,4),mult1
      data digit/'1','2','3','4','5','6','7','8','9','0','.'/,
     1 charb/148*' '/
      empty=.true.
      a1=0.0
      a2=0.0
      a3=0.0
      t=0.0
C ---- set sig
10    sig=1
      num=0
      first=.true.
C ---- clear b
11    k=1
C ---- next
12    j=j+1
      if (j .le. 145) goto 36
      chara=';'
      goto 18
36    chara=line(j)
      if (chara .eq. ' ') goto 12
      empty=.false.
      if (chara .eq. '+' .or. chara .eq. '-') goto 13
      if (chara .eq. 'X' .or. chara .eq. 'Y' .or. chara .eq. 'Z')goto 14
      if (chara .eq. '/') goto 15
      do 16 i=1,11
         if (chara .eq. digit(i)) goto 17
16    continue
      goto 18
C ---- digits
17    k=k+1
      charb(k)=chara
      first=.false.
      goto 12
C ---- slash
15    if (k .eq. 1) goto 19
      call iformt(charb,2,k-1,num)
      goto 11
19    num=0
      goto 11
C ---- xyz
14    if (k .eq. 1) goto 24
      call fformt(charb,2,k-1,number)
      if (num .eq. 0) goto 21
      number=sig*num/number
      goto 23
21    number=sig*number
      goto 23
24    number=sig
23    if (chara .ne. 'X') goto 25
      a1=number
      goto 10
25    if (chara .ne. 'Y') goto 26
      a2=number
      goto 10
26    a3=number
      goto 10
C ---- signs
13    if (.not. first) goto 27
      if (chara .ne. '+') goto 28
      sig=1
      goto 29
28    sig=-1
29    first=.false.
27    if (k .eq. 1) goto 30
      call fformt(charb,2,k-1,t)
      if (num .eq. 0) goto 32
      t=sig*num/t
      goto 33
32    t=sig*t
33    if (chara .ne. '+') goto 31
      sig=1
      goto 30
31    sig=-1
30    num=0
      goto 11
C ---- terminator
18    if (chara .ne. ',' .and. chara .ne. ';') then
        call comment
     1  ('Wrong equivalent position delimiter: '//chara)
      endif
      termin=chara .eq. ';'
      if (num .eq. 0 .and. k .eq. 1) return
      if (k .eq. 1) goto 35
      call fformt(charb,2,k-1,t)
      goto 22
35    t=0.0
22    if (num .eq. 0) goto 34
      t=sig*num/t
      return
34    t=sig*t
      return
      end
C
C----------------------------------------------------------------------C
C
      logical function CHECKRTMAT(value,mult)
C ---- function returns true if the set of RT matrices from routines
C      GETSPEC and SYMMOPS are identical.
      common /symnew/nr,msym,rs(3,3,48),rt(3,48),nt,t(3,4)
      common /cifmod/nr1,r1(192,3,4),mult1
      character*80 value 
      real t1(3,4),t2(3,4),rnormx
      logical meqrt
      icount=0
      CHECKRTMAT =.false.

C --- make sure RT matrices are in common block  symnew
      call getspmat1(value,ispgr,nlt)
      if(ispgr .le. 0) return
      if(mult .ne. mult1) return
      do i=1,nr1
        do j=1,3
          do k=1,4
            t1(j,k)=r1(i,j,k)
            if(k .eq. 4) t1(j,k) = rnormx(t1(j,k))
          enddo
        enddo
        do l=1,nr
          do ii=1,nt
            do jj=1,3
              do kk=1,3
                t2(jj,kk) = rs(jj,kk,l) 
              enddo
              t2(jj,4) = rt(jj,l) + t(jj,ii)
              t2(jj,4) = rnormx(t2(jj,4))
            enddo
           if(meqrt(t1,t2)) icount=icount+1
          enddo
        enddo
      enddo
      CHECKRTMAT=(icount .eq. nr1)
      end
C
C----------------------------------------------------------------------C
C
      logical function chkchar(item,value,val)
C ---- Tests whether character item "item" exists, and returns its value.
C      This routine takes into account text type entries for a
C      character string.
      include 'ciftbx.cmn'
      character*80 item,value
      character val
      chkchar=.false.
      value=' '
      if(.not. test_(item)) then
         if(val .ne. ' ') call absent(val,item)
         return
      endif
1     if(char_(item,value)) then
         call trimstr(value)
         if(value.ne.' ' .and. value.ne. '?' .and. value.ne.'.') then
           chkchar=.true.
           goto 2
         endif
      endif
      if(text_) goto 1
      if(val .ne. ' ') call absent(val,item)
      return
2     call locase(value)
      end
C
C----------------------------------------------------------------------C
C
      logical function chknum(item,value,su,val)
C ---- checks whether data item "item" is present and returns its value
C      and esd if appropriate. NOTE that items with a possible value of
C      zero should not use this routine. Use chknum1 instead

      include 'ciftbx.cmn'
      character*(*) item,val
      real dvalue,dsu
      chknum=.false.
      tol=1.0e-6
      dvalue=0.0
      dsu=0.0
      if(.not. numb_(item,dvalue,dsu)) then
        if(val .ne. ' ') call absent(val,item)
        value=dvalue
        su=dsu
        return
      endif
      value=dvalue
      su=dsu
      if(abs(value) .le. tol) then
        if(val .ne. ' ') call absent(val,item)
        return
      endif
      chknum=.true.
      end
C
C----------------------------------------------------------------------C
C
      logical function chknum1(item,value,su,val)
C ---- checks whether data item "item" is present and returns its value
C      and esd if appropriate. NOTE that items with a possible value of
C      zero should use this routine. 

      include 'ciftbx.cmn'
      character*(*) item,val
      real dvalue,dsu
      chknum1=.false.
      tol=1.0e-6
      dvalue=0.0
      dsu=0.0
      if(.not. numb_(item,dvalue,dsu)) then
        if(val .ne. ' ') call absent(val,item)
        value=dvalue
        su=dsu
        return
      endif
      value=dvalue
      su=dsu
      chknum1=.true.
      end
C
C----------------------------------------------------------------------C
C
      subroutine check_label(value,ierr)
C ---- new routine added for function SUMWT()
      character*(*) value
      character*80 buf
      character*26 aleph
      integer ierr
      data aleph/'abcdefghijklmnopqrstuvwxyz'/
      ierr=0
      buf=value
      call trimstr(buf)
      call locase(buf)
      call upcase(buf(1:1))
      buf(3:)=' '
      if(index(aleph,buf(2:2)) .eq. 0) buf(2:2)=' '
      value=buf
      call ELINFO(buf,ielem,atwt)
      if(ielem .eq. 0) ierr=1
      end
C
C----------------------------------------------------------------------C
C
      subroutine comment(string)
      character*(*) string
      write (*,100) string(1:leng(string))
      write (14,100) string(1:leng(string))
100   format(/' COMMENT - ',a)
      end
C
C----------------------------------------------------------------------C
C
      subroutine comment1(string)
      character*(*) string
      write (*,100) string(1:leng(string))
100   format(/' COMMENT - ',a)
      end
C
C----------------------------------------------------------------------C
C
      subroutine ELINFO(el,iel,atwt)
C
C ---- returns Z and atomic weight for element "el" if recognised,
C      otherwise zero for both variables

      character*(*) el
      character*2 celem(92), buf
      real atwt1(104)
      integer*2 iatnr(104)
      data celem/
     *'H ','He','Li','Be','B ','C ','N ','O ','F ','Ne','Na','Mg',
     *'Al','Si','P ','S ','Cl','Ar','K ','Ca','Sc','Ti','V ','Cr',
     *'Mn','Fe','Co','Ni','Cu','Zn','Ga','Ge','As','Se','Br','Kr',
     *'Rb','Sr','Y ','Zr','Nb','Mo','Tc','Ru','Rh','Pd','Ag','Cd',
     *'In','Sn','Sb','Te','I ','Xe','Cs','Ba','La','Ce','Pr','Nd',
     *'Pm','Sm','Eu','Gd','Tb','Dy','Ho','Er','Tm','Yb','Lu','Hf',
     *'Ta','W ','Re','Os','Ir','Pt','Au','Hg','Tl','Pb','Bi','Po',
     *'At','Rn','Fr','Ra','Ac','Th','Pa','U '/
      data iatnr /
     1    1,    6,    8,    7,   17,   16,   35,   15,   53,   29,
     1   28,   89,   47,   13,   95,   18,   33,   85,   79,    5,
     1   56,    4,   83,   97,   20,   48,   58,   98,   96,   27,
     1   24,   55,   -1,   66,   68,   99,   63,    9,   26,  100,
     1   87,   31,   64,   32,    2,   72,   80,   67,   49,   77,
     1   19,   36,   57,    3,   71,  103,  101,   12,   25,   42,
     1   11,   41,   60,   10,  102,   93,   76,   91,   82,   46,
     1   61,   84,   59,   78,   94,   88,   37,   75,   45,   86,
     1   44,   51,   21,   34,   14,   62,   50,   38,   73,   65,
     1   43,   52,   90,   22,   81,   69,   92,   23,   74,   54,
     1   39,   70,   30,   40/
c ---- Atomic weights (C12-IUPAC 1989) - Handbook for Physics & Chemistry
c      updated (IUPAC 1993) from Inorg. Chim. Acta 217 (1994) 217-218
      data atwt1 /
     1   1.00794,    12.011,   15.9994, 14.00674,   35.4527,   32.066,
     2    79.904, 30.973762, 126.90447,   63.546,   58.6934, 227.0278,
     3  107.8682, 26.981539,    241.06,   39.948,  74.92159,   209.99,
     4 196.96654,    10.811,   137.327, 9.012182, 208.98037,   249.08,
     5    40.078,   112.411,   140.115,   252.08,    244.06, 58.93320,
     6   51.9961, 132.90543,     2.014,   162.50,    167.26,   252.08,
     7   151.965, 18.998403,    55.845,   257.10,    223.02,   69.723,
     8    157.25,     72.61,  4.002602,   178.49,    200.59, 164.9303,
     9    114.82,    192.22,   39.0983,    83.80,  138.9055,    6.941,
     *   174.967,    262.11,    258.10,  24.3050,  54.93805,    95.94,
     1 22.989768,  92.90638,    144.24,  20.1797,    259.10, 237.0482,
     2    190.23, 231.03588,     207.2,   106.42,    146.92,   209.98,
     3 140.90765,    195.08,    239.05, 226.0254,   85.4678,  186.207,
     4 102.90550,    222.02,    101.07,  121.757, 44.955910,    78.96,
     5   28.0855,    150.36,   118.710,    87.62,  180.9479, 158.9253,
     6    98.906,    127.60,  232.0381,   47.867,  204.3833, 168.9342,
     7  238.0289,   50.9415,    183.84,   131.29,  88.90585,   173.04,
     8     65.39,    91.224/
      call trimstr(el)
      buf=el(1:2)
      call upcase(el(1:1))
      call locase(el(2:2))
      iel=0
      do i=1,92
        if(buf .eq. celem(i)) iel=i
      enddo
      atwt=0.0
      if(iel .ne. 0) then
        do i=1,104
          if(iatnr(i) .eq. iel) atwt=atwt1(i)
        enddo
      endif
      end
C
C----------------------------------------------------------------------C
C
      subroutine errstop(string)
      character*(*) string
      write (*,100) string(1:leng(string))
      write (14,100) string(1:leng(string))
100   format(/' ERROR   - ',a)
      stop
      end
C
C----------------------------------------------------------------------C
C
      subroutine error(string)
      character*(*) string
      write (*,100) string(1:leng(string))
      write (14,100) string(1:leng(string))
100   format(/' ERROR   - ',a)
      end
C
C----------------------------------------------------------------------C
C
      subroutine notice(string)
      character*(*) string
      write (14,100) string(1:leng(string))
100   format('         - ',a)
      end
C
C----------------------------------------------------------------------C
C
      logical function integ(char1)
C ----- tests whether char1 is a numeric character
      character char1,number(11)
      data number/'0','1','2','3','4','5','6','7','8','9','.'/
      integ=.false.
      do i=1,11
         if (char1 .eq. number(i)) integ=.true.
      enddo
      end
C
C----------------------------------------------------------------------C
C
      subroutine iucr_test(func)
      parameter (nt=54)
      common /tests/itest(nt),ntests,ifail(3)
      logical func
      external func
      save i
      data i/0/
      i=i+1
      if(i .gt. nt) i=1
      if(.not. func() ) then
        ntests=ntests+1
        itest(ntests)=i
      endif
      end
C
C----------------------------------------------------------------------C
C
      subroutine parsfm(buf,bufout,nmoi,moieties,ierr)
      character*80 moieties(20),buf,bufout
      character*11 numbers
      logical found
      integer nmoi
      numbers='0123456789.'
      ierr=1
C ---- separate string into moieties
      nmoi=0
1     ind=index(buf,',')
      if(ind .eq. 0) then
         if(buf .ne. ' ') then
           nmoi=nmoi+1
           moieties(nmoi)=buf
          call trimstr(moieties(nmoi))
         endif
      else
        nmoi=nmoi+1
        moieties(nmoi) = buf(1:ind-1)
        call trimstr(moieties(nmoi))
        buf(1:ind) =' '
        goto 1
      endif
C ---- remove any charges from strings
      do i=1, nmoi
        indp  = index(moieties(i),'+')
        indm = index(moieties(i),'-')
        if(indp .gt. 0 .and. indm .gt. 0) goto 10
        ind=max(indp,indm)
        if(ind .ne. 0) then
          moieties(i)(ind:ind) = ' '
          do j=ind-1,1,-1
            found = .false.
            if(index(numbers,moieties(i)(j:j)).ne. 0) found = .true.
            if(found) moieties(i)(j:j) = ' '
            if(.not.found) goto 2
          enddo
2         continue
        endif
      enddo
C ---- replace any pre-multipliers with post-multipliers
      if(nmoi .le. 1) goto 3
      do i=1,nmoi
        ibrl=0
        ibrr=0
        imrk1=0
        imrk2=0
        do j=1,80
          if(moieties(i)(j:j) .eq. '(') then
            ibrl = ibrl + 1
            imrk1 = j
          endif
          if(moieties(i)(j:j) .eq. ')') then
            ibrr = ibrr + 1
            imrk2 = j
          endif
        enddo
        if(ibrr .ne. ibrl) goto 20
        if(ibrr .gt. 1) goto 30
        if(imrk1 .gt. 1) then
C ---- check whether pre-multiplier is numeric character
          buf=moieties(i)(1:imrk1-1)
          do k=1,imrk1-1
            if(index(numbers,buf(k:k)) .eq. 0) goto 40
          enddo
          moieties(i)(1:imrk1-1)=' '
          moieties(i)(imrk2+1:) = buf
          call trimstr(moieties(i))
        endif
      enddo
3     write(bufout,'(20(a,1x))')
     1  (moieties(i)(1:leng(moieties(i))),i=1,nmoi)
      ierr=0
      return
10    continue
      call alert('C','Both -ve and +ve charges in moiety : '//
     1moieties(i))
      return
20    continue
      call alert('C','Unmatched parentheses in moiety : '//
     1moieties(i))
      return
30    continue
      call alert('C','Illegal nested parentheses in moiety : '//
     1moieties(i))
      return
40    continue
      call alert('C','Non-numeric characters before parentheses in moiet
     1y : '//moieties(i))
      end
C
C----------------------------------------------------------------------C
C
      subroutine readep(name,ierr)
      character*(*)name
      character line(148),buf*148
      integer fch(20),fwd(20)
      ierr=0
      do i=1,148
         line(i)=' '
      enddo
      call upcase(name)
      write (buf,'(''SYMM  '',a)')name
      read (buf,'(72a1)')(line(i),i=2,73)
      call parse(line,nfields,fch,fwd)
      call symin(line,fch,ierr)
      end
C
C----------------------------------------------------------------------C
C
      subroutine symin(line,firstc,ierr)
      character line(148)
      integer firstc(20),start
      logical semico,empty
      common /cifmod/nr,r(192,3,4),mult1
      ierr=0
      tol=1.0e-30
      start=firstc(2)-1
11    nr=nr+1
      do 10 i=1,3
         call cdin(line,start,r(nr,i,1),r(nr,i,2),r(nr,i,3),r(nr,i,4),
     1   semico,empty)
         if (.not. empty) goto 12
         nr=nr-1
         return
12       if (.not. ((semico .and. i .ne. 3) .or.
     1              (.not. semico .and. i .eq. 3))) goto 10
         call comment
     1   ('Equivalent position sequence is incorrect')
         ierr=1
         return
10    continue
C
C ---- ignore identity operation
      do 102 i=1,3
C         if (r(nr,i,i) .ne. 1.0) goto 13
         if (abs(r(nr,i,i) - 1.0) .gt. tol ) goto 13
         do 101 j=1,4
            if (i .eq. j) goto 101
C            if (r(nr,i,j) .ne. 0.0) goto 13
            if (abs(r(nr,i,j)) .gt. tol) goto 13
101      continue
102   continue
      nr=nr-1
13    if (start .lt. 145) goto 11
      return
      end
C
C----------------------------------------------------------------------C
C
      real function SGMULT(x)
C ---- returns the space group multiplicity of the site with fractional
C      coordinates in vector x(3)
C      x and z are normalised to 0.0000 =< x,z < 0.9999
C
C   NOTE: the multiplicity given here is NOT the same as that defined
C   in International Tables A nor for atom_site_symmetry_multiplicity
C   as given in CIF_CORE.DIC. Here the multiplicity is defined as :
C
C       multiplicity of general position/multiplicity of the site in x
C
C       possible values range from 1.0 (for a general position) to 48.0
C       SHELXL97-2 gives the WRONG value for multiplicity (i.e. the same
C       as defined here)
C
      common /cifmod/nr,RT(192,3,4),mult1
      real x(3),z(3),rnormx
      tol=1.0e-3
      SGMULT=0.0
      do i=1,3
       x(i)=rnormx(x(i))
      enddo
      do i=1,nr
        x1=x(1)
        x2=x(2)
        x3=x(3)
        z(1)=(RT(i,1,1)*x1)+(RT(i,1,2)*x2)+(RT(i,1,3)*x3)+RT(i,1,4)
        z(2)=(RT(i,2,1)*x1)+(RT(i,2,2)*x2)+(RT(i,2,3)*x3)+RT(i,2,4)
        z(3)=(RT(i,3,1)*x1)+(RT(i,3,2)*x2)+(RT(i,3,3)*x3)+RT(i,3,4)
        do j=1,3
          z(j)=rnormx(z(j))
        enddo
        if(abs(x(1)-z(1)) .le. tol .and. abs(x(2)-z(2)) .le. tol .and.
     1     abs(x(3)-z(3)) .le. tol) SGMULT=SGMULT+1.0
      enddo
      end
C
C----------------------------------------------------------------------C
C
      real function SUMWT()
C ---- returns the formula weight of the asymmetric unit as read from
C      the atom list
      parameter(it=30)
      include 'ciftbx.cmn'
      character*80 item,value,string,value1
      character*26 alp
      character*2 ftype(it)
      logical f1
      real atweights(it),SGMULT,x(3)
      real valocc,valmult,dx(3),dum
      itype=0
      SUMWT=0.0
      alp='abcdefghijklmnopqrstuvwxyz'

C ---- first obtain all atomic types 
      item='_atom_type_symbol'
1     f1=char_(item,value)
      if(.not. f1) then
        call absent('A',item)
        return
      endif
      call trimstr(value)
      call upcase(value)
      call locase(value(2:2))
      value(3:)=' '
      if(index(alp,value(2:2)) .eq. 0) value(2:2)=' '
      itype=itype+1
      ftype(itype)=value(1:2)
      if(loop_) goto 1
C ---- check elemental types and obtain atomic weights
      do i=1,itype
         call ELINFO(ftype(i),iel,atwt)
         atweights(i) = atwt
         if(iel .eq. 0) then
           call alert('A','Urecognisable atom type : '//ftype(i))
           return
         endif
      enddo

C ---- now loop over all atomic sites - if occupancy absent assume = 1.0
2     item='_atom_site_type_symbol'
c      if(.not. char_(item,value)) then
c        call absent('A',item)
c        return
c      endif

C ---- LJF change 28-11-98 - allow interpretation of _atom_site_label in
C      the absence of item _atom_site_type_symbol

      if(.not. char_(item,value)) then
         item='_atom_site_label'
         if(.not. char_(item,value)) then
           call absent('A','_atom_site_type_symbol')
           return
         endif
         value1=value
         call check_label(value,ierr)
         if(ierr .ne. 0) then
           call comment
     1     ('Item _atom_site_type_symbol missing. Interpret labels.')
           call comment('Uninterpretable atom label : '//value1)
           return
         endif
      endif
C ---- end change


      call trimstr(value)
      call upcase(value)
      call locase(value(2:2))
      value(3:)=' '
      if(index(alp,value(2:2)) .eq. 0) value(2:2)=' '
      valocc=1.0
      f1=numb_('_atom_site_occupancy',valocc,dum) 

C ---- get atom site multiplicity
      valmult=1.0
C temporary disable as SHELXL97-2 gives wrong value for this item
C      if(.not.numb_('_atom_site_symmetry_multiplicity',valmult,dum))then
        dx(1)=0.0
        dx(2)=0.0
        dx(3)=0.0
        f1=numb_('_atom_site_fract_x',dx(1),dum)
        f1=numb_('_atom_site_fract_y',dx(2),dum)
        f1=numb_('_atom_site_fract_z',dx(3),dum)
        do  i=1,3
          x(i)=dx(i)
        enddo
        if(f1) then
          valmult=SGMULT(x)
        else
          valmult=1.0
        endif
C      endif
      if(valmult .le. 0.0) call errstop('MULT is zero or less !')
      ie=0
      do i=1,itype
        if(value(1:2) .eq. ftype(i)) ie=i
      enddo
      if(ie .eq. 0) then
        write(string,1000) value(1:2)
        call alert('A',string)
        return
      endif
      SUMWT=SUMWT + valocc*atweights(ie)/valmult
      if(loop_) goto 2
1000  format('Atom type ',a2,' has no match with given atom_type_symbol'
     1)
      end
C
C----------------------------------------------------------------------C
C
      subroutine SYMMPOS(value,ispgr,cspg,isym,Hall,Scfl,cset)
C ---- verifies input space group symbol using lookup table SGINFO.DAT

C      on input : value contains space group string to be verified

C      on output: value contains condensed space group string
C                 ispgr contains space group number
C                 cspg contains expanded Hermann-Mauguin symbol for
C                 interpretation by routine SPGR
C                 isym contains multiplicity of general position
C                 Hall contains Hall space group symbol
C                 Scfl contains Schoenflies symmetry
C                 cset contains setting
C      if unverified, return values: ispgr & isym = 0, value & cspg = ' '
C      if SGINFO.DAT not found  ispgr = -1

      character*80 value,line,buf,contstr,filepath,spec
      character cspgr*20,Hall*17,cspg*10,cspgr1*10,cspgr2*10
      character cset*6,Scfl*8,ibuf*3
      buf=value
      ilen=leng(buf)
C
C ---- strip brackets a la SHELXTL style from symbol
C
      if(ilen .gt. 0) then
         do i=1,ilen
            if(buf(i:i) .eq. ')' .or. buf(i:i) .eq. '(' ) buf(i:i)=' '
            if(buf(i:i) .eq. ';') buf(i:i)=':'
         enddo
      endif
      buf=contstr(buf)
      call upcase(buf)
      ispgr=0
      isym=0
      cspg=' '
      cset=' '
      lfi=99
      open(lfi,file=filepath('SGINFO.DAT'),status='old',
     1 err=1)
10    read(lfi,'(a)',end=3,err=2) line
      if(line(1:1) .ne. ' ') goto 10
      Scfl = line(13:20)
      Hall = line(52:68)
      cset = line(6:11)
      cspg  = line(41:50)
      ind=index(Hall,'(')
      if(ind .ne. 0) Hall(ind:) = ' '
      read(line,'(i4)',err=2) ispgr
      ibuf=line(69:71)
      ilen=leng(ibuf)
      write(spec,'(''(i'',i1,'')'')') ilen
      read(ibuf(1:ilen),spec,err=2) isym
      cspgr = line(21:40)
      call upcase(cspgr)
      cspgr1=' '
      cspgr2=' '
      ind=index(cspgr,'=')
      if(ind .ne. 0) then
         cspgr1=cspgr(1:ind-1)
         cspgr2=cspgr(ind+2:)
         cspgr=cspgr1
      endif

      ind=index(buf,':')
      if(ind .eq. 0) then
C
C ---- setting is not given on input string
C      so remove settings from compare string "cspgr"
C
        ind=index(cspgr,':')
        if(ind .ne. 0) cspgr(ind:) = ' '
C ---- use setting #2 (centre of symmetry at origin) by default
        if(cset(1:1) .eq. '1') goto 10
      endif
      if(buf .eq. cspgr .or. buf .eq. cspgr2) then
        call locase(cspgr(2:))
        value=cspgr
        goto 2
      endif
      goto 10
3     close(lfi)
      cspg=value(1:10)
      value=' '
      Hall=' '
      Scfl=' '
      cset=' '
      ispgr=0
      isym=0
      return
2     close(lfi)
      return
1     ispgr=-1
      end
C
C----------------------------------------------------------------------C
C
      subroutine SYMMOPS(centro,isymm)
C ---- returns the number of symmetry operators, and centricity
      include 'ciftbx.cmn'
      real r1(3,3),t1(3),r2(3,3),t2(3)
      common /cifmod/nr,r(192,3,4),mult1
      logical meq1,modt1,centro
      external meq1,modt1
      character*80 item,value
      centro=.true.
      isymm=0
      mult1=0
      nr=1
      do i=1,3
         do j=1,4
            r(1,i,j)=0.0
         enddo
      enddo
C
      r(1,1,1)=1.0
      r(1,2,2)=1.0
      r(1,3,3)=1.0

      item= '_symmetry_equiv_pos_as_xyz'
1     if(.not. char_(item,value)) return
      call readep(value,ierr)
      if(ierr.eq.1) return
      isymm=isymm+1
      mult1=isymm
      if (loop_) goto 1
C
C ---- outer loop
      centro=.false.
      do i1=1,nr
         do j1=1,3
            t1(j1)=r(i1,j1,4)
            do k1=1,3
               r1(j1,k1)=r(i1,j1,k1)
            enddo
         enddo
C
C ---- inner loop
         do i2=i1+1,nr
            do j2=1,3
               t2(j2)=r(i2,j2,4)
               do k2=1,3
                  r2(j2,k2)=r(i2,j2,k2)
               enddo
            enddo
C ---- check for centrosymmetric equivalents
            if (meq1(r1,r2) .and. modt1(t1,t2)) centro=.true.
         enddo
      enddo
      end
C
C----------------------------------------------------------------------C
C
      subroutine TABSPH(angle,muR,afact,tfact)
C
C ---- given muR and theta values(deg), this routine returns
C      afact = the RELATIVE spherical absorption correction factor A*
C      tfact =  the absolute transmission factor 
C      Only valid for 0.1 < muR < 10.0
C
C***********************************************************************
C*                                                                     *
C* SPHERABS: a FORTRAN77 program for calculating X-ray absorption for  *
C* a spherical single crystal sample.                                  *
C*                                                                     *
C*                           by M.NARDELLI                             *
C*                                                                     *
C* Istituto di Chimica Generale ed Inorganica della Universita` degli  *
C* Studi di Parma, Centro di Studio per la Strutturistica Diffrattome- *
C* trica del C.N.R., Viale delle Scienze, I-43100 Parma, Italy.        *
C*                                                                     *
C* The program calculates: the absorption correction by interpolating  *
C* data from a table giving the absorption correction factors. The     *
C* table is that for spherical crystals: Int.Tab. vol. C (1992) p. 523;*
C* Tab. 6.3.3.3 for values of muR in the range 0-2.5, and Int.Tab.     *
C* vol.II (1959) p.302; Table 5.3.6 B for muR in the range 2.6-10.0.   *                    *
C* The interpolation procedure illustrated by C.W.Dwiggins Jr (Acta    *
C* Cryst.(1975).A31,146-148) is used with some modification.           *
C***********************************************************************

      real angle,muR
      real*8 st,al0,al1,al2,al3
      dimension af(101,19),rm(101),th(19),as(19)
      data ((af(i,k),i=1,101),k=1,2)/1.0,1.1609,1.3457,1.5574,1.7994,
     1 2.0755,2.3897,2.7467,3.1511,3.6082,4.1237,4.7035,5.3542,6.082,
     1 6.895,7.801,8.806,9.920,11.151,12.507,13.998,15.632,17.419,
     1 19.369,21.489,23.791,26.3,29.0,31.9,35.0,38.4,42.0,45.8,49.9,
     1 54.3,58.9,63.8,69.0,74.6,80.4,86.5,93.0,99.8,107,114,122,130,
     1 139,148,157,167,177,188,199,210,222,234,247,260,274,288,303,318,
     1 334,350,366,383,401,419,438,457,477,498,519,540,563,585,609,633,
     1 657,683,709,735,762,790,819,848,878,909,940,972,1005,1038,1072,
     1 1107,1143,1180,1217,1255,1294,1333,1.0,1.1609,1.3456,1.5571,
     1 1.7988,2.0743,2.3877,2.7434,3.1461,3.6009,4.1131,4.6886,5.3335,
     1 6.054,6.857,7.750,8.740,9.834,11.040,12.366,13.819,15.408,
     1 17.141,19.025,21.069,23.280,25.6,28.2,30.9,33.9,37.0,40.4,43.9,
     1 47.7,51.7,56.0,60.4,65.1,70.1,75.3,80.7,86.4,92.4,98.6,105,112,
     1 119,126,134,141,150,158,167,176,185,194,204,214,224,235,246,257,
     1 269,280,293,305,318,331,344,357,371,385,400,415,430,445,460,476,
     1 493,509,526,543,560,578,596,614,632,651,670,690,709,729,749,770,
     1 791,812,833,855,877,899,921/
      data ((af(i,k),i=1,101),k=3,4)/1.0,1.1609,1.3452,1.5561,1.7968,
     1 2.0706,2.3816,2.7336,3.1312,3.5789,4.0815,4.6442,5.2722,5.9710,
     1 6.746,7.604,8.549,9.587,10.725,11.967,13.320,14.788,16.376,
     1 18.089,19.931,21.907,24.0,26.2,28.6,31.2,33.9,36.7,39.7,42.8,
     1 46.0,49.5,53.0,56.8,60.6,64.6,68.8,73.1,77.5,82.1,86.8,91.7,96.7,
     1 102,107,112,118,124,129,135,141,147,153,160,166,173,179,186,193,
     1 200,207,214,221,229,236,244,251,259,267,275,283,291,299,307,315,
     1 324,332,341,349,358,367,375,384,393,402,411,420,429,439,448,457,
     1 467,476,485,495,505,514,1.0,1.1607,1.3447,1.5546,1.7935,2.0647,
     1 2.3715,2.7177,3.1069,3.5431,4.0304,4.5729,5.1747,5.8399,6.573,
     1 7.377,8.256,9.214,10.254,11.380,12.593,13.895,15.290,16.778,
     1 18.361,20.040,21.8,23.7,25.6,27.7,29.9,32.1,34.4,36.8,39.3,41.9,
     1 44.6,47.3,50.2,53.1,56.0,59.1,62.2,65.3,68.6,71.9,75.2,78.6,82.1,
     1 85.6,89.1,92.7,96.4,100,104,108,111,115,119,123,127,131,135,139,
     1 143,148,152,156,160,165,169,173,177,182,186,191,195,200,204,209,
     1 213,218,222,227,232,236,241,246,250,255,260,264,269,274,279,284,
     1 288,293,298,303,308/
      data ((af(i,k),i=1,101),k=5,6)/1.0,1.1606,1.3439,1.5525,1.7891,
     1 2.0565,2.3578,2.6959,3.0740,3.4952,3.9625,4.4790,5.0476,5.6710,
     1 6.352,7.092,7.894,8.759,9.689,10.685,11.746,12.874,14.067,
     1 15.327,16.652,18.041,19.5,21.0,22.6,24.2,25.9,27.7,29.5,31.3,
     1 33.2,35.2,37.1,39.2,41.2,43.3,45.5,47.6,49.9,52.1,54.4,56.7,59.0,
     1 61.3,63.7,66.1,68.5,71.0,73.4,75.9,78.4,80.9,83.5,86.0,88.6,91.2,
     1 93.8,96.4,99.1,102,104,107,110,112,115,118,121,123,126,129,132,
     1 135,137,140,143,146,149,152,154,157,160,163,166,169,172,175,178,
     1 180,183,186,189,192,195,198,201,204,207,1.0,1.1603,1.3428,1.5497,
     1 1.7833,2.0462,2.3406,2.6691,3.0339,3.4374,3.8816,4.3686,4.9001,
     1 5.4776,6.102,6.775,7.497,8.268,9.088,9.957,10.873,11.837,12.847,
     1 13.902,15.000,16.142,17.3,18.6,19.8,21.1,22.4,23.8,25.2,26.6,
     1 28.1,29.5,31.0,32.6,34.1,35.7,37.2,38.8,40.5,42.1,43.7,45.4,47.1,
     1 48.8,50.5,52.2,53.9,55.6,57.4,59.2,60.9,62.7,64.5,66.3,68.1,69.9,
     1 71.7,73.6,75.4,77.2,79.1,81.0,82.8,84.7,86.6,88.5,90.4,92.2,94.1,
     1 96.0,98.0,99.9,102,104,106,108,110,111,113,115,117,119,121,123,
     1 125,127,129,131,133,135,137,139,141,143,145,147,149/
      data ((af(i,k),i=1,101),k=7,8)/1.0,1.1600,1.3415,1.5463,1.7765,
     1 2.0340,2.3206,2.6382,2.9882,3.3723,3.7917,4.2474,4.7404,5.2711,
     1 5.8400,6.447,7.092,7.774,8.492,9.246,10.034,10.855,11.708,
     1 12.592,13.504,14.445,15.4,16.4,17.4,18.5,19.5,20.6,21.7,22.8,
     1 23.9,25.1,26.2,27.4,28.6,29.8,31.0,32.2,33.4,34.7,35.9,37.2,38.4,
     1 39.7,41.0,42.2,43.5,44.8,46.1,47.4,48.8,50.1,51.4,52.7,54.1,55.4,
     1 56.8,58.1,59.5,60.8,62.2,63.5,64.9,66.3,67.7,69.0,70.4,71.8,73.2,
     1 74.6,76.0,77.4,78.8,80.2,81.6,83.0,84.4,85.8,87.2,88.6,90.1,91.5,
     1 92.9,94.3,95.8,97.2,98.6,100,101,103,104,106,107,109,110,112,113,
     1 1.0,1.1597,1.3400,1.5426,1.7689,2.0204,2.2984,2.6042,2.9386,
     1 3.3026,3.6966,4.1211,4.5761,5.0617,5.5774,6.123,6.697,7.299,
     1 7.928,8.583,9.262,9.964,10.688,11.433,12.198,12.982,13.8,14.6,
     1 15.4,16.3,17.1,18.0,18.9,19.8,20.7,21.6,22.5,23.4,24.4,25.3,26.3,
     1 27.2,28.2,29.2,30.1,31.1,32.1,33.1,34.1,35.1,36.1,37.1,38.1,39.1,
     1 40.1,41.1,42.2,43.2,44.2,45.2,46.3,47.3,48.4,49.4,50.4,51.5,52.5,
     1 53.6,54.6,55.7,56.7,57.8,58.9,59.9,61.0,62.0,63.1,64.2,65.2,66.3,
     1 67.4,68.4,69.5,70.6,71.7,72.7,73.8,74.9,76.0,77.1,78.2,79.3,80.4,
     1 81.5,82.6,83.7,84.8,85.9,87.0,88.2,89.3/
      data ((af(i,k),i=1,101),k=9,10)/1.0,1.1593,1.3383,1.5383,1.7604,
     1 2.0056,2.2746,2.5683,2.8869,3.3208,3.6001,3.9945,4.4137,4.8573,
     1 5.3244,5.8143,6.326,6.859,7.411,7.982,8.570,9.175,9.795,10.429,
     1 11.077,11.738,12.4,13.1,13.8,14.5,15.2,15.9,16.6,17.4,18.1,18.9,
     1 19.6,20.4,21.1,21.9,22.7,23.4,24.2,25.0,25.8,26.6,27.4,28.1,28.9,
     1 29.7,30.5,31.3,32.2,33.0,33.8,34.6,35.4,36.2,37.0,37.9,38.7,39.5,
     1 40.3,41.2,42.0,42.8,43.6,44.5,45.3,46.1,47.0,47.8,48.7,49.5,50.3,
     1 51.2,52.0,52.8,53.7,54.5,55.4,56.2,57.1,57.9,58.8,59.6,60.4,61.3,
     1 62.1,63.0,63.9,64.7,65.6,66.4,67.3,68.2,69.1,69.9,70.8,71.7,72.6,
     1 1.0,1.1589,1.3366,1.5339,1.7515,1.9901,2.2500,2.5316,2.8347,
     1 3.1592,3.5048,3.8710,4.2571,4.6625,5.0862,5.5273,5.9849,6.458,
     1 6.946,7.447,7.961,8.486,9.023,9.569,10.125,10.690,11.3,11.8,
     1 12.4,13.0,13.6,14.2,14.8,15.4,16.1,16.7,17.3,17.9,18.6,19.2,19.8,
     1 20.5,21.1,21.8,22.4,23.1,23.7,24.4,25.0,25.7,26.4,27.0,27.7,28.3,
     1 29.0,29.7,30.3,31.0,31.7,32.3,33.0,33.7,34.4,35.0,35.7,36.4,37.1,
     1 37.8,38.4,39.1,39.8,40.5,41.2,41.8,42.5,43.2,43.9,44.6,45.3,45.9,
     1 46.6,47.3,48.0,48.7,49.4,50.1,50.7,51.4,52.1,52.8,53.5,54.2,54.9,
     1 55.6,56.3,57.0,57.7,58.4,59.1,59.8,60.5/
      data ((af(i,k),i=1,101),k=11,12)/1.0,1.1586,1.3348,1.5293,1.7425,
     1 1.9745,2.2255,2.4952,2.7835,3.0898,3.4135,3.7540,4.1104,4.4819,
     1 4.8676,5.2666,5.6780,6.101,6.535,6.978,7.431,7.893,8.362,8.839,
     1 9.322,9.810,10.3,10.8,11.3,11.8,12.3,12.8,13.4,13.9,14.4,14.9,
     1 15.5,16.0,16.5,17.1,17.6,18.2,18.7,19.3,19.8,20.3,20.9,21.4,22.0,
     1 22.6,23.1,23.7,24.2,24.8,25.3,25.9,26.4,27.0,27.6,28.1,28.7,29.3,
     1 29.8,30.4,31.0,31.5,32.1,32.7,33.2,33.8,34.4,34.9,35.5,36.1,36.6,
     1 37.2,37.8,38.4,38.9,39.5,40.1,40.6,41.2,41.8,42.4,42.9,43.5,44.1,
     1 44.7,45.2,45.8,46.4,46.9,47.5,48.0,48.6,49.1,49.6,50.2,50.7,51.2,
     1 1.0,1.1582,1.3331,1.5248,1.7335,1.9592,2.2015,2.4602,2.7346,
     1 3.0241,3.3280,3.6455,3.9756,4.3175,4.6703,5.0333,5.4057,5.7867,
     1 6.176,6.572,6.975,7.385,7.800,8.220,8.645,9.074,9.51,9.94,
     1 10.4,10.8,11.3,11.7,12.2,12.6,13.1,13.5,14.0,14.5,14.9,15.4,15.8,
     1 16.3,16.8,17.2,17.7,18.2,18.7,19.1,19.6,20.1,20.6,21.0,21.5,22.0,
     1 22.5,22.9,23.4,23.9,24.4,24.9,25.3,25.8,26.3,26.8,27.3,27.8,28.2,
     1 28.7,29.2,29.7,30.2,30.7,31.1,31.6,32.1,32.6,33.1,33.6,34.1,34.6,
     1 35.0,35.5,36.0,36.5,37.0,37.5,38.0,38.5,39.0,39.5,39.9,40.4,40.9,
     1 41.3,41.8,42.2,42.7,43.1,43.5,43.9,44.2/
      data ((af(i,k),i=1,101),k=13,14)/1.0,1.1579,1.3313,1.5204,1.7249,
     1 1.9445,2.1789,2.4274,2.6892,2.9637,3.2499,3.5470,3.8542,4.1706,
     1 4.4955,4.8281,5.1678,5.5140,5.8662,6.224,6.587,6.955,7.327,7.702,
     1 8.081,8.462,8.85,9.23,9.62,10.0,10.4,10.8,11.2,11.6,12.0,12.4,
     1 12.8,13.2,13.6,14.0,14.4,14.8,15.2,15.7,16.1,16.5,16.9,17.3,17.7,
     1 18.1,18.5,19.0,19.4,19.8,20.2,20.6,21.0,21.5,21.9,22.3,22.7,23.1,
     1 23.6,24.0,24.4,24.8,25.2,25.7,26.1,26.5,26.9,27.3,27.8,28.2,28.6,
     1 29.0,29.4,29.9,30.3,30.7,31.1,31.6,32.0,32.4,32.9,33.3,33.7,34.1,
     1 34.6,35.0,35.4,35.8,36.2,36.6,37.0,37.4,37.7,38.1,38.4,38.7,39.0,
     1 1.0,1.1575,1.3297,1.5162,1.7169,1.9311,2.1583,2.3977,2.6484,
     1 2.9098,3.1807,3.4605,3.7483,4.0432,4.3447,4.6520,4.9647,5.2823,
     1 5.6045,5.9308,6.261,6.595,6.932,7.272,7.614,7.957,8.30,8.64,
     1 8.99,9.34,9.70,10.0,10.4,10.8,11.1,11.5,11.8,12.2,12.6,12.9,13.3,
     1 13.6,14.0,14.4,14.7,15.1,15.5,15.8,16.2,16.6,16.9,17.3,17.7,18.0,
     1 18.4,18.8,19.1,19.5,19.9,20.3,20.6,21.0,21.4,21.7,22.1,22.5,22.9,
     1 23.2,23.6,24.0,24.3,24.7,25.1,25.5,25.8,26.2,26.6,27.0,27.3,27.7,
     1 28.1,28.5,28.8,29.2,29.6,30.0,30.3,30.7,31.1,31.5,31.8,32.2,32.6,
     1 32.9,33.3,33.6,33.9,34.3,34.6,34.9,35.2/
      data ((af(i,k),i=1,101),k=15,16)/1.0,1.1572,1.3282,1.5126,1.7099,
     1 1.9194,2.1403,2.3719,2.6133,2.8634,3.1216,3.3870,3.6586,3.9360,
     1 4.2183,4.5052,4.7961,5.0907,5.3888,5.6900,5.9942,6.301,6.610,
     1 6.922,7.235,7.548,7.86,8.17,8.49,8.81,9.13,9.45,9.77,10.1,10.4,
     1 10.7,11.1,11.4,11.7,12.0,12.4,12.7,13.0,13.4,13.7,14.0,14.3,14.7,
     1 15.0,15.3,15.7,16.0,16.3,16.7,17.0,17.3,17.7,18.0,18.3,18.7,19.0,
     1 19.3,19.7,20.0,20.3,20.7,21.0,21.3,21.7,22.0,22.3,22.7,23.0,23.4,
     1 23.7,24.0,24.4,24.7,25.0,25.4,25.7,26.0,26.4,26.7,27.1,27.4,27.7,
     1 28.1,28.4,28.7,29.1,29.4,29.7,30.1,30.4,30.7,31.0,31.4,31.7,32.0,
     1 32.3,1.0,1.1570,1.3271,1.5096,1.7041,1.9097,1.1256,2.3508,2.5845,
     1 2.8258,3.0738,3.3276,3.5866,3.8500,4.1174,4.3883,4.6622,4.9390,
     1 5.2184,5.5001,5.7842,6.070,6.358,6.648,6.938,7.229,7.52,7.81,
     1 8.10,8.40,8.70,8.99,9.29,9.59,9.88,10.2,10.5,10.8,11.1,11.4,11.7,
     1 12.0,12.3,12.6,12.9,13.2,13.5,13.8,14.1,14.4,14.7,15.0,15.3,15.6,
     1 15.9,16.2,16.5,16.8,17.2,17.5,17.8,18.1,18.4,18.7,19.0,19.3,19.6,
     1 19.9,20.2,20.5,20.8,21.1,21.4,21.7,22.1,22.4,22.7,23.0,23.3,23.6,
     1 23.9,24.2,24.5,24.8,25.1,25.4,25.7,26.1,26.4,26.7,27.0,27.3,27.6,
     1 27.9,28.2,28.5,28.8,29.1,29.5,29.8,30.1/
      data ((af(i,k),i=1,101),k=17,18)/1.0,1.1568,1.3262,1.5074,1.6997,
     1 1.9024,2.1145,2.3351,2.5632,2.7979,3.0383,3.2838,3.5334,3.7868,
     1 4.0432,4.3024,4.5641,4.8279,5.0936,5.3613,5.6307,5.9017,6.174,
     1 6.448,6.722,6.996,7.27,7.54,7.82,8.10,8.38,8.66,8.94,9.21,9.49,
     1 9.77,10.1,10.3,10.6,10.9,11.2,11.5,11.7,12.0,12.3,12.6,12.9,13.2,
     1 13.4,13.7,14.0,14.3,14.6,14.9,15.1,15.4,15.7,16.0,16.3,16.6,16.8,
     1 17.1,17.4,17.7,18.0,18.3,18.5,18.8,19.1,19.4,19.7,20.0,20.3,20.5,
     1 20.8,21.1,21.4,21.7,22.0,22.3,22.5,22.8,23.1,23.4,23.7,24.0,24.3,
     1 24.5,24.8,25.1,25.4,25.7,26.0,26.3,26.5,26.8,27.1,27.4,27.7,28.0,
     1 28.3,1.0,1.1567,1.3256,1.5059,1.6970,1.8979,2.1076,2.3253,2.5499,
     1 2.7805,3.0163,3.2566,3.5005,3.7477,3.9974,4.2495,4.5036,4.7595,
     1 5.0170,5.2760,5.5365,5.7982,6.061,6.325,6.589,6.853,7.12,7.38,
     1 7.65,7.92,8.18,8.45,8.72,8.99,9.25,9.52,9.79,10.1,10.3,10.6,10.9,
     1 11.1,11.4,11.7,11.9,12.2,12.5,12.8,13.0,13.3,13.6,13.8,14.1,14.4,
     1 14.6,14.9,15.2,15.5,15.7,16.0,16.3,16.5,16.8,17.1,17.4,17.6,17.9,
     1 18.2,18.4,18.7,19.0,19.3,19.5,19.8,20.1,20.3,20.6,20.9,21.2,21.4,
     1 21.7,22.0,22.2,22.5,22.8,23.1,23.3,23.6,23.9,24.2,24.4,24.7,25.0,
     1 25.2,25.5,25.8,26.1,26.3,26.6,26.9,27.2/
      data (af(i,19),i=1,101)/1.0,1.1567,1.3254,1.5055,
     1 1.6961,1.8964,2.1063,2.3220,2.5454,2.7747,3.0090,3.2474,3.4894,
     1 3.7344,3.9819,4.2315,4.4830,4.7361,4.9908,5.2468,5.5041,5.7627,
     1 6.022,6.282,6.543,6.803,7.06,7.33,7.59,7.85,8.11,8.38,8.64,8.90,
     1 9.17,9.43,9.69,9.96,10.2,10.5,10.8,11.0,11.3,11.5,11.8,12.1,12.3,
     1 12.6,12.9,13.1,13.4,13.7,13.9,14.2,14.5,14.7,15.0,15.3,15.5,15.8,
     1 16.1,16.3,16.6,16.9,17.1,17.4,17.7,17.9,18.2,18.4,18.7,19.0,19.2,
     1 19.5,19.8,20.0,20.3,20.6,20.8,21.1,21.4,21.6,21.9,22.2,22.4,22.7,
     1 23.0,23.2,23.5,23.8,24.0,24.3,24.6,24.8,25.1,25.4,25.6,25.9,26.2,
     1 26.4,26.7/
      data rm/0.0,0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9,1.0,1.1,1.2,1.3,
     1 1.4,1.5,1.6,1.7,1.8,1.9,2.0,2.1,2.2,2.3,2.4,2.5,2.6,2.7,2.8,2.9,
     1 3.0,3.1,3.2,3.3,3.4,3.5,3.6,3.7,3.8,3.9,4.0,4.1,4.2,4.3,4.4,
     1 4.5,4.6,4.7,4.8,4.9,5.0,5.1,5.2,5.3,5.4,5.5,5.6,5.7,5.8,5.9,6.0,
     1 6.1,6.2,6.3,6.4,6.5,6.6,6.7,6.8,6.9,7.0,7.1,7.2,7.3,7.4,7.5,7.6,
     1 7.7,7.8,7.9,8.0,8.1,8.2,8.3,8.4,8.5,8.6,8.7,8.8,8.9,9.0,9.1,9.2,
     1 9.3,9.4,9.5,9.6,9.7,9.8,9.9,10.0/
      data th/0.0,0.08727,0.17453,0.26180,0.34907,0.43633,0.52360,
     1 0.61087,0.69813,0.78540,0.87266,0.95993,1.04720,1.13446,1.22173,
     1 1.30900,1.39626,1.48353,1.57080/
      rd=0.017453293
      kf=0
      tol=0.05
      pi=4.0*atan(1.0)
      x0=muR
      ang=abs(angle)*pi/180.0
      snt=sin(ang)
      afact=1.0
      tfact=1.0
      if (x0 .lt. 0.1 ) x0=0.1
      do 1 i=1,101
         if (abs(x0-rm(i+1)) .le. tol) goto 2
         if (x0 .gt. rm(i+1)) goto 1
         x1=rm(i-1)
         x2=rm(i)
         x3=rm(i+1)
         do 3 k=1,19
            y1=af(i-1,k)
            y2=af(i,k)
            y3=af(i+1,k)
            call TAB_3L(x1,x2,x3,y1,y2,y3,ak0,ak1,ak2)
3           as(k)=exp(ak0+ak1*x0+ak2*x0**2)
         goto 9
1     continue
      return
2     do 6 k=1,19
6        as(k)=af(i+1,k)
9     sas=0.0
      do 200 i=1,19
         sas=sas+as(i)
200      thd=th(i)/rd
      sa=sas/19
      th(1)=0.043633
      as0=as(1)
      as(1)=(as(1)+as(2))/2.0
      t0=asin(snt)
      st=snt
C-----Choose the standard theta values for interpolation
      do 10 i=1,19
         kk=i
         if (abs(t0-th(i)) .le. 0.0001) goto 15
         if (t0 .gt. th(2)) goto 11
         t1=th(1)
         t2=th(2)
         t3=th(3)
         t4=th(4)
         y1=as(1)
         y2=as(2)
         y3=as(3)
         y4=as(4)
         goto 14
11       if (t0 .lt. th(18)) goto 12
         t1=th(16)
         t2=th(17)
         t3=th(18)
         t4=th(19)
         y1=as(16)
         y2=as(17)
         y3=as(18)
         y4=as(19)
         goto 14
12       if (t0 .lt. th(i) .or. t0 .gt. th(i+1)) goto 10
         if (abs(t0-th(i)) .le. 0.0001) then
            kk=i
            goto 15
         endif
         if (abs(t0-th(i+1)) .le. 0.0001) then
            kk=i+1
            goto 15
         endif
         if (t0 .gt. th(2) .and. t0 .lt. th(3)) then
            t1=th(2)
            t2=th(3)
            t3=th(4)
            t4=th(5)
            y1=as(2)
            y2=as(3)
            y3=as(4)
            y4=as(5)
            goto 14
         endif
         t1=th(i-1)
         t2=th(i)
         t3=th(i+1)
         t4=th(i+2)
         y1=as(i-1)
         y2=as(i)
         y3=as(i+1)
         y4=as(i+2)
         goto 14
10    continue
14    call TAB_4L(t1,t2,t3,t4,y1,y2,y3,y4,al0,al1,al2,al3)
      st=st**2
      ac=(al0+al1*st+al2*st**2+al3*st**3)
      goto 16
15    ac=as(kk)
16    if (ac .gt. as0) ac = as0
      afact=ac/as0
      tfact=1.0/ac
      end
C
C----------------------------------------------------------------------C
C
      subroutine TAB_3L(x1,x2,x3,y1,y2,y3,ak0,ak1,ak2)
C-----This routine solves a system of three linear equations
      y1=alog(y1)
      y2=alog(y2)
      y3=alog(y3)
      aa=(x2*x3**2-x3*x2**2)
      bb=(x3*x1**2-x1*x3**2)
      cc=(x1*x2**2-x2*x1**2)
      d1=aa+bb+cc
      ak0=(y1*aa+y2*bb+y3*cc)/d1
      ak1=((y1-ak0)*x2**2-(y2-ak0)*x1**2)/cc
      ak2=(y1-ak0-ak1*x1)/x1**2
      end
C
C----------------------------------------------------------------------C
C
      subroutine TAB_4L(t1,t2,t3,t4,y1,y2,y3,y4,al0,al1,al2,al3)
C-----This routine solves a system of four linear equations
      real*8 x1,x2,x3,x4,a1,a2,a3,a4,a5,a6,b1,b2,b3,b4,
     1 d,dy1,dy2,dy3,dy4,al0,al1,al2,al3,dt1,dt2,dt3,dt4
      rd=0.017453293
      dy1=y1
      dy2=y2
      dy3=y3
      dy4=y4
      dt1=t1
      dt2=t2
      dt3=t3
      dt4=t4
      x1=dsin(dt1)**2
      x2=dsin(dt2)**2
      x3=dsin(dt3)**2
      x4=dsin(dt4)**2
      a1=x3**2*x4**2*(x4-x3)
      a2=x2**2*x4**2*(x4-x2)
      a3=x2**2*x3**2*(x3-x2)
      a4=x1**2*x4**2*(x4-x1)
      a5=x1**2*x3**2*(x3-x1)
      a6=x1**2*x2**2*(x2-x1)
      b1=x2*a1-x3*a2+x4*a3
      b2=x1*a1-x3*a4+x4*a5
      b3=x1*a2-x2*a4+x4*a6
      b4=x1*a3-x2*a5+x3*a6
      d=b1-b2+b3-b4
      al0=(dy1*b1-dy2*b2+dy3*b3-dy4*b4)/d
      al1=((dy2-al0)*a1-(dy3-al0)*a2+(dy4-al0)*a3)/b1
      al2=((dy3-al0-al1*x3)*x4**3-(dy4-al0-al1*x4)*x3**3)/a1
      al3=(dy2-al0-al1*x2-al2*x2**2)/x2**3
      end
C
C----------------------------------------------------------------------C
C
      integer function ZMAX()
C ---- returns the Z value for heaviest element found under data item
C      _chemical_formula_sum. A zero value is returned if data item is
C      absent, or if chemical formula in uninterpretable

      parameter (itypes=30)
      include 'ciftbx.cmn'
      real fnamtp(itypes)
      character*80 item,chemfm
      character namtyp(itypes)*8
      ZMAX=0
      item='_chemical_formula_sum'
      if(.not. char_(item,chemfm)) return
      call chform(chemfm,namtyp,fnamtp,natmtp)
      if(natmtp .le. 0) return
      do i=1,natmtp
         call ELINFO(namtyp(i),iel,atwt)
         if(iel .gt. ZMAX) ZMAX=iel
      enddo
      end
C
C---------------- GENERAL FUNCTIONS -----------------------------------C
C
      subroutine aformt(line,istart,length,value)
C
C ---- packs a character string of given field-position & width
C
      character*(*)value
      character*80 vbuf,buf1*2,cbuf1*2
      character*80 spec,line1*200
      character buf(80),line(148)
      equivalence (buf(1),vbuf)
      ifail=0
      if (length .eq. 0) return
      if (length .gt. 800) goto 2
      do 1 i=istart,istart+length-1
1        buf(i-istart+1)=line(i)
      write (spec,10)length
10    format ('(a',i2,')')
      read (vbuf,spec,err=2)value
      return
2     continue
      write (buf1,'(i2)')istart-1
      read (buf1,'(a)')cbuf1
      do i=1,144
         line1(i:i)=line(i+1)
      enddo
      call notice('Error from AFORMT - cannot decode following line')
      call notice(line1(1:80))
      stop
      end
C
C----------------------------------------------------------------------C
C
      subroutine chform(chemfm,namtyp,fnamtp,natmtp)
C
C===== Conversion of a string record of the chemical formula into the
C     component sums of elements. Adapted from a subroutine by B.Kilger
C===== 07/83 modified by R.Sievers 9.85 and 7.88 and L.J. Farrugia 5.96
C
C    definition of arguments:
C    chemfm:  input string (character*(80)) with the structural
C     formula
C    namtyp:  character*8-array (length: 30) with natmtp found
C     element symbols
C    nelist:  return of element numbers, alphabetical
C    fnamtp:  return of corresponding indices
C    30:      maximum length of arrays namtyp, nelist and fnamtp
C    natmtp:    > 0 - number of found elements
C               < 0 - Error. :  -1 : syntax error
C                               -2 : illegal special symbol
C                               -3 : unrecognised element
C                               -4 : unmatched brackets
C                               -5 : conversion error
C                               -6 : too many different elements
C
      character*80 buf,chemfm
      character*120 buf1
      character*8 namtyp(30)
      character*1 idmxx
      character*1 nb,flank,nc2,nz
      character*2 nel4
      character*2 nele(106)
      character*2 ssymb(7)
      logical integ
C
      real f,ftot,fsav
      real fak(20),fnamtp(30)
C
      integer iendfl,ilpar,irpar,idig,iele,idezpt,istp,ipnt
      integer izifer,nv,nh,nr,i,jbeg,jl,ja,nesav,jend,klein
      integer ji,kleinp,j,ibuf
      integer nelist(30)
C
C----Listing order for the elements. The following 6 elements
C    appear in the indicated order, all other elements will
C    appear in alphabetical order : C H O N S P
C
      integer ielord(6)
      integer icind(7)
      data ielord/15,38,65,57,82,67/
C
      data nele/'AC','AG','AL','AM','AR','AS','AT','AU','B ','BA',
C                1    2    3    4    5    6    7    8     9    10
     1 'BE','BI','BK','BR','C ','CA','CD','CE','CF','CL','CM','CO',
C       11   12   13   15    15   16   17   18   19   20   21   22
     1 'CR','CS','CU','D ','DY','ER','ES','EU','F ','FE','FM','FR',
C       23   24   25    26   27    28   29   30    31   32   33   34
     1 'GA','GD','GE','H ','HE','HF','HG','HO','I ','IN','IR',
C       35   36   37    38   39   40   41   42    43   44   45
     1 'K ','KR','KU','LA','LI','LU','LW','MD','MG','MN','MO',
C       46   47   48    49   50   51   52    53   54   55   56
     1 'N ','NA','NB','ND','NE','NI','NO','NP','O ','OS',
C       57   58   59   60   61   62   63   64    65   66
     1 'P ','PA','PB','PD','PM','PO','PR','PT','PU','RA','RB','RE',
C       67   68   69   70   71   72   73   74   75    76   77   78
     1 'RH','RN','RU','S ','SB','SC','SE','SI','SM','SN','SR','T',
C       79   80   81    82   83   84   85   86   87   88   89   90
     1 'TA','TB','TC','TE','TH','TI','TL','TM','U ','V ','W ',
C       91   92   93   94   95   96   97   98    99   100    101
     1 'XE','Y ','YB','ZN','ZR'/
C       102   103  104   105  106
C
      data nb/' '/
      data flank/' '/
      data nel4/' '/
      data nc2/' '/
      data idmxx/' '/
C
C ---- new feature - special symbols allowed in formulae
C ---- Ph = phenyl; Me = methyl, Et = ethyl, Pp = propyl, Bu = butyl
C ---- Cy = cyclohexyl, Cp = cyclopentadienyl
      data ssymb/'PH','ME','ET','PP','BU','CY','CP'/
C
      natmtp=0
      f=1.0
      iendfl=0
      ilpar=0
      irpar=0
      idig=0
      iele=0
      idezpt=0
      istp=0
      ipnt=81
      nc2='0'
C
C --- first check for special symbols & remove unecessary blanks from string
C
      do i=1,80
        if(chemfm(i:i) .eq. ',') chemfm(i:i)=' '
      enddo
      call trimstr(chemfm)
      call upcase(chemfm)
52    buf1=' '
      do i=1,7
        icind(i)=index(chemfm,ssymb(i))
      enddo
      do i=1,7
        if(icind(i) .ne. 0) goto 50
      enddo
      goto 51
50    iind=i
      i1=icind(iind) - 1
      if (i1 .ne. 0 ) then

      if (iind .eq. 1) then
        buf1=chemfm(1:i1)//'(C6 H5)'//chemfm(i1+3:leng(chemfm))
      elseif (iind .eq. 2) then
        buf1=chemfm(1:i1)//'(C H3)'//chemfm(i1+3:leng(chemfm))
      elseif (iind .eq. 3) then
        buf1=chemfm(1:i1)//'(C2 H5)'//chemfm(i1+3:leng(chemfm))
      elseif (iind .eq. 4) then
        buf1=chemfm(1:i1)//'(C3 H7)'//chemfm(i1+3:leng(chemfm))
      elseif (iind .eq. 5) then
        buf1=chemfm(1:i1)//'(C4 H9)'//chemfm(i1+3:leng(chemfm))
      elseif (iind .eq. 6) then
        buf1=chemfm(1:i1)//'(C6 H11)'//chemfm(i1+3:leng(chemfm))
      elseif (iind .eq. 7) then
        buf1=chemfm(1:i1)//'(C5 H5)'//chemfm(i1+3:leng(chemfm))
      endif

      elseif (i1 .eq. 0) then

        if (iind .eq. 1) then
          buf1='(C6 H5)'//chemfm(i1+3:leng(chemfm))
        elseif (iind .eq. 2) then
          buf1='(C H3)'//chemfm(i1+3:leng(chemfm))
        elseif (iind .eq. 3) then
          buf1='(C2 H5)'//chemfm(i1+3:leng(chemfm))
        elseif (iind .eq. 4) then
          buf1='(C3 H7)'//chemfm(i1+3:leng(chemfm))
        elseif (iind .eq. 5) then
          buf1='(C4 H9)'//chemfm(i1+3:leng(chemfm))
        elseif (iind .eq. 6) then
          buf1='(C6 H11)'//chemfm(i1+3:leng(chemfm))
        elseif (iind .eq. 7) then
         buf1='(C5 H5)'//chemfm(i1+3:leng(chemfm))
        endif

      endif
      if(leng(buf1) .gt. 80) then
         call alert('A','_chemical_formula_sum too long - max 80 chars')
         chemfm=buf1(1:80)
         natmtp=-7
         return
      endif
      chemfm=buf1(1:80)
      goto 52

51    ibuf=0
      buf=' '
      do i=1,80
         if (i .eq. 80 .and. chemfm(i:i) .ne. ' ') then
            ibuf=ibuf+1
            buf(ibuf:ibuf)=chemfm(i:i)
            goto 1
         endif
         if (i .eq. 80) goto 1
         if (chemfm(i:i) .eq. ' ' .and. integ(chemfm(i+1:i+1)))
     1   goto 1
         if (chemfm(i:i) .eq. ' ' .and. chemfm(i+1:i+1) .eq. ' ')
     1   goto 1
         ibuf=ibuf+1
         buf(ibuf:ibuf)=chemfm(i:i)
1        continue
      enddo
      chemfm=buf
C
C
C     ***  read input string from right to left  ***
100   ipnt=ipnt-1
      if (ipnt .lt. 1) goto 7000
      nb(1:1)=chemfm(ipnt:ipnt)
      nz(1:1)=chemfm(ipnt:ipnt)
C
C             A...Z     a...z     0...9   blank  special char.
C    -----------------------------------------------------
C    EBCDIC 193..233  129...169 240..249    64    65..128
C    ASCII   65...90   97...122  48...57    32    33...47
C
C----lower- or uppercase character or integer?
      if (lge(nz,'0')) goto 200
C
C----special character ?
      if (nz .eq. ' ') goto 5000
      if (nz .eq. '(') goto 3000
      if (nz .eq. ')') goto 4000
      if (nz .eq. '.') goto 4500
      goto 9902
C
C---- lower- or uppercase character or integer
C     (uppercase character ...)
200   if (lle(nz,'Z') .and. lge(nz,'A')) goto 2000
      if (lle(nz,'9') .and. lge(nz,'0')) goto 1000
C     (lowercase character ...)
      if (lgt(nz,'z') .or. llt(nz,'a')) goto 9902
C     lower -> upper case conversion
      if (lge(nb,'a') .and. lle(nb,'z')) then
         nb=char(ichar(nb)-ichar('a')+ichar('A'))
      endif
      goto 2000
C
C     ***  integer  ***
1000  if (idig .gt. 0) goto 1100
      if (iele .gt. 0 .or. ilpar .gt. 0) goto 9901
      if (idezpt .le. 0)f=0.0
C
1100  izifer=ichar(nz)-ichar('0')
      f=f+float(izifer)*10.0**(idig-idezpt)
      idig=idig+1
      irpar=0
      goto 100
C
C     ***  character  ***
2000  if (ilpar .gt. 0) goto 9901
      irpar=0
      idig=0
      idezpt=0
      iele=iele+1
      goto (2020,2030,9903),iele
2020  nel4(1:1)=nb
      nel4(2:2)=flank
      goto 100
2030  idmxx=nel4(1:1)
      nel4(2:2)=idmxx
      nel4(1:1)=nb
      goto 100
C
C     ***  left bracket  ***
3000  if (idig .gt. 0 .or. irpar .gt. 0) goto 9901
      ilpar=1
      if (iele .gt. 0) goto 6000
3010  istp=istp-1
      if (istp .lt. 0) goto 9904
      goto 100
C
C     ***  right bracket  ***
4000  if (iele .gt. 0 .or. ilpar .gt. 0) goto 9901
      idig=0
      idezpt=0
      irpar=1
      istp=istp+1
      if (istp .gt. 24) goto 9904
      fak(istp)=f
      f=1.0
      goto 100
C
C     ***  decimal point  ***
4500  if (idezpt .gt. 0) goto 9905
      f=f*10.0**(-idig)
      idig=idig+1
      idezpt=idig
      goto 100
C
C     ***  blank  ***
5000  if (idig .gt. 0) goto 9901
      ilpar=0
      if (iele .lt. 1) goto 100
C
C **********************************************************************
6000  continue
C
      nv=1
      nh=106
      nr=65
      goto 25
10    nv=nr+1
20    if (nv .gt. nh) goto 30
      nr=(nv+nh)/2
25    if (nel4 .eq. nele(nr)) goto 40
      if (lgt(nel4,nele(nr))) goto 10
      nh=nr-1
      goto 20
C
30    nr=0
40    continue
C **********************************************************************
C
      if (nr .le. 0) goto 9903
      ftot=f
      if (istp .le. 0) goto 6050
      do 6020 i=1,istp
6020     ftot=ftot*fak(i)
6050  if (natmtp .le. 0) goto 6100
      do 6070 i=1,natmtp
         if (nr .ne. nelist(i)) goto 6070
         fnamtp(i)=fnamtp(i)+ftot
         goto 6150
6070  continue
6100  natmtp=natmtp+1
      if (natmtp .gt. 30) goto 9906
      fnamtp(natmtp)=ftot
      nelist(natmtp)=nr
6150  f=1.0
      iele=0
      if (ilpar .gt. 0) goto 3010
      if (iendfl .le. 0) goto 100
C
C     ***  end of scan from right to left  ***
7000  iendfl=1
      if (istp .ne. 0) goto 9904
      if (iele .gt. 0) goto 6000
C
C     *** ordering of elements  ***
      if (natmtp .le. 0) goto 9999
      jbeg=1
C
C----The specified elements are placed first.
C---- LJF change - order is now C,H,alphabetic

      do 7030 jl=1,2
C----search for element ielord(jl) in the formula:
         do 7010 ja=jbeg,natmtp
            if (nelist(ja) .eq. ielord(jl)) goto 7020
7010     continue
C---- not in the formula
         goto 7030
C
7020     nesav=nelist(ja)
         nelist(ja)=nelist(jbeg)
         nelist(jbeg)=nesav
         fsav=fnamtp(ja)
         fnamtp(ja)=fnamtp(jbeg)
         fnamtp(jbeg)=fsav
C
         jbeg=jbeg+1
         if (jbeg .eq. natmtp) goto 9000
7030  continue
C
      jend=natmtp-1
      do 7100 ja=jbeg,jend
         klein=999
         do 7060 ji=ja,natmtp
            if (nelist(ji) .ge. klein) goto 7060
            klein=nelist(ji)
            kleinp=ji
7060     continue
         fsav=fnamtp(kleinp)
         fnamtp(kleinp)=fnamtp(ja)
         fnamtp(ja)=fsav
         nesav=nelist(kleinp)
         nelist(kleinp)=nelist(ja)
         nelist(ja)=nesav
7100  continue
C
9000  if (natmtp .gt. 0) then
         do 9020 j=1,natmtp
            namtyp(j)=nele(nelist(j))
C
C           2. convert to lowercase character
            if (namtyp(j)(2:2) .ne. ' ') then
               namtyp(j)(2:2)=char(ichar(namtyp(j)(2:2))-
     1         ichar('A')+ichar('a'))
            endif
9020     continue
      endif
C
      goto 9999
C
C     ***  general syntax error  ***
9901  natmtp=-1
      goto 9999
C     ***  illegal special symbol  ***
9902  natmtp=-2
      goto 9999
C     ***  illegal element symbol  ***
9903  natmtp=-3
      goto 9999
C     ***  brackets not matched  ***
9904  natmtp=-4
      goto 9999
C     ***  convertion error  ***
9905  natmtp=-5
      goto 9999
C     ***  too many elements in formula  ***
9906  natmtp=-6
C
9999  continue
      return
      end
C
C----------------------------------------------------------------------C
C
      character*(*) function contstr(string)
C ---- removes embedded blanks from "string"
      character*(*) string
      character*1000 buf
      ilen=len(string)
      if (ilen .gt. 1000) ilen=1000
      icount=0
      do i=1,ilen
        if(string(i:i) .ne. ' ') then
          icount=icount+1
          buf(icount:icount)=string(i:i)
        endif
      enddo
      contstr=' '
      if(icount .gt. 0) contstr=buf(1:icount)
      end
C
C---------------------------------------------------------------------C
C
      subroutine fformt(line,istart,length,value)
C
C     decodes a floating-point number of given field-position & width
C
      character line(148),buf(20),buf1*2,cbuf1*2
      character*20 vbuf,line1*200
      character*80 spec
      equivalence (buf(1),vbuf)
      ifail=0
      imessage=0
      if (length .eq. 0) return
      do 1 i=istart,istart+length-1
1        buf(i-istart+1)=line(i)
      width=length
      write (spec,10)width
10    format ('(f',f4.1,')')
      read (vbuf,spec,err=2)value
      return
2     continue
      write (buf1,'(i2)')istart-1
      read (buf1,'(a)')cbuf1
      do i=1,144
         line1(i:i)=line(i+1)
      enddo
      call notice('Error from FFORMT - cannot decode following line')
      call notice(line1(1:80))
      stop
      end
C
C----------------------------------------------------------------------C
C
      character*(*) function filepath(string)
      character*(*) string
      character*80 dir
      character delim

      delim='/'

      call  getenv('IUCRDIR',dir)
      if(dir .eq. ' ') then
         filepath=string
      else
         ilen=leng(dir)
         if(dir(ilen:ilen) .ne. delim) dir(ilen+1:ilen+1)=delim
         filepath=dir(1:leng(dir))//string
      endif
      end
C
C----------------------------------------------------------------------C
C
      subroutine iformt(line,istart,length,ivalue)
C
C     decodes an integer of given field-position & width
C
      character line(148),buf(20),buf1*2,cbuf1*2
      character*20 vbuf,line1*200
      character*80 spec
      equivalence (buf(1),vbuf)
      ifail=0
      imessage=0
      if (length .eq. 0) return
      do 1 i=istart,istart+length-1
1        buf(i-istart+1)=line(i)
      write (spec,10)length
10    format ('(i',i2,')')
      read (vbuf,spec,err=2)ivalue
      return
2     continue
      write (buf1,'(i2)')istart-1
      read (buf1,'(a)')cbuf1
      do i=1,144
         line1(i:i)=line(i+1)
      enddo
      call notice('Error from IFORMT - cannot decode following line')
      call notice(line1(1:80))
      stop
      end
C
C----------------------------------------------------------------------C
C
      integer function leng(string)
C ---- returns significant length of  'string'
      character*(*)string
      do 1 leng=len(string),0,-1
1        if (string(leng:leng) .ne. ' ') goto 2
2     end
C
C----------------------------------------------------------------------C
C
      subroutine locase(string)
C ---- sets 'string' to lower case
      character*(*)string
      ilen=len(string)
      do 1 j=1,ilen
         if (lge(string(j:j),'A') .and. lle(string(j:j),'Z'))
     1   string(j:j)=char(ichar(string(j:j))-ichar('A')+ichar('a'))
1     continue
      end
C
C----------------------------------------------------------------------C
C
      logical function meqrt(r1,r2)
C ---- tests whether the RT matrices r1 and r2 are equal
      real r1(3,4),r2(3,4),tol
      tol=1.e-6
      meqrt=.true.
      do i=1,3
         do j=1,4
            if (abs(r1(i,j) - r2(i,j)) .ge. tol) meqrt=.false.
         enddo
      enddo
      end
C
C----------------------------------------------------------------------C
C
      logical function meq1(r1,r2)
C ---- tests whether one rotation matrix is inverse of other
      real r1(3,3),r2(3,3),tol
      tol=1.e-6
      meq1=.true.
      do i=1,3
         do j=1,3
            if (abs(r1(i,j)+r2(i,j)) .ge. tol) meq1=.false.
         enddo
      enddo
      end
C
C----------------------------------------------------------------------C
C
      logical function modt1(t1,t2)
C  ---- tests whether two vectors sum to zero (modulo 1)
      real t1(3),t2(3),tol
      tol=1.e-6
      modt1=.true.
      do i=1,3
         if (abs(mod((t1(i)+t2(i)),1.0)) .ge. tol)modt1=.false.
      enddo
      end
C
C----------------------------------------------------------------------C
C
      subroutine parse(line,j,npos,length)
C
C     given a line of input, returns number of fields,
C     beginning and length of each field
C
      character line(148)
      dimension npos(20),length(20)
      j=0
      i=1
1     i=i+1
      if (i .gt. 145) return
      if (line(i) .eq. ' ') goto 1
      j=j+1
      if (j .gt. 20) then
         j=20
         return
      endif
      npos(j)=i
      if (line(i) .eq. ',') goto 3
2     i=i+1
      if (i .gt. 145) goto 3
      if (line(i) .ne. ' ' .and. line
     1        (i) .ne. ',') goto 2
3     length(j)=min(40,i-npos(j))
      goto 1
      end
C
C----------------------------------------------------------------------C
C
      SUBROUTINE QSORTA (ORD,N,A)
C
C====== SORTS THE CHARACTER ARRAY A(I),I=1,2,...,N BY PUTTING THE
C   ASCENDING ORDER VECTOR IN ORD.  THAT IS ASCENDING ORDERED A
C   IS A(ORD(I)),I=1,2,...,N; DESCENDING ORDER A IS A(ORD(N-I+1)),
C   I=1,2,...,N .  THIS SORT RUNS IN TIME PROPORTIONAL TO N LOG N .
C   ARRAY IS SORTED ON THE BASIS OF THE FIRST 80 CHARACTERS, WHICH
C   SHOULD BE SUFFICIENT FOR MOST PURPOSES - CHANGE PARAMETER NDIM
C   IF NECESSARY

      IMPLICIT INTEGER (A-Z)
C
      DIMENSION ORD(N),POPLST(2,20)
      CHARACTER*(80) X,XX,Z,ZZ,Y
C
C
      CHARACTER*(*) A(N)
C
      NDEEP=0
      U1=N
      L1=1
      DO 1  I=1,N
    1 ORD(I)=I
    2 IF (U1.LE.L1) RETURN
C
    3 L=L1
      U=U1
C
C PART
C
    4 P=L
      Q=U
      X = A(ORD(P))
      Z = A(ORD(Q))
      IF (X .LE. Z) GO TO 5
      Y=X
      X=Z
      Z=Y
      YP=ORD(P)
      ORD(P)=ORD(Q)
      ORD(Q)=YP
    5 IF (U-L.LE.1) GO TO 15
      XX=X
      IX=P
      ZZ=Z
      IZ=Q
C
C LEFT
C
    6 P=P+1
      IF (P.GE.Q) GO TO 7
      X=A(ORD(P))
      IF (X.GE.XX) GO TO 8
      GO TO 6
    7 P=Q-1
      GO TO 13
C
C RIGHT
C
    8 Q=Q-1
      IF (Q.LE.P) GO TO 9
      Z=A(ORD(Q))
      IF (Z.LE.ZZ) GO TO 10
      GO TO 8
    9 Q=P
      P=P-1
      Z=X
      X=A(ORD(P))
C
C DIST
C
   10 IF (X.LE.Z) GO TO 11
      Y=X
      X=Z
      Z=Y
      IP=ORD(P)
      ORD(P)=ORD(Q)
      ORD(Q)=IP
   11 IF (X.LE.XX) GO TO 12
      XX=X
      IX=P
   12 IF (Z.GE.ZZ) GO TO 6
      ZZ=Z
      IZ=Q
      GO TO 6
C
C OUT
C
   13 CONTINUE
      IF (.NOT.(P.NE.IX.AND.X.NE.XX)) GO TO 14
      IP=ORD(P)
      ORD(P)=ORD(IX)
      ORD(IX)=IP
   14 CONTINUE
      IF (.NOT.(Q.NE.IZ.AND.Z.NE.ZZ)) GO TO 15
      IQ=ORD(Q)
      ORD(Q)=ORD(IZ)
      ORD(IZ)=IQ
   15 CONTINUE
      IF (U-Q.LE.P-L) GO TO 16
      L1=L
      U1=P-1
      L=Q+1
      GO TO 17
   16 U1=U
      L1=Q+1
      U=P-1
   17 CONTINUE
      IF (U1.LE.L1) GO TO 18
C
C START RECURSIVE CALL
C
      NDEEP=NDEEP+1
      POPLST(1,NDEEP)=U
      POPLST(2,NDEEP)=L
      GO TO 3
   18 IF (U.GT.L) GO TO 4
C
C POP BACK UP IN THE RECURSION LIST
C
      IF (NDEEP.EQ.0) GO TO 2
      U=POPLST(1,NDEEP)
      L=POPLST(2,NDEEP)
      NDEEP=NDEEP-1
      GO TO 18
      END
C
C----------------------------------------------------------------------C
C
      subroutine readbuf(buf,value,ival)
C ---- returns the real value in the ival'th field of buf
      character*(*) buf
      character line(148)
      real value
      integer fch(20),fwd(20)
      value=0.0
      do i=1,148
        line(i)=' '
      enddo
      ilen=leng(buf)
      if(ilen .gt. 145) ilen=145
      do i=1,ilen
        line(i+1)=buf(i:i)
      enddo
      call parse(line,nfields,fch,fwd)
      if(nfields .ge. ival) call fformt(line,fch(ival),fwd(ival),value)
      end
C
C----------------------------------------------------------------------C
C
      real function rnormx(z)
C ---- normalises x so that 0.0 =< x < 1.0
      x=z
1     if(x .lt. 0.0) then
        x = x + 1.0
      goto 1
      endif
2     if(x .ge. 1.0) then
        x = x - 1.0
        goto 2
      endif
      rnormx=x
      end
C
C----------------------------------------------------------------------C
C
      subroutine sortchar(f,if)
C --- returns a sorted character array F of dimension IF
C     up to array dimension NDIM
      parameter (ndim=100)
      character*(*) f(if)
      character*80 new_f(ndim)
      integer*4 ip(ndim)
      if(if .lt. 1) return
      if(if .gt. ndim) return
      call qsorta(ip,if,f)
      do i=1,if
        new_f(i) = f(ip(i))
      enddo
      do i=1,if
        f(i) = new_f(i)
      enddo
      end
C
C----------------------------------------------------------------------C
C
      subroutine trimstr(string)
C ---- remove leading blanks from 'string' 
      character*(*)string
      integer i,ilen,iblank
      ilen=len(string)
      iblank=0
      do 1 i=1,ilen
         if (string(i:i) .ne. ' ') goto 2
1        if (string(i:i) .eq. ' ')iblank=iblank+1
2     if (iblank .ne. 0) then
         do 3 i=1,ilen-iblank
3           string(i:i)=string(i+iblank:i+iblank)
         string(ilen-iblank+1:ilen)=' '
      endif
      end
C
C----------------------------------------------------------------------C
C
      subroutine upcase(string)
C ---- sets 'string' to upper case
      character*(*)string
      ilen=len(string)
      do 1 j=1,ilen
         if (lge(string(j:j),'a') .and. lle(string(j:j),'z'))
     1   string(j:j)=char(ichar(string(j:j))-ichar('a')+ichar('A'))
1     continue
      end
C
C--------------------- SPACE GROUP ROUTINES -------------------------------C
C
      subroutine getsginfo(hms,ispgr,isym,cspg,Halls,Scfl,cset,csym,
     1 ncsym,csyma,ncsyma,cspos,nsp,clatt,claue,cpoint,cl,cc,t,nt,mode)
      character*80 hms,hall
      character*17 Halls
      character*10 cspg
      character*12 clatt,claue,cpoint
      character*6 cset
      character*8 Scfl
      character*40 csym(49),csyma(49)
      character*56 cspos(100)
      character cl,cc
      integer mode
      real t(3,4)
      ispgr=0
      cspg=' '
      hall=' '
      Halls=' '
      Scfl=' '
      cset=' '
      do i=1,49
        csym(i)=' '
        csyma(i)=' '
      enddo
      ncsym=0
      ncsyma=0
      do i=1,100
        cspos(i)=' '
      enddo
      nsp=0
      clatt=' '
      claue=' '
      cpoint=' '
      call symmpos(hms,ispgr,cspg,isym,Halls,Scfl,cset)
      hall = Halls
      if(ispgr .le. 0) return
      call getspec(mode,hall,csym,ncsym,csyma,ncsyma,cspos,nsp,cl,cc,t,
     1             nt)
      clatt='Triclinic'
      claue='-1'
      cpoint='1'
      if(ispgr .gt. 1) cpoint='-1'
      if(ispgr .ge. 3 .and. ispgr .le. 15) then
        clatt='Monoclinic'
        claue='2/m'
        cpoint='2'
        if(ispgr .gt. 5) cpoint='m'
        if(ispgr .gt. 9) cpoint='2/m'
      elseif(ispgr .ge. 16 .and. ispgr .le. 74) then
        clatt='Orthorhombic'
        claue='mmm'
        cpoint='222'
        if(ispgr .gt. 24) cpoint='mm2'
        if(ispgr .gt. 46) cpoint='mmm'
      elseif(ispgr .ge. 75 .and. ispgr .le. 142) then
        clatt='Tetragonal'
        claue='4/m'
        cpoint='4'
        if(ispgr .gt. 88) claue='4/mmm'
        if(ispgr .gt. 80) cpoint='-4'
        if(ispgr .gt. 82) cpoint='4/m'
        if(ispgr .gt. 88) cpoint='422'
        if(ispgr .gt. 98) cpoint='4mm'
        if(ispgr .gt. 110) cpoint ='-42m'
        if(ispgr .gt. 120) cpoint='4/mmm'
      elseif(ispgr .ge. 143 .and. ispgr .le. 167) then
        clatt='Trigonal'
        claue='-3'
        cpoint='3'
      elseif(ispgr .ge. 168 .and. ispgr. le. 194) then
        clatt = 'Hexagonal'
        claue='6/m'
        cpoint='6'
        if(ispgr .gt. 176) claue='6/mmm'
        if(ispgr .gt. 173) cpoint='-6'
        if(ispgr .eq. 174) cpoint='-6'
        if(ispgr .gt. 174) cpoint='622'
        if(ispgr .gt. 182) cpoint='6mm'
        if(ispgr .gt. 186) cpoint='-6m2'
        if(ispgr .gt. 190) cpoint='6/mmm'
      elseif(ispgr .ge. 195) then
        clatt='Cubic'
        claue='m-3'
        if(ispgr .gt. 206) claue='m-3m'
        cpoint='23'
        if(ispgr .gt. 199) cpoint='m-3'
        if(ispgr .gt. 206) cpoint='432'
        if(ispgr .gt. 214) cpoint='-43m'
        if(ispgr .gt. 220) cpoint='m-3m'
      endif
      end
C
C----------------------------------------------------------------------C
C
      subroutine getspos(hms,ispgr,csym,ncsym)
C ---- stub routine to get SHELX "SYMM" and "LATT" cards
      character*80 hms
      character*17 Halls
      character*10 cspg
      character*12 clatt,claue,cpoint
      character*6 cset
      character*8 Scfl
      character*40 csym(49),csyma(49)
      character*56 cspos(100)
      character cl,cc
      integer mode
      real t(3,4)
      mode=0
      call getsginfo(hms,ispgr,isym,cspg,Halls,Scfl,cset,csym,
     1 ncsym,csyma,ncsyma,cspos,nsp,clatt,claue,cpoint,cl,cc,t,nt,mode)
      end
C
C----------------------------------------------------------------------C
C
      subroutine getspmat(hms,ispgr,nsymz,nltz,sz,tz,tuz)
C ---- stub routine to get matrices
      common /symnew/    nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      character*80 hms
      character*17 Halls
      character*10 cspg
      character*12 clatt,claue,cpoint
      character*6 cset
      character*8 Scfl
      character*40 csym(49),csyma(49)
      character*56 cspos(100)
      character cl,cc
      integer mode
      real tx(3,4),sz(3,3,48),tz(3,48),tuz(3,4)
      integer nsymz,nltz
      mode=0
      call getsginfo(hms,ispgr,isym,cspg,Halls,Scfl,cset,csym,
     1 ncsym,csyma,ncsyma,cspos,nsp,clatt,claue,cpoint,cl,cc,tx,nt,mode)
      nsymz=nsym
      nltz=nlt
      do i=1,3
        do j=1,4
          tuz(i,j)=tu(i,j)
        enddo
      enddo
      do i=1,48
        do j=1,3
          tz(j,i)=t(j,i)
          do k=1,3
            sz(k,j,i) = s(k,j,i)
          enddo
        enddo
      enddo
      end
C
C----------------------------------------------------------------------C
C
      subroutine getspmat1(hms,ispgr,nltz)
C ---- stub routine to get matrices
      common /symnew/    nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      character*80 hms
      character*17 Halls
      character*10 cspg
      character*12 clatt,claue,cpoint
      character*6 cset
      character*8 Scfl
      character*40 csym(49),csyma(49)
      character*56 cspos(100)
      character cl,cc
      integer mode
      real tz(3,4)
      integer nltz
      mode=0
      call getsginfo(hms,ispgr,isym,cspg,Halls,Scfl,cset,csym,
     1 ncsym,csyma,ncsyma,cspos,nsp,clatt,claue,cpoint,cl,cc,tz,nt,mode)
      nltz=nlt
      end
C
C----------------------------------------------------------------------C
C
      subroutine getspec(mode,Halls,csym,ncsym,csyma,ncsyma,cspos,nspos,
     1                   cl,cc,tl,n_lt)
C
C   GETSPEC calculates symmetry operators and special positions for any
C   given space group from the HALL space group symbol (which can be
C   obtained from the HERMANN-MAUGUIN symbol stored in the file SPGR.DAT).
C   The program is described by Altermatt and Brown in Acta Cryst. (1987)
C   A34, 125-130.
C
C
C    WRITTEN BY DANIEL ALTERMATT & I. D. BROWN
C    MCMASTER UNIVERSITY, HAMILTON, ONTARIO, CANADA
C
C    L.J. Farrugia - NOTE this is a very cut-down version of GETSPEC
C
      implicit character*8 (g)
      implicit character*80 (h)
      integer mode
      character*40 csym(49),csyma(49)
      character*56 cspos(100)
      character   cl,cc
      real tl(3,4)
      INTEGER NCSYM
C
C     system common blocks
C
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp
      common /symnew/    nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      common /gsym/   glat,gcent,hmsym,hall,gsysp(2,30)
      common /syma/   mult(48,48),it(3,48),nsp,msp,isptab(48,30),
     1                multsp(30),sp(3,3,30),tp(3,30)
C
C
      time = 0.0
      NIN = 5
      NOUT = 6
      NDEBUG = 0
      NREAD = NIN
      NWRITE = 9
      NSPGR = 10
      NSYM = 0
      MSYM = 48
      NT = 1
      DO I=1,4
        DO J=1,3
          TU(J,I) = 0.0
        ENDDO
      ENDDO
C
      GLAT = ' '
      GCENT = ' '
      HMSYM = ' '
      HALL = Halls
C
      NSP = 0
      MSP = 30
      call upcase(hall)
      call trimstr(hall)
      CALL SGHALL(CSYM,NCSYM)
      IER = 0
      CALL GRMULT(IER)
      IF (IER.NE.0) RETURN
      DO I=1,NSYM
         CALL CONXYZ(S(1,1,I),T(1,I),HBUF)
         CALL UPCASE(HBUF)
         WRITE(CSYMA(I),2120) HBUF
 2120    FORMAT(A30)
      ENDDO
      NCSYMA=NSYM
      CL=GLAT
      CC=GCENT
      DO I=1,3
         DO J=1,4
           TL(I,J)=TU(I,J)
         ENDDO
      ENDDO
      N_LT=NLT
      NSPOS=NSP
      END
C
C----------------------------------------------------------------------C
C
      SUBROUTINE SGHALL(CSYM,NCSYM)
*
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
      CHARACTER csym(49)*40
      INTEGER NCSYM
C
      INTEGER OLD
      INTEGER ROT(12,20)
      INTEGER SMAT(16,50)
      INTEGER SVEC(3)
      DIMENSION ISHIFT(3),GCHR(30)
      INTEGER TAB(3,9)
      INTEGER VEC(3,8)
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP
      COMMON /symnew/ NSYM,MSYM,S(3,3,48),T(3,48),NLT,TU(3,4)
      common /gsym/   glat,gcent,hmsym,hall,gsysp(2,30)
      DATA ROT/
     1 1,0,0,0, 0,1,0,0, 0,0,1,0,
     2 1,0,0,0, 0,-1,0,0, 0,0,-1,0,
     3 -1,0,0,0, 0,1,0,0, 0,0,-1,0,
     4 -1,0,0,0, 0,-1,0,0, 0,0,1,0,
     5 1,0,0,0, 0,0,1,0, 0,-1,-1,0,
     6 -1,0,-1,0, 0,1,0,0, 1,0,0,0,
     7 0,1,0,0, -1,-1,0,0, 0,0,1,0,
     8 1,0,0,0, 0,0,1,0, 0,-1,0,0,
     9 0,0,-1,0, 0,1,0,0, 1,0,0,0,
     1 0,1,0,0, -1,0,0,0, 0,0,1,0,
     2 1,0,0,0, 0,1,1,0, 0,-1,0,0,
     3 0,0,-1,0, 0,1,0,0, 1,0,1,0,
     4 1,1,0,0, -1,0,0,0, 0,0,1,0,
     5 0,0,1,0, 1,0,0,0, 0,1,0,0,
     6 -1,0,0,0, 0,0,-1,0, 0,-1,0,0,
     7 0,0,-1,0, 0,-1,0,0, -1,0,0,0,
     8 0,-1,0,0, -1,0,0,0, 0,0,-1,0,
     9 -1,0,0,0, 0,0,1,0, 0,1,0,0,
     1 0,0,1,0, 0,-1,0,0, 1,0,0,0,  0,1,0,0, 1,0,0,0, 0,0,-1,0/
*
      DATA GCHR/'1','2','3','4','5','6','P','R','A','B','C','I','F',
     1'N','U','V','W','D',' ','-','X','Y','Z','*','''','"','(','0','H',
     2'H'/
*
      DATA TAB/1,1,1,2,3,4,5,6,7,8,9,10,0,0,0,11,12,13,                 
     1  14,14,14,15,16,17,18,19,20/
*
      DATA VEC/6,0,0, 0,6,0, 0,0,6, 6,6,6,                              
     1  3,0,0, 0,3,0, 0,0,3, 3,3,3/
*
* VARIABLE INITIALIZATION.
*
      JCENT=0
      GCENT=GCHR(14) 
      GLAT=' '
      NGEN=1
      NSYM=1
      ISGN=12
      NLT=1
*
      NROT=0
      LLTCODE=0
      IROT=0
      JROT=0
      IAXS=0
      JAXS=0
      DO 1 I=1,3
      SVEC(I)=0
      TU(I,NLT)=0.0
    1 CONTINUE
*
* SET THE (4,4) ELEMENT TO 1.0.
*
      DO I=1,50
        DO J=1,16
          SMAT(J,I)=0
        ENDDO
      ENDDO
      DO 2 I=1,50
      SMAT(16,I)=12 
    2 CONTINUE
*
* SET ORIGIN SHIFT TO ZERO
*
      DO 30 I=1,3
   30   ISHIFT(I)=0 
*
* SET IDENTITY ROTATIONS.
*
      DO 3 I=1,12
      SMAT(I,1)=ROT(I,1)*12
    3 CONTINUE
*
* SET IDENTITY TRANSLATIONS.
*
      DO 4 I=13,16
      SMAT(I,1)=ROT(I-10,1)*12
    4 CONTINUE
*
* CHECKING ELEMENT BY ELEMENT AND SET VARIABLES AS NEEDED.
*
      DO 66 N=1,15
      DO 6 M=1,29
      IF (HALL(N:N).EQ.GCHR(M)) GO TO 606
    6 CONTINUE
*
* ERROR CONDITION CHARACTER NOT IDENTIFIED.
*
      WRITE(NOUT,1670) HALL(N:N),N
 1670 FORMAT(' ++ERROR++SGHALL++1670++ ILLEGAL CHARACTER: ',A1,
     1       ' IN COLUMN ',I1)
*
* SKIP INACTIVE BLANKS.
*
  606 IF ((M.EQ.19).AND.(NROT.EQ.0)) GO TO 66
C
C  BLANK INITIATES CONSTRUCTION OF SYMMETRY GENERATOR
C
      IF ( M.EQ.19 ) GO TO 1011
C
C PARENTHESIS INITIATES READING OF ORIGIN SHIFT
C
      IF(M.EQ.27) GO TO 40
*
*
* SET GCENT AND GLAT, DEPENDENT ON FIRST ELEMENT(S).
*
      IF ( LLTCODE .NE. 0) GO TO 1000
*
* FIRST NEGATIVE SIGN INDICATES CENTRE. 
*
      IF (M .NE. 20) GO TO 1001
      JCENT=1
      GCENT=GCHR(11)
      GO TO 66
*
* ALLOWED CHARACTERS: P,R,A,B,C,I,F TO SET LTCODE.
*
 1001 IF ((M .GE. 7).AND.(M .LE.13)) GO TO 2000
      WRITE(NOUT,1600) GCHR(M)
 1600 FORMAT(' ++ERROR++SGHALL++1600++ UNRECOGNISED LATTICE TYPE ',
     1 A1)
      GO TO 150
 2000 CONTINUE
      LLTCODE=M-6
      GLAT=GCHR(M)
*
* SPACE GROUPS WITH NAMES 'R' TYPE ARE ASSIGNED
* GLAT 'H' FOR A HEXAGONAL SETTING. THIS IS A
* LOCAL CONVENTION. 
*
      IF ( M .EQ. 8 ) GLAT=GCHR(30)
*
* ADD CENTERING TRANSLATIONS TO =TU=
*
      IF(GLAT.EQ.'P') GOTO 660
      IF(GLAT.EQ.'I') THEN
        NLT=NLT+1
        DO 60 I=1,3
   60   TU(I,NLT)=0.5
        GOTO 660
        ENDIF
      IF(GLAT.EQ.'A'.OR.GLAT.EQ.'F') THEN
        NLT=NLT+1
        TU(1,NLT)=0.0
        TU(2,NLT)=0.5
        TU(3,NLT)=0.5
        IF(GLAT.EQ.'A') GOTO 660
        ENDIF
      IF(GLAT.EQ.'B'.OR.GLAT.EQ.'F') THEN
        NLT=NLT+1
        TU(1,NLT)=0.5
        TU(2,NLT)=0.0
        TU(3,NLT)=0.5
        IF(GLAT.EQ.'B') GOTO 660
        ENDIF
      IF(GLAT.EQ.'C'.OR.GLAT.EQ.'F') THEN
        NLT=NLT+1
        TU(1,NLT)=0.5
        TU(2,NLT)=0.5
        TU(3,NLT)=0.0
        GOTO 660
        ENDIF
      IF(GLAT.EQ.'H') THEN
        NLT=NLT+1
        TU(1,NLT)=0.3333333
        TU(2,NLT)=0.6666667
        TU(3,NLT)=0.6666667
        NLT=NLT+1
        TU(1,NLT)=0.6666667
        TU(2,NLT)=0.3333333
        TU(3,NLT)=0.3333333
        ENDIF
  660 IF(NDEBUG.NE.0) WRITE(NDEBUG,2020) ((TU(I1,I2),I1=1,3),I2=1,NLT)
 2020 FORMAT(/,' ++DEBUG++SGHALL++2020++  TU(3,NLT)',4(/,26X,3F5.2))
      GOTO 66
*
* IF CHARACTER IS A NEGATIVE SIGN, SET ISGN TO NEGATIVE,
* OTHERWISE POSITIVE. THE LATER NEGATIVE SIGNS INDICATE
* INVERSIONS.
*
 1000 IF (M.NE.20) GO TO 999
      ISGN=-12
      GO TO 66
*
* ROTATION MUST FOLLOW LATTICE TYPE.
*
*
* IF ROTATION AXIS ALREADY SET GO TO 1003.
*
  999 IF (NROT .NE. 0) GO TO 1003
      IF (M .LE.6) GO TO 2011 
      WRITE(NOUT,2010) GCHR(M)
 2010 FORMAT(' ++ERROR++SGHALL++2010++ ILLEGAL ROTATION CHARACTER '
     1 ,A1)
      GO TO 150
 2011 CONTINUE
*
* SET ROTATION AXIS IN NROT.
*
      IF ((M .EQ. 1) .AND. (ISGN .LT.0)) NSYM=NSYM*2
      NROT=M
      IROT=M
      IF ((M.NE.1).OR.(ISGN.LT.0)) NGEN=NGEN+1
      IF(NGEN.LE.MSYM) GO TO 1005
      WRITE(NOUT,1501)
 1501 FORMAT(' ++ERROR++SGHALL++1501++ NGEN GREATER THAN MSYM.')
      NSYM=0
      RETURN
*
*********** SELECT AXIS EXPLICILY **************
*
* IF CHARACTER DOES NOT SET AXIS GO TO 1005.
*
 1003 IF (M .LE. 20) GO TO 1009
      IAXS=M-20
      GO TO 1004
*
* SET AXIS IMPLICITLY.
*
*
*  IF DEFAULT AXIS NOT REQUIRED GO TO 1004.
*
 1005 CONTINUE
*
*  FIRST AXIS ASSUMED Z.
*
      IF ((NGEN .NE.2).AND.(IROT .NE.1)) GO TO 1006
      IAXS=3
      GO TO 1004
*
* SECOND AXIS ASSUMED X IF NROT=2 AND PREVIOUS AXIS WAS 2 OR 4.
*
 1006 IF ((NROT .NE.2) .OR. (JROT .NE.2)) GO TO 1007
      IAXS=1
      GO TO 1004
*
* AXIS AFTER 3,6 ASSUMED AS X-Y.
*
 1007 IF (( NROT .NE.2) .OR.(JROT .NE.3))  GO TO 1008
      IAXS=5
      GO TO 1004
 1008 IF (NROT .EQ.3) GO TO 2500
      WRITE(NOUT,1639)
 1639 FORMAT(' ++ERROR++SGHALL++1639++ UNABLE TO ASSIGN IMPLICIT ',
     1 'AXIS.')
      GO TO 150
 2500 IAXS=4
*
*
* FOR NON-PRINCIPAL AXES.
*
 1004 IF (IAXS .LE.3) GO TO 66
      IROT=IAXS+3
      IAXS=JAXS
      IF((IAXS.EQ.0).AND.(IROT.EQ.7))IAXS=1
      GO TO 66
*
********** SPECIFY TRANSLATION ***********
*
*
* ONLY FOR X,Y,Z DIRECTIONS.
*
 1009 IF(M .GT. 6) GO TO 1010 
      IF (IROT .GT.M) GO TO 2510
      WRITE(NOUT,1660) GCHR(M)
 1660 FORMAT(' ++ERROR++SGHALL++1660++ ILLEGAL SCREW AXIS SYMBOL ',
     1 A1)
      GO TO 150
 2510 CONTINUE
*
* CALCULATE TRANSLATIONS IN 12 THS.
*
      SVEC(IAXS)=SVEC(IAXS)+M*12/NROT
*
* TEST FOR A,B,C,N,U,V,W,D.
*
 1010 IF(((M.LE.8).OR.(M.GE.12)).AND.((M.LE.13).OR.(M.GE.19)))GOTO1011
      IF (M .GE. 12) GO TO 1111
      I=M-8
      GO TO 1112
 1111 I=M-10
 1112 DO 7 J=1,3
      SVEC(J)=SVEC(J)+VEC(J,I)
    7 CONTINUE
      GO TO 66
*
* CONSTRUCT GENERATOR WHEN BLANK IS REACHED.
*
 1011 IF (M .NE.19) GO TO 66
*
* POINT TO ROTATION MATIX.
*
      K=TAB(IAXS,IROT)
      DO 8 I=1,12
      SMAT(I,NGEN)=ROT(I,K)*ISGN
    8 CONTINUE
      DO 9 I=13,15
*
* DEPOSIT TRANSLATON BASE 12. 
*
      SMAT(I,NGEN)=MOD(SVEC(I-12)+120,12)
    9 CONTINUE
*
* CALCULATE THE NUMBER OF EQUIVALENT POSITIONS.
*
      NSYM=NSYM*NROT
      IF (NROT .GT. 3) NROT=NROT/2
*
* RESET ORDER FLAGS AND AXIS FLAGS.
*
      JROT=NROT
      NROT=0
      IROT=0
      JAXS=IAXS
      IAXS=0
      ISGN=12
*
* ZERO VECTOR ACCUMULATOR.
*
      DO 10 I=1,3
      SVEC(I)=0
   10 CONTINUE
*
      IF(HALL(N:N+1).EQ.'  ') GOTO 666
      GO TO 66
C
C   READ ORIGIN SHIFT
C
   40 IAX=0
      NN=N
   41 ISIGN=1
      IAX=IAX+1
      IF(IAX.GT.3) GO TO 666
   42 NN=NN+1
      DO 45 M=1,29
   45 IF(HALL(NN:NN).EQ.GCHR(M)) GO TO 47
      WRITE(NOUT,1670) HALL(NN:NN),NN
      GO TO 150
   47 IF(M.EQ.20) THEN
        ISIGN=-1
        GO TO 42
        ENDIF
      IF(M.EQ.28) THEN
        ISHIFT(IAX)=0
        GO TO 41
        ENDIF
      IF(M.LE.6) THEN
        ISHIFT(IAX)=ISIGN*M
        GO TO 41
        ENDIF
      IF(M.EQ.29) GO TO 666
      WRITE(NOUT,200) HALL(NN:NN),NN
      NSYM=0
      RETURN
   66 CONTINUE
  666 CONTINUE
      IF(NDEBUG.NE.0) THEN
        WRITE(NDEBUG,1666) GLAT,NROT,ISHIFT
        DO 667 I=1,NGEN
          WRITE(NDEBUG,1667) (SMAT(J,I),J=1,16)
  667     CONTINUE
 1666   FORMAT(' ++DEBUG++SGHALL++1666++ LATCODE=',A1,' NROT=',I3,
     1   ' ISHIFT=',3I4)
 1667   FORMAT(10X,16I4)
        ENDIF
*
* END OF INPUT DATA (HALL SPACE GROUP NAME) EVALUATION LOOPS.
*
* CHECK FOR ERROR CONDITION.
*
      IF ((NROT .LE.0).AND.(LLTCODE .NE.0)) GO TO 151
  150 CONTINUE
*
* ERROR CONDITION.
*
      WRITE(NOUT,200) HALL(N:N),N,NROT,LLTCODE,GLAT,GCENT
  200 FORMAT (' ++DEBUG++SGHALL++200++  ILLEGAL CONSTRUCTION AT',
     1       ' CHARACTER ',A1,' IN COL.',I3,/,25X,'NROT =',I3,
     1       ', LLTCODE =',I3,', GLAT = ',A1,', GCENT = ',A1,/)
      NSYM=0
      RETURN
  151 CONTINUE
*
*
*GENERATE MATRICES BY PROGRESSIVE BINARY PRODUCTS.
*
*
*
* SKIP GENERATOR IF P1 OR -P1.
*
      IF (NGEN .LE.1) GO TO 117
      M=2 
  723 IF(M.LE.NGEN) GO TO 273 
      GO TO 105
*
* SAVE CURRENT MATRIX COUNT.
*
  273 OLD=NGEN
      N=M 
  724 IF(N.LE.NGEN) GO TO 274 
      GO TO 11
*
* POINT TO GENERATED MATRIX IN SMAT.
*
  274 NEW=NGEN+1
*
      DO 13 I=1,3
      DO 131 J=1,13,4
  131 SMAT(I+J-1,NEW)=((SMAT(I,M)*SMAT(J,N))+(SMAT(I+4,M)*SMAT(J+1,N))+ 
     1   (SMAT(I+8,M)*SMAT(J+2,N))+(SMAT(I+12,M)*SMAT(J+3,N)))/12
      SMAT(I+12,NEW)=MOD(SMAT(I+12,NEW)+120,12)
   13 CONTINUE
*
* IS MATRIX UNIQUE? 
*
      LFLAG=1
      DO 14 L=1,NGEN
      DO 103 I=1,12 
  103 IF (SMAT(I,L).NE.SMAT(I,NEW)) GO TO 1674
      IF (LLTCODE .NE. 1) GO TO 251
      DO 15 II=13,16
      IF (SMAT(II,L) .EQ. SMAT(II,NEW)) GO TO 15
*
* ERROR CONDITION FOUND.
*
      WRITE (NOUT,300)
  300 FORMAT('++SGHALL++ERROR: MATRICES RELATED BY PURE TRANSLATION.')
      NSYM=0
      RETURN
   15 CONTINUE
*
      GO TO 251
*
 1674 DO 16 J=1,12
   16 IF (SMAT(J,L) .NE. SMAT(J,NEW)) GO TO 20
*
* ERROR CONDITION NOT AVOIDED ( IE NOT SKIPPED OVER).
*
      WRITE(NOUT,400)
  400 FORMAT('++SGHALL++ERROR: SOME MATRICES RELATED BY INVERSION.')
      NSYM=0
      RETURN
*
   20 CONTINUE
      LFLAG=LFLAG+1 
   14 CONTINUE
  251 IF (LFLAG .GT. NGEN) NGEN=NEW
      N=N+1
      GO TO 724
   11 IF ((NGEN.EQ.OLD).AND.(NGEN.GE. NSYM)) GO TO 105
      M=M+1
      GO TO 723
  105 CONTINUE
*
*
* SORT ON ZERO PATTERN.
*
      DO 17 N=2,NGEN
      DO 18 L=N,NGEN
      DO 181 K=1,12 
*
* EXIST ON ZERO MISMATCH.
*
  181 IF((SMAT(K,N-1).EQ.0).AND.(SMAT(K,L).NE.0))GO TO 18
      GO TO 1171
   18 CONTINUE
      GO TO 17
*
*
* EXCHANGE SORTED ELEMENTS.
*
 1171 DO 19 K=1,16
      I=SMAT(K,L)
      SMAT(K,L)=SMAT(K,N)
      SMAT(K,N)=I
   19 CONTINUE
*
   17 CONTINUE
  117 CONTINUE
*
* ERROR CHECK OF NGEN RELATIVE TO NSYM. 
*
      IF (NGEN .LE. NSYM) GO TO 1013
      WRITE(NOUT,500) NGEN,NSYM
  500  FORMAT('++SGHALL++ERROR: ',I5,' MATRICES GENERATED,',I5,
     1       ' EXPECTED')
      NSYM=0
*
* THE FOLLOWING LOOPS ENTERS THE VALUES OF SMAT(16,48),
* WHICH IS THE ARRAY USED IN THIS ROUTINE TO STORE THE
* ROTATION AND TRANSLATION MATRICES, INTO TWO COMPARABLE
* ARRAYS S(3,3,48) AND T(3,48) WHICH ARE USED IN OTHER
* ROUTIES EXTERNAL TO SGHALL. 
*
 1013 DO 7775 M=1,48
      DO 7775 J=1,3 
      DO 7775 I=1,3 
      K=4*(J-1)+I
*
* REAL DIVISION BY ZERO IS NEEDED TO CONVERT THE 12 THS
* USED IN SGHALL INTO ROTATIONS AND TRANSLATIONS
* UNDERSTANDABLE TO ROUTINE WHICH USE S(3,3,48) AND T(3,48).
* THE VALUES IN S AND T ARE NOW 1 OR DECIMAL FRACTIONS.
*
      S(I,J,M)=SMAT(K,M)/12.0 
      T(I,M)=SMAT(I+12,M)/12.0
 7775 CONTINUE
C
C  ADD ORIGIN SHIFT 
C
      DO 7780 M=1,48
        DO 7777 I=1,3
          IF(S(I,I,M).LT.-0.001) THEN
            T(I,M)=T(I,M)+ISHIFT(I)/6.0 
            IF(T(I,M).GT.0.99) T(I,M)=T(I,M)-1.0
            IF(T(I,M).LT.-0.01) T(I,M)=T(I,M)+1.0 
            ENDIF
 7777     CONTINUE
 7780   CONTINUE
      IF(GCENT.EQ.GCHR(11)) CALL SYMD(S,T,NSYM,MSYM)
      CALL SYMXYZ (NSYM,S,T,CSYM,NCSYM)
      IF(NDEBUG.NE.0) THEN
        NOUT1=NOUT
        NOUT=NDEBUG
        CALL SYMXYZ (NSYM,S,T,CSYM,NCSYM)
        NOUT=NOUT1
        ENDIF
      RETURN
      END 
C
C----------------------------------------------------------------------C
C
      SUBROUTINE SYMD(S,T,NSYM,MSYM)
      DIMENSION S(3,3,MSYM),T(3,MSYM)
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP
      DO 5 I=1,NSYM 
      IF(NSYM+I.LE.MSYM) GOTO 10
      WRITE(NOUT,100)
  100 FORMAT(40H ERROR IN SUBROUTINE:SYMD.MORE SYM OP.S
     1   ,40HCALCULATED THAN ALLOWED(48).                             )
      RETURN
   10 DO 15 J=1,3
      DO 25 K=1,3
   25 S(J,K,NSYM+I)=-S(J,K,I) 
   15 T(J,NSYM+I)=T(J,I)
    5 CONTINUE
      NSYM=2*NSYM
      RETURN
C**  THIS PROGRAM VALID ON FTN4 AND FTN5 **
      END 
C
C----------------------------------------------------------------------C
C
      SUBROUTINE SYMXYZ(NSYM,S,T,CSYM,NCSYM)
      IMPLICIT  CHARACTER*8 (G)
      IMPLICIT  CHARACTER*80 (H)
      CHARACTER BUF,BUF1
      CHARACTER CSYM(49)*40
      INTEGER NCSYM
*
      CHARACTER TRANS(30),INUM(8),CHR(26)
      INTEGER   SMAT(16,48)
      CHARACTER BLNK,COMA,OVER,PLUS,SUBT
      DIMENSION S(3,3,NSYM), T(3,NSYM)
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp
      common /gsym/   glat,gcent,hmsym,hall,gsysp(2,30)
      DATA BLNK,COMA,PLUS,SUBT,OVER/' ' ,',','+','-','/'/
      DATA INUM/'1','2','3','5','3','4','6','0'/
      DATA CHR/'1','2','3','4','5','6','P','R','A','B','C','I','F',     
     1  'N','U','V','W','D',' ','-','X','Y','Z','*','''','"'/
*
* IN THIS ROUTINE NGEN IS EQUAL TO NSYM.
*
      IF (NSYM .LE. 0) RETURN
*
* EXTERNALLY THE ROTATIONS AND TRANSLATIONS ARE STORED AND
* MANIPULATED IN ARRAYS S(3,3,48) AND T(3,48) RESPECTIVELY. 
* IN SYMXYZ ( THIS ROUTINE ) THE ARRAY SMAT(16,48) SERVES
* AS STORAGE FOR BOTH THE ROTATIONS AND TRANSLATIONS.
*
      DO 7775 M=1,NSYM
         DO 7775 J=1,3 
           DO 7775 I=1,3 
              K=4*(J-1)+I
              SMAT(K,M)=INTEGER((S(I,J,M)*12),0)
*
* MULTIPLICATION BY 12 IS NECESSARY TO CHANGE THE DECIMAL
* FRACTIONS OF S(3,3,48) AND T(3,48) INTO THE 12 THS
* WHICH THIS ROUTINE REQUIRES.
* 12 THS ARE USED BECAUSE THE TRANSLATION FRACTIONS
* (1/4, 1/3, 3/4, 1/2, 1/6, 5/6 ETC) ALL ARE NICE, IE EVEN
* MULTIPLIES OF 12. 
*
              SMAT(I+12,M)=INTEGER((T(I,M)*12),0)
 7775 CONTINUE
*
* PRINT HALL SPACE GROUP SYMBOL.
*
c      WRITE(NOUT,100)   GLAT,GCENT
c  100 FORMAT('LATT  ',A,A)
*
c      WRITE(NOUT,105)
c  105 FORMAT('    ')
*
* WRITE SHELX STYLE "LATT" AND "SYMM" CARD IMAGES
*
      NCSYM = 0
      N_SYM=NSYM
      IF(GCENT .EQ. 'C') N_SYM=N_SYM/2
      WRITE(BUF,'(I1)') INDEX('PIHFABC',GLAT(1:1))
      BUF1=' '
      IF(GCENT .NE. 'C') BUF1='-'
      NCSYM=NCSYM+1
      WRITE(CSYM(NCSYM),'(''LATT'',3X,A,A)') BUF1,BUF
      IF(N_SYM .LT. 2) RETURN

      DO 20 M=2,N_SYM
*
* PRESET ARRAY TRANS TO BLANKS.
*
      DO 21 I=1,30
      TRANS(I)=BLNK 
   21 CONTINUE
      N=1 
*
* EXTRACT TRANSLATIONAL ELEMENTS.
*
      DO 22 L=1,3
      J=SMAT(L+12,M)
C
C SPECIAL POSITION WITH COORDINATE(S) '0'
C
      NSUM=IABS(SMAT(4*L-3,M))+IABS(SMAT(4*L-2,M))+IABS(SMAT(4*L-1,M))
     1     +IABS(J) 
      IF(NSUM.EQ.0) THEN
         TRANS(N)=INUM(8)
         N=N+1
         ENDIF
      IF (J.LE.0) GO TO 1017
*
* SET THE NUMERATOR AS ONE (1).
*
      IF (J.GE.8) GO TO 1014
      TRANS(N)=INUM(1)
      GO TO 1015
*
* SET NUMERATOR AS 2, 3 OR 5. 
*
 1014 TRANS(N)=INUM(J-6)
*
* INSERT SLASH INTO FRACTION. 
*
 1015 TRANS(N+1)=OVER
*
* POINT TO DENOMINATOR CODE.
*
      I=IABS(6-J)
      IF (J.NE.6) GO TO 1016
*
* SET DENOMINATOR AS TWO (2)
*
      TRANS(N+2)=INUM(2)
      GO TO 1020
*
* SET DENOMINTATOR AS 3, 4 OR 6
 1016 TRANS(N+2)=INUM(I+3)
 1020 N=N+3
 1017 DO 23 K=1,3
      J=L+(K-1)*4
      J=SMAT(J,M)
      IF (J.EQ.0) GO TO 23
      IF (J.LE.0) GO TO 1018
*
* INSERT PLUS SIGN. 
*
      TRANS(N)=PLUS 
      GO TO 1019
*
* INSERT NEGATIVE SIGN.
*
 1018 TRANS(N)=SUBT 
*
* INSERT X, Y, OR Z.
*
 1019 TRANS(N+1)=CHR(K+20)
      N=N+2
   23 CONTINUE
*
* SEPARATE WITH COMMA.
*
      TRANS(N)=COMA 
      N=N+1
   22 CONTINUE
*
* REMOVE THE LAST COMMA FROM THE OUTPUT.
*
      TRANS(N-1)=BLNK
*
* LIST SYMMETRY AS X,Y,Z.
*
      NCSYM=NCSYM+1
      WRITE(CSYM(NCSYM),700)TRANS

  700 FORMAT('SYMM',3X,30A1)
   20 CONTINUE
      END 
C
C----------------------------------------------------------------------C
C
      SUBROUTINE GRMULT(IER)
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp
      common /symnew/    nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      common /syma/   mult(48,48),it(3,48),nsp,msp,isptab(48,30),
     1                multsp(30),sp(3,3,30),tp(3,30)
C
      DIMENSION SS(3,3)
C
      IER=0
      DO 200 I=1,NSYM
        DO 150 J=1,NSYM
          CALL MXM(SS,S(1,1,J),S(1,1,I),3)
          DO 100 K=1,NSYM
            IF(ICOMMA(S(1,1,K),SS,3,2).EQ.0) THEN 
              MULT(I,J)=K
              GO TO 150
              ENDIF 
  100       CONTINUE
          WRITE(NOUT,1000) I,J
 1000     FORMAT(' ++ERROR++GRMULT++1000++ NO MATCH FOR PRODUCT OF'
     +     ,' SYMMETRY OPERATORS ',2I3) 
          IER=1
          RETURN
  150    CONTINUE
  200 CONTINUE
      END 
C
C----------------------------------------------------------------------C
C
      SUBROUTINE SYMOP(Y,S,T,X)
      DIMENSION X(3),Y(3),S(3,3),T(3)
      DO 50 I=1,3
        Y(I)=0.
        DO 20 J=1,3 
  20      Y(I)=Y(I)+S(I,J)*X(J)
        Y(I)=Y(I)+T(I)
   50   CONTINUE
      RETURN
      END 
C
C----------------------------------------------------------------------C
C
      SUBROUTINE SYMULT(S3,T3,S2,T2,S1,T1)
      DIMENSION S1(3,3),S2(3,3),S3(3,3),T1(3),T2(3),T3(3),T4(3)
      CALL MXM(S3,S2,S1,3)
      CALL MXV(T4,S2,T1)
      CALL VPLUSV(T3,T4,T2)
      RETURN
      END 
C
C----------------------------------------------------------------------C
C
      subroutine conxyz(sp,tp,hbuf)
      implicit character*80 (h)
      character*8 g1(12),g2(12),g3(12),g4(12)
c
      dimension sp(3,3),tp(3)
      data g1/'1/8','1/4','3/8','1/2','5/8','3/4','7/8','1',4*'   '/
      data g2/'1/12','1/6','1/4','1/3','5/12','1/2','7/12','2/3',
     1        '3/4','5/6','11/12','1'/
      data g3/'9/8','5/4','11/8','3/2','13/8','7/4','15/8','2',4*' '/
      data g4/'13/12','7/6','5/4','4/3','17/12','3/2','19/12','5/3',
     1        '7/4','11/6','23/12','2'/
c
      hline=' '
      do 10 i=1,3
        j=(i-1)*17+1
        if((abs(sp(i,1))+abs(sp(i,2))+abs(sp(i,3))+abs(tp(i))).lt.
     1    0.01) then
c   --    i-th coordinate 0
          hline(j:j)='0'
          if(i.ne.3) hline(j+1:j+1)=','
          goto 10
          endif
        if(integer(sp(i,1),0).eq.0) goto 20
c   --    X-component
          hline(j+1:j+1)='X'
          if(sp(i,1).gt.1.01) hline(j:j)='2'
          if(sp(i,1).lt.-0.1) hline(j:j)='-'
20      if(integer(sp(i,2),0).eq.0) goto 30
c   --    Y-component
          hline(j+4:j+4)='Y'
          if(sp(i,2).gt.1.01) hline(j+3:j+3)='2'
          if(sp(i,2).lt.-0.1) hline(j+2:j+2)='-'
          if(hline(j+1:j+2).eq.'X ') hline(j+2:j+2)='+'
30      if(integer(sp(i,3),0).eq.0) goto 40
c   --    Z-component
          hline(j+7:j+7)='Z'
          if(sp(i,3).gt.1.01) hline(j+6:j+6)='2'
          if(sp(i,3).lt.-0.1) hline(j+5:j+5)='-'
          if(hline(j+5:j+5).eq.' '.and.hline(j:j+4).ne.'     ')
     1      hline(j+5:j+5)='+'
40      if(integer(tp(i),3).eq.0) goto 50
c   --    translation
          if(hline(j:j+7).ne.'       ') hline(j+8:j+8)='+'
          if(tp(i).lt.-0.1) hline(j+8:j+8)='-'
          do 45 k=1,12
            xk=k
            k1=k*125
            k2=integer((xk*83.33),0)
            if(iabs(integer(tp(i),3)).eq.k1) 
     1        hline(j+9:j+14)=g1(k)(1:6)
            if(iabs(integer(tp(i),3)).eq.(k1+1000))
     1        hline(j+9:j+14)=g3(k)(1:6)
            if(iabs(integer(tp(i),3)).eq.k2) 
     1        hline(j+9:j+14)=g2(k)(1:6)
            if(iabs(integer(tp(i),3)).eq.(k2+1000))
     1        hline(j+9:j+14)=g4(k)(1:6)
45          continue
50        if(i.ne.3) hline(j+15:j+15)=','
10      continue
c
c     remove blanks, store in hbuf
c
      hbuf=' '
      j=1
      do 100 i=1,80
        if(hline(i:i).eq.' ') goto 100
        hbuf(j:j)=hline(i:i)
        j=j+1
100     continue
      return
      end
C
C----------------------------------------------------------------------C
C
      SUBROUTINE MMINM(C,A,B,N)
      DIMENSION A(N,N),B(N,N),C(N,N)
      DO 50 I=1,N
        DO 50 J=1,N 
   50     C(I,J)=A(I,J)-B(I,J)
      RETURN
      END 
C
C----------------------------------------------------------------------C
C
      SUBROUTINE MXM (C,A,B,N)
      DIMENSION A(N,N),B(N,N),C(N,N)
      DO 10 I=1,N
      DO 20 K=1,N
      C(I,K)=0
      DO 30 J=1,N
      C(I,K)=C(I,K)+A(I,J)*B(J,K)
   30 CONTINUE
   20 CONTINUE
   10 CONTINUE
      RETURN
      END 
C
C----------------------------------------------------------------------C
C
      SUBROUTINE MXV(VPROD,SM,V)
      DIMENSION VPROD(3),SM(3,3),V(3)
      DO 50 I=1,3
         VPROD(I) = 0
         DO 50 J=1,3
   50       VPROD(I) = VPROD(I) + SM(I,J)*V(J)
      RETURN
      END 
C
C----------------------------------------------------------------------C
C
      FUNCTION ICOMMA (A,B,N,ID)
      DIMENSION A(N,N),B(N,N) 
      ICOMMA=0
      DO 10 I=1,N
      DO 20 J=1,N
        IF(INTEGER(A(I,J),ID).NE.INTEGER(B(I,J),ID)) THEN
          ICOMMA=1 
          RETURN
          ENDIF
   20 CONTINUE
   10 CONTINUE
      END 
C
C----------------------------------------------------------------------C
C
      FUNCTION ICOMVE (X,Y,N,ID)
      DIMENSION X(N),Y(N)
      ICOMVE=0
      DO 100 I=1,N
  100   IF(INTEGER(X(I),ID).NE.INTEGER(Y(I),ID)) ICOMVE=1
      RETURN
      END 
C
C----------------------------------------------------------------------C
C
      FUNCTION INTEGER(X,N)
      Y = 1
      IF(N.LT.1) GO TO 20
      DO 10 I=1,N
   10    Y = Y*10.
   20 IF(X.LT.0.) GO TO 30
      INTEGER = X*Y + 0.5
      RETURN
   30 INTEGER = X*Y - 0.5
      RETURN
      END 
C
C----------------------------------------------------------------------C
C
      SUBROUTINE VMINV(Z,X,Y)
      DIMENSION X(3),Y(3),Z(3)
      DO 10 I=1,3
   10   Z(I) = X(I) - Y(I)
      RETURN
      END 
C
C----------------------------------------------------------------------C
C
      SUBROUTINE VPLUSV(VPROD,V1,V2)
      DIMENSION VPROD(3),V1(3),V2(3)
      DO 50 I=1,3
   50    VPROD(I) = V1(I) + V2(I)
      RETURN
      END 
C
C----------------------------------------------------------------------C
C
      SUBROUTINE REDEFI(SPS,TPS)
      DIMENSION SPS(3,3),TPS(3),AROW(3)
      DO 100 I=1,3
        AROW(I)=ABS(SPS(I,1))+ABS(SPS(I,2))+ABS(SPS(I,3))
        IF(AROW(I).LT.0.01) GOTO 100
        K1=I-1
        K2=I+1
        IF(K1.LT.1) K1=3
        IF(K2.GT.3) K2=1
        IF(ABS(SPS(I,I)).GT.0.01.AND.(ABS(SPS(I,K1))+ABS(SPS(I,K2)))
     1    .LT.0.01) THEN
          TPS(K1)=TPS(K1)-SPS(K1,I)*TPS(I)/SPS(I,I)
          TPS(K2)=TPS(K2)-SPS(K2,I)*TPS(I)/SPS(I,I)
          TPS(I)=0.0
          IF(SPS(I,I).LT.-.01) THEN
            SPS(I,I)=ABS(SPS(I,I))
            SPS(K1,I)=-SPS(K1,I)
            SPS(K2,I)=-SPS(K2,I)
            ENDIF
          ENDIF
  100   CONTINUE
      I30 = 3
      CALL RECENT(TPS,I30)
      RETURN
      END
C
C----------------------------------------------------------------------C
C
      SUBROUTINE NEWAXS(SPS,TPS)
      DIMENSION SPS(3,3),TPS(3),NEWFL(3)
      DO 5 I=1,3
    5   NEWFL(I)=0
C
      DO 10 I=1,3
        XK=ABS(SPS(I,1))+ABS(SPS(I,2))+ABS(SPS(I,3))
        IK=INTEGER(XK,0)
        IF(IK.NE.1.OR.ABS(SPS(I,I)).GT.0.1) GOTO 10
          K1=I-1
          K2=I+1
          IF(K1.LT.1) K1=3
          IF(K2.GT.3) K2=1
          IF(ABS(SPS(I,K1)).GT.0.1.AND.ABS(SPS(K2,K1)).LT.0.1) THEN
            IF(NEWFL(K1).EQ.0.AND.ABS(SPS(K1,K1)).GT.0.1) GOTO 10
            NEWFL(I)=1
            SPS(I,I)=ABS(SPS(I,K1))
            SPS(I,K1)=0.0
            TPS(I)=0.0
            GOTO 10
          ELSE IF(ABS(SPS(I,K2)).GT.0.1.AND.ABS(SPS(K1,K2)).LT.0.1) THEN
            IF(NEWFL(K2).EQ.0.AND.ABS(SPS(K2,K2)).GT.0.1) GOTO 10
            NEWFL(I)=1
            SPS(I,I)=ABS(SPS(I,K2))
            SPS(I,K2)=0.0
            TPS(I)=0.0
            ENDIF
   10   CONTINUE
      RETURN
      END
C
C----------------------------------------------------------------------C
C
      SUBROUTINE TEHECU(SP,SPS)
      DIMENSION SP(3,3),SPS(3,3),KS(3),KP(3)
      DO 10 I=1,3
        KS(I)=INTEGER(SP(I,I),0)
   10   KP(I)=INTEGER(SPS(I,I),0)
C
      DO 20 I=1,3
        IF(KS(I).NE.0.AND.KP(I).EQ.0) THEN
          I1=I+1
          DO 30 J=I1,3
            IF(ABS(SPS(I,J)).GT.0.1.AND.ABS(SPS(J,I)).GT.0.1) THEN
              SPS(I,I)=SPS(I,J)
              SPS(I,J)=0.0
              SPS(J,J)=SPS(J,I)
              SPS(J,I)=0.0
              ENDIF
   30       CONTINUE
          ENDIF
   20   CONTINUE
      RETURN
      END
C
C----------------------------------------------------------------------C
C
      SUBROUTINE SHCTOR(TP,SP,TU,NLT)
      DIMENSION TP(3),SP(3,3),T1(3),T2(3),TU(3,NLT)
      DO 10 I=1,3
   10 T2(I)=TP(I)
      DO 20 I=9,NLT
        DO 30 K=1,3
   30     T1(K)=TU(K,I)+TP(K)
        I30 = 3
        CALL RECENT(T1,I30)
        CALL REDEFI(SP,T1)
        SUMP=TP(1)+TP(2)+TP(3)
        SUM1=T1(1)+T1(2)+T1(3)
        SUM2=T2(1)+T2(2)+T2(3)
        IF(SUM1.LT.(SUMP-0.01).AND.SUM1.LT.(SUM2-0.01)) GOTO 25
        IF(SUMP.LT.(SUM1-0.01).OR.SUM2.LT.(SUM1-0.01)) GOTO 20
        SUMP=SUMP-TP(3)
        SUM1=SUM1-T1(3)
        SUM2=SUM2-T2(3)
        IF(SUM1.LT.(SUMP-0.01).AND.SUM1.LT.(SUM2-0.01)) GOTO 25
        IF(SUMP.LT.(SUM1-0.01).OR.SUM2.LT.(SUM1-0.01)) GOTO 20
        IF(TP(1).LT.(T1(1)-0.01).OR.T2(1).LT.(T1(1)-0.01)) GOTO 20
   25   DO 40 L=1,3
   40     T2(L)=T1(L)
   20   CONTINUE
      DO 50 I=1,3
   50   TP(I)=T2(I)
      RETURN
      END
C
C----------------------------------------------------------------------C
C
      FUNCTION ITESTR(TA,TB,TU,NLT,SP)
      DIMENSION TA(3),TB(3),TC(3),TU(3,NLT),SP(3,3)
      ITESTR=1
      DO 200 I=8,NLT
        DO 100 J=1,3
  100     TC(J)=TB(J)+TU(J,I)
        CALL REDEFI(SP,TC)
        IF(ICOMVE(TA,TC,3,2).EQ.0) THEN
          ITESTR=0
          RETURN
          ENDIF
  200   CONTINUE
      RETURN
      END
C
C----------------------------------------------------------------------C
C
      SUBROUTINE RECENT(X,N)
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp
      DIMENSION X(N)
      DO 100 K=1,N
   10   IF(X(K).LE.1.0.AND.X(K).GE.0.0) GOTO 50
        IF(X(K).GE.1.) X(K)=X(K)-1.0
        IF(X(K).LT.0.) X(K)=X(K)+1.0
        GOTO 10
   50   IF(X(K).GT.0.9999) X(K)=0.0
  100   CONTINUE
      RETURN
      END
C
C----------------------------------------------------------------------C
C
      SUBROUTINE LTRANS(NLT,T,TU)
      DIMENSION T(3),TU(3,14)
      J=0
      NLT=NLT+1
      DO 50 I=1,3
        TU(I,NLT)=T(I)
        IF(T(I).LT.0.1) J=I
   50   CONTINUE
      IF(J.EQ.0) RETURN
C
C   IF ONE COMPONENT OF T IS ZERO ADD A UNIT TRANSLATION
C
      NLT=NLT+1
      DO 100 I=1,3
        TU(I,NLT)=T(I)
        IF(I.EQ.J) TU(I,NLT)=TU(I,NLT)+1.
  100   CONTINUE
      RETURN
      END
C
C
C
C--------------- IMPORTANT !!!  Comment out the following routine if
C                compiling for SUN OS 4/5 - leave alone for all other
C                operating systems
C                
      integer function ieee_flags(a1,a2,a3,a4)
      character*(*) a1,a2,a3,a4
      ieee_flags=0
      end
C
C--------------- IMPORTANT !!!  Remove comments (or provide a replacement
C                routine) if compiling for non-UNIX systems
C                

C      subroutine getenv(var1,var2)
CC --- returns value of environment variable "var1" in string "var2"
C      character*(*) var1,var2
C      var2=' '
C      end



      include 'ciftbx.f'
